/*
*  Name:      deterministic_finite_automaton.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Deterministic finite automaton
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_AUTOMATON_DETERMINISTIC_FINITE_AUTOMATON__
#define _MPCL_AUTOMATON_DETERMINISTIC_FINITE_AUTOMATON__

#include <set>
#include "defs.hh"
#include "../util/collection/map.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for AUTOMATON.
  namespace automaton
  {

    using mpcl::util::collection::TMap;
    using std::pair;
    using std::set;

    /**
    *  Deterministic Finite Automaton (DFA) class.
    */
    template <typename TState, typename TEvent>
    class TDeterministicFiniteAutomaton
    {

      protected:

        ///  Pairs state with input event type.
        typedef pair<TState, TEvent>   TPair;

        ///  Map with state with input event to state transitions type.
        typedef TMap<TPair, TState>   TTransitionMap;

        ///  Set of final states type.
        typedef set<TState>   TStateSet;

        ///  Current state (initially it musts contain initial state).
        TState   tCurrentState;

        ///  Set of final states.
        TStateSet   tFinalStateSet;

        ///  Initial state.
        TState   tInitialState;

        ///  Map with state with input event to state transitions.
        TTransitionMap   tTransitionMap;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TDeterministicFiniteAutomaton (void) :
          tCurrentState  ()                  ,
          tFinalStateSet ()                  ,
          tInitialState  ()                  ,
          tTransitionMap ()                  {}

        ///  Destroys the instance.
        virtual ~TDeterministicFiniteAutomaton (void) {}

        /**
        *  Move to the state \a ktSTATE.
        *  @param rktSTATE Next state.
        */
        virtual void move (const TState& rktSTATE)
        {
          tCurrentState = rktSTATE;
        }


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Returns current state.
        *  @return Current state.
        */
        TState current (void) const
        {
          return tCurrentState;
        }

        /**
        *  Returns initial state.
        *  @return Initial state.
        */
        TState initial (void) const
        {
          return tInitialState;
        }

        /**
        *  Returns true if \a rktSOURCE_STATE is a final state.
        *  @param rktSOURCE_STATE Source state.
        *  @return True if rktSOURCE_STATE is a final state.
        */
        virtual bool isFinal (const TState& rktSOURCE_STATE) const
        {
          return ( tFinalStateSet.find (rktSOURCE_STATE) != tFinalStateSet.end() );
        }

        /**
        *  Gets next state from current state and the input event
        *  \a rktSOURCE_EVENT.
        *  @param rktSOURCE_EVENT Source input event.
        *  @return The next state.
        *  @exception TNotFoundException If there is no next state.
        */
        const TState& next (const TEvent& rktSOURCE_EVENT) const
          throw (TNotFoundException)
        {
          typename TTransitionMap::const_iterator   I;

          I = tTransitionMap.find (std::make_pair (tCurrentState, rktSOURCE_EVENT));
          if ( I == tTransitionMap.end() )
          {
            throw TNotFoundException ("no next state", __FILE__, __LINE__);
          }
          return I->second;
        }

    };  // class TDeterministicFiniteAutomaton

  }  // namespace automaton

}  // namespace mpcl


#endif  // not _MPCL_AUTOMATON_DETERMINISTIC_FINITE_AUTOMATON__
