/*
*  Name:      threaded_event_handler.hh
*  Author:    Carlos Jimenez Moreno
*             Rafael Jesus Alcantara Perez
*  Summary:   Threaded Event Handler
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999       Carlos Jimenez Moreno <carlosj@dedaloingenieros.com>
*  Copyright (C) 2002-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/


#ifndef _MPCL_EVENT_THREADED_EVENT_HANDLER__
#define _MPCL_EVENT_THREADED_EVENT_HANDLER__

#include <mpcl/event/event_handler.hh>
#include <mpcl/system/all.hh>


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for EVENT.
  namespace event
  {

    /**
    *  Threaded event handler for TStreamableDfa.
    *  It protects from concurrent access to shared class data.
    *  @see TStreamableDfa.
    */
    template <typename TEvent>
    class TThreadedEventHandler      :
        public TEventHandler<TEvent> ,
        public mpcl::system::TThread
    {

      private:

        /**
        *  Binary mutex.  If  we do  not  want to  modify  the  declaration  of
        *  TEventHandler::isEmpty(), then it  must make this attribute mutable.
        */
        mutable mpcl::system::TMutex   tMutex;

        
      public:

        //
        // C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TThreadedEventHandler (void) :
          TEventHandler<TEvent> ()   ,
          TThread               ()   ,
          tMutex                ()   {}

        /**
        *  Extracts next event (and removes from the queue).
        *  It is mutex protected.
        *  @return Next event.
        */
        TEvent pop (void)
        {
          TEvent   tEvent;

          tMutex.lock();
          tEvent = TEventHandler<TEvent>::pop();
          tMutex.unlock();
          return tEvent;
        }

        /**
        *  Pushes an event into the queue.
        *  It is mutex protected.
        *  @param rktSOURCE_EVENT Pushed event.
        */
        void push (const TEvent& rktSOURCE_EVENT)
        {
          tMutex.lock();
          TEventHandler<TEvent>::push (rktSOURCE_EVENT);
          tMutex.unlock();
        }


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Returns true if there is no more events.
        *  It is mutex protected.
        *  @return True if there is no more events.
        */
        bool isEmpty (void) const
        {
          bool   gIsEmpty;
  
          tMutex.lock();
          gIsEmpty = TEventHandler<TEvent>::isEmpty();
          tMutex.unlock();
          return gIsEmpty;
        }

    };  // TThreadedEventHandler
    
  }  // namespace event

}  // namespace mpcl


#endif  // not _MPCL_EVENT_THREADED_EVENT_HANDLER__
