/*
*  Name:      exception.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Exception class
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_EXCEPTION__
#define _MPCL_EXCEPTION__

#include <string>
#include "io/output_streamable.hh"


///  Name-space for MPCL.
namespace mpcl
{

  using io::IOutputStreamable;
  using std::basic_ostream;
  using std::string;

  /**
  *  Exception: exception class.
  *
  *  \warning  The  order of inheritance  from  exception classes,  has special
  *            significance  due  to  the next fact:  the constructor  of  base
  *            classes  inherited of "virtual" fashion,  are called only  once.
  *
  *  \warning  It uses the class 'std::string' to remove loop dependencies.
  */
  class TException: public IOutputStreamable<>
  {

    protected:

      ///  General description of the exception.
      string   ySynopsis;

      ///  Specific description of the exception.
      string   yRationale;

      ///  File name where the exception has its meaning.
      string   yFileName;

      ///  Line-number in the former file name.
      int   iLineNumber;


    public:

      //
      //  C O N S T R U C T O R S
      //
 
      /**
      *  Builds the instance without any kind of descriptions of the
      *  exception.
      */
      TException (void)         :
        IOutputStreamable<> ()  ,
        ySynopsis           ()  ,
        yRationale          ()  ,
        yFileName           ()  ,
        iLineNumber         (0) {}

      /**
      *  Builds the instance with a general description and (if passed)
      *  a specific description.
      *  @param rkySYNOPSYS   General exception description.
      *  @param rkyRATIONALE  Specific exception description.
      *  @param rkyFILE_NAME  File name where the exception throws.
      *  @param kiLINE_NUMBER Line number from that file.
      */
      TException ( const string& rkySYNOPSYS              ,
                   const string& rkyRATIONALE  = string() ,
                   const string& rkyFILE_NAME  = string() ,
                   const int     kiLINE_NUMBER = 0        ) :
        IOutputStreamable<> ()                              ,
        ySynopsis           (rkySYNOPSYS)                   ,
        yRationale          (rkyRATIONALE)                  ,
        yFileName           (rkyFILE_NAME)                  ,
        iLineNumber         (kiLINE_NUMBER)                 {}

      /**
      *  Builds the instance as a copy of the instance
      *  \a rktSOURCE_EXCEPTION.
      *  @param rktSOURCE_EXCEPTION Source instance.
      */
      TException (const TException& rktSOURCE_EXCEPTION) :
        IOutputStreamable<> ()                                ,
        ySynopsis           (rktSOURCE_EXCEPTION.ySynopsis)   ,
        yRationale          (rktSOURCE_EXCEPTION.yRationale)  ,
        yFileName           (rktSOURCE_EXCEPTION.yFileName)   ,
        iLineNumber         (rktSOURCE_EXCEPTION.iLineNumber) {}


    public:

      //
      //  S E L E C T O R S
      //

      /**
      *  Gets a general description of the exception.
      *  @return General description of the exception.
      */
      const string& generalDescription (void) const throw()
      {
        return ySynopsis;
      }

      /**
      *  Gets a specific description of the exception.
      *  @return Specific description of the exception.
      */
      const string& specificDescription (void) const throw()
      {
        return yRationale;
      }

      /**
      *  Gets the file name where the exception was thrown.
      *  @return File name where the exception was thrown.
      */
      const string& fileName (void) const throw()
      {
        return yFileName;
      }

      /**
      *  Gets a line number of the exception.
      *  @return Specific description of the exception.
      */
      int lineNumber (void) const throw()
      {
        return iLineNumber;
      }

      /**
      *  Gets a description of the exception.
      *  @return Description of the exception.
      */
      virtual const string what (void) const;


    protected:

      //
      //  S E L E C T O R S
      //
      
      /**
      *  Put onto stream \a rtTARGET_OSTREAM a formatted string
      *  with information about this exception.
      */
      void write (basic_ostream<char_type, traits_type>& rtOUTPUT_OSTREAM) const
      {
        rtOUTPUT_OSTREAM << what() << std::endl;
      }

  };  // class TException

}  // namespace mpcl


#endif  // not _MPCL_EXCEPTION__
