/*
*  Name:      verifier.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Invariant Base verifier
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_INVARIANT_VERIFIER__
#define _MPCL_INVARIANT_VERIFIER__

#include <cstdarg>
#include "../text/string.hh"
#include "../util/collection/map.hh"
#include "../util/variable_argument_array.hh"
#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for INVARIANT.
  namespace invariant
  {

    using text::TString;
    using util::collection::TMap;
    using util::TVariableArgumentArray;

    /**
    *  Base invariant verifier.
    *  This  class  verifies  a invariant set, having each  invariant  a  unique
    *  domain  for  application (conformance level).  Depending  to  the  active
    *  conformance set, only the corresponding invariants will be active.
    */
    template <typename TParameter>
    class IVerifier
    {

      protected:

        ///  Invariant holder.
        class TInvariant
        {

          public:

            ///  String with a message explaining the invariant.
            TString   yMessage;

            ///  Integer with the conformance identifier for the invariant.
            int   iConformanceLevel;


          public:

            //
            //  C O N S T R U C T O R S
            //

            /**
            *  Builds a new instance.
            *  @param pkcMESSAGE         Invariant description message.
            *  @param iCONFORMANCE_LEVEL Conformance level.
            */
            TInvariant (const char* pkcMESSAGE, int iCONFORMANCE_LEVEL) :
              yMessage          (pkcMESSAGE)                            ,
              iConformanceLevel (iCONFORMANCE_LEVEL)                    {}

        };  // class TInvariant


      protected:

        ///  Integer with default conformance level.
        int   iDefaultConformanceLevel;


      public:

        /**
        *  Map with restrictions.
        *  @see #addInvariant().
        */
        TMap<long int, TInvariant>   tInvariantMap;

        /**
        *  Handler for variable size list of arguments.
        *  @see #require().
        */
        mutable TVariableArgumentArray<TParameter>   tArgumentArray;

        /**
        *  Set with conformance identifiers.
        *  @see #addConformanceLevel(), #removeConformanceLevel().
        */
        TConformanceLevelSet   tConformanceLevelSet;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds a new instance.
        *  @param iDEFAULT_CONFORMANCE_LEVEL Default conformance level.
        */
        IVerifier (int iDEFAULT_CONFORMANCE_LEVEL)              :
          iDefaultConformanceLevel (iDEFAULT_CONFORMANCE_LEVEL) ,
          tInvariantMap        ()                               ,
          tArgumentArray       ()                               ,
          tConformanceLevelSet ()                               {}

        ///  Destroyes the instance.
        virtual ~IVerifier (void) {}

        /**
        *  Adds a new invariant.
        *  @param liINVARIANT_IDENTIFIER Invariant identifier (must be unique).
        *  @param iCONFORMANCE_LEVEL     Conformance level.
        *  @param pkcMESSAGE             Description message.
        */
        void addInvariant ( long int    liINVARIANT_IDENTIFIER ,
                            int         iCONFORMANCE_LEVEL     ,
                            const char* pkcMESSAGE             )
        {
          tInvariantMap.bind ( liINVARIANT_IDENTIFIER                      ,
                               TInvariant (pkcMESSAGE, iCONFORMANCE_LEVEL) );
        }

        /**
        *  Adds a new invariant.
        *  @param liINVARIANT_IDENTIFIER Invariant identifier (must be unique).
        *  @param pkcMESSAGE             Description message.
        */
        void addInvariant (long int liINVARIANT_IDENTIFIER, const char* pkcMESSAGE)
        {
          tInvariantMap.bind ( liINVARIANT_IDENTIFIER                            ,
                               TInvariant (pkcMESSAGE, iDefaultConformanceLevel) );
        }

        /**
        *  Activates a new conformance level.
        *  @param iCONFORMANCE_LEVEL Conformance level to be activated.
        */
        void addConformanceLevel (int iCONFORMANCE_LEVEL)
        {
          if ( tConformanceLevelSet.end() == tConformanceLevelSet.find (iCONFORMANCE_LEVEL) )
          {
            tConformanceLevelSet.insert (iCONFORMANCE_LEVEL);
          }
        }

        /**
        *  Deactivates a conformance level.
        *  @param iCONFORMANCE_LEVEL Conformance level to be deactivated.
        */
        void removeConformanceLevel (int iCONFORMANCE_LEVEL)
        {
          if ( tConformanceLevelSet.end() != tConformanceLevelSet.find (iCONFORMANCE_LEVEL) )
          {
            tConformanceLevelSet.erase (iCONFORMANCE_LEVEL);
          }
        }

        ///  Empties conformance level set.
        void clearConformanceLevelSet (void)
        {
          tConformanceLevelSet.clear();
        }


        ///  Sets default conformance level.
        void resetConformanceLevelSet (void)
        {
          tConformanceLevelSet.erase();
          tConformanceLevelSet.insert (iDefaultConformanceLevel);
        }


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Verifies that invariant corresponding to \a liINVARIANT_IDENTIFIER,
        *  is true. If not, then throws the corresponding exception.
        *  @param liINVARIANT_IDENTIFIER Invariant identifier.
        *  @param ...                    Parameters (TParameter).
        *  @see TParameter.
        */
        void require (long int liINVARIANT_IDENTIFIER...) const
        {
          va_list   tVa_list;

          va_start (tVa_list, liINVARIANT_IDENTIFIER);
          requireList (liINVARIANT_IDENTIFIER, tVa_list);
          va_end (tVa_list);
        }

        /**
        *  Verifies that invariant corresponding to \a liINVARIANT_IDENTIFIER,
        *  is true. If not, then throws the corresponding exception.
        *  @param liINVARIANT_IDENTIFIER Invariant identifier.
        *  @param tVA_LIST               Variable argument parameter list.
        */
        void requireList (long int liINVARIANT_IDENTIFIER, va_list tVA_LIST) const
        {
          const TInvariant&   rktInvariant (tInvariantMap [liINVARIANT_IDENTIFIER]);

          //
          //  Only checks invariants that belong to an active
          //  conformance-level.
          //
          tArgumentArray.initialize (tVA_LIST);
          if ( tConformanceLevelSet.end() != tConformanceLevelSet.find (rktInvariant.iConformanceLevel) )
          {
            if ( !verify (liINVARIANT_IDENTIFIER) )
            {
              va_end (tVA_LIST);
              throw TViolationException ( rktInvariant.iConformanceLevel , 
                                          rktInvariant.yMessage.c_str()  );
            }
          }
        }

        /**
        *  Verifies that the invariant corresponding to
        *  \a liINVARIANT_IDENTIFIER, is true.
        *  @param liINVARIANT_IDENTIFIER Invariant identifier.
        *  @return True if invariant is verified.
        */
        virtual bool verify (long int liINVARIANT_IDENTIFIER) const = 0;

    };  // class IVerifier

  }  // namespace invariant

}  // namespace mpcl

  
#endif  // not _MPCL_INVARIANT_VERIFIER__
