/*
*  Name:      base_smart_pointer.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Base class for smart pointers
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2001-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_MEMORY_BASE_SMART_POINTER__
#define _MPCL_MEMORY_BASE_SMART_POINTER__

#include "../exceptions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for MEMORY.
  namespace memory
  {

    /**
    *  Shared cell class.  It contains  the pointer  to the object  being shared.
    *  @warning This class has been extracted  from  TSmartPointer to make easier
    *           the  copy  between  different instantiations  of  TSmartPointers.
    */
    class TSharedCell
    {

      public:

        ///  Reference count to the object being shared.
        long unsigned int   luiReferenceCount;

        ///  Pointer to the object being shared.
        void*   pvItem;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TSharedCell (void)         :
          luiReferenceCount (0L)   ,
          pvItem            (NULL) {}

        /**
        *  Builds a new instance from \a pkvITEM.
        *  @param pkvITEM Source item.
        */
        TSharedCell (const void* pkvITEM)                 :
          luiReferenceCount (1L)                          ,
          pvItem            (const_cast<void*> (pkvITEM)) {}

    };  // struct TSharedCell

    ///  Base class for smart pointers.
    class TBaseSmartPointer
    {

      protected:

        /**
        *  Cell const reference type.
        *  @see lib.container.requirements.
        */
        typedef
          const TSharedCell&
          cell_const_reference;

        /**
        *  Cell pointer type.
        *  @see lib.container.requirements.
        */
        typedef
          TSharedCell*
          cell_pointer;

        /**
        *  Cell reference type.
        *  @see lib.container.requirements.
        */
        typedef
          TSharedCell&
          cell_reference;

        ///  Pointer to shared cell.
        mutable cell_pointer   ptSharedCell;


      public:

        ///  Cell type.
        typedef
          TSharedCell
          cell_type;


      protected:

        //
        //  C O N S T R U C T O R S
        //

        void attachTo (const TBaseSmartPointer& rkqtBASE_SMART_POINTER)
        {
          ptSharedCell = rkqtBASE_SMART_POINTER.ptSharedCell;
          if ( ptSharedCell )
          {
            ++(ptSharedCell->luiReferenceCount);
          }
        }


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TBaseSmartPointer (void)
          throw()             :
          ptSharedCell (NULL) {}

        ///  Destroys the instance.
        virtual ~TBaseSmartPointer (void) {}


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Returns the address of the object pointed to.
        *  @return Current pointed object.
        */
        const void* address (void) const throw()
        {
          const void*   pkvRet = NULL;

          if ( ptSharedCell )
          {
            pkvRet = ptSharedCell->pvItem;
          }
          return pkvRet;
        }

        bool equal (const TBaseSmartPointer& rkqtBASE_SMART_POINTER) const
          throw()
        {
          //
          //  Checks that both  smart pointers  point to the  same cell and that both
          //  pointers are not null.
          //
          register bool   gEqual = ( !ptSharedCell && !rkqtBASE_SMART_POINTER.ptSharedCell );

          if ( ptSharedCell && rkqtBASE_SMART_POINTER.ptSharedCell )
          {
            gEqual = ( ptSharedCell->pvItem == rkqtBASE_SMART_POINTER.ptSharedCell->pvItem );
          }
          return gEqual;
        }

        /**
        *  Checks if the smart pointer is empty.
        *  @return True if the instance is pointing to nothing.
        */
        bool isNull (void) const throw()
        {
          return ( ptSharedCell == NULL );
        }

        /**
        *  Returns the reference count to the item pointed by this.
        *  @return Reference count.
        *  @exception TConstraintException Whenever pointer is null.
        */
        long unsigned int references (void) const
          throw (TConstraintException)
        {
          if ( !ptSharedCell )
          {
            throw TConstraintException ("null pointer", __FILE__, __LINE__);
          }
          return ptSharedCell->luiReferenceCount;
        }

    };  // class TBaseSmartPointer

  }  // namespace memory

}  // namespace mpcl


#endif  // not _MPCL_MEMORY_BASE_SMART_POINTER__
