/*
*  Name:      smart_pointer.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Smart pointer
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_MEMORY_SMART_POINTER__
#define _MPCL_MEMORY_SMART_POINTER__

#include <memory>
#include "base_smart_pointer.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for MEMORY.
  namespace memory
  {

    /**
    *  Smart pointer for objects obtained via the operator new.
    *
    *  This class  provides  a semantics  of shared ownership.  After the initial
    *  construction,  a  TSmartPointer  owns the  object it  holds a  pointer to.
    *  Copying a  TSmartPointer copies the  pointer and shares ownership with the
    *  destination.  If more than  one TSmartPointer  owns the same object at the
    *  same  time the behaviour of the  program is to delete the object when  the
    *  last   TSmartPointer  is  destroyed.   This  class   instances  should  be
    *  initialized from a TItem class (or derived) instances created with new, or
    *  from other TSmartPointer instances. This is so, because when this instance
    *  is asigned to another  pointer value,  the former pointer value is deleted
    *  when there are no more  references to it.
    *
    *  A  common  error when using  smart pointers is  assigning a  TSmartPointer
    *  object to another TSmartPointer object type-casted to TItem*.  This is one
    *  of worst things you can do,  because both instances think that they do not
    *  share the object with the other smart pointer.
    *
    *  Until version 1.42,  this class implemented  sharing between derived smart
    *  pointers.  This is an  error prone  'feature' that  let the  smart pointer
    *  holding  an address  of a  parent  object and  thinking  that it  holds an
    *  address of a  derived object.  Now let us see  some examples of bad use of
    *  this old features:
    *
    *    qtFather = new TSon();         //  qtFather == :TSon + x
    *    DynamicCast (qtSon, qtFather); //  qtSon    == :TSon + x => ERROR
    *
    *    qtSon    = new TSon(); //  qtSon    == :TSon
    *    qtFather = qtSon;      //  qtFather == :TSon => ERROR
    *
    *  These are examples of bad use of current smart pointers:
    *
    *    class TParent {};
    *    class TSon : public TParent {};
    *
    *    TParent                  tParent;
    *    TParent*                 ptParent1 (new TParent());
    *    TParent*                 ptParent2 (new TParent());
    *    TSmartPointer<TParent>   qtParent (&tParent);
    *
    *    qtParent = new TSon();  //  Error when releasing a static allocated object (&tParent).
    *    qtParent = ptParent1;
    *    qtParent = ptParent2;   //  Object pointed by ptParent1 is deleted.
    *    qtParent = ptParent1;   //  Error, ptParent1 points to a deleted object.
    *
    *  @pre ( !ptSharedCell )
    *  @pre ( ptSharedCell != NULL ) => ( ptSharedCell->pvItem != NULL )
    *  @see operator new().
    *  @warning This class is not thread safe.
    */
    template <typename TItem>
    class TSmartPointer: public TBaseSmartPointer
    {

      public:

        /**
        *  Item const pointer type.
        *  @see lib.container.requirements.
        */
        typedef
          const TItem*
          const_pointer;

        /**
        *  Item const reference type.
        *  @see lib.container.requirements.
        */
        typedef
          const TItem&
          const_reference;

        /**
        *  Item pointer type.
        *  @see lib.container.requirements.
        */
        typedef
          TItem*
          pointer;

        /**
        *  Item reference type.
        *  @see lib.container.requirements.
        */
        typedef
          TItem&
          reference;

        /**
        *  Item size type.
        *  @see lib.container.requirements.
        */
        typedef
          size_t
          size_type;

        /**
        *  Item value type.
        *  @see lib.container.requirements.
        */
        typedef
          TItem
          value_type;


      protected:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Frees space of the cell.
        *  @pre ( ptSharedCell != NULL )
        *  @pre ( ptSharedCell->pvItem != NULL )
        */
        void freeCell (void)
        {
          delete (pointer) ptSharedCell->pvItem;
          delete ptSharedCell;
        }

        /**
        *  Reserves space for a new cell that will contains \a pktITEM.
        *  @param pktITEM Item pointer to contain.
        */
        void reserveCell (const_pointer pktITEM)
        {
          if ( pktITEM )
          {
            ptSharedCell = new TSharedCell (pktITEM);
          }
        }


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TSmartPointer (void)  :
          TBaseSmartPointer() {}

        /**
        *  Builds a new instance from another instance.
        *  @param rkqtITEM Source instance.
        */
        TSmartPointer (const TSmartPointer& rkqtITEM) :
          TBaseSmartPointer()
        {
          attachTo (rkqtITEM);
        }

  // <FIXME id="smart-pointers" comment="this method can assign other type of objects">
        /**
        *  Builds a new instance from another TSmartPointer instantiation.
        *  @param rkqtITEM_DERIVED Source instance.
        */
        template <typename TDerivedItem>
        explicit TSmartPointer (const TSmartPointer<TDerivedItem>& rkqtITEM_DERIVED) :
          TBaseSmartPointer()
        {

          //
          //  Declares two variables for checking assign ability.
          //
          register pointer         ptItem;
          register TDerivedItem*   ptDerivedItem (NULL);

          //
          //  Checks for possible syntax errors about using classes with no relation with
          //  this  instance to assign  to this  instance.  At  last,  it attaches to the
          //  derived pointer.
          //
          ptItem = ptDerivedItem;
          attachTo (rkqtITEM_DERIVED);

        }  // TSmartPointer()
  // </FIXME>

        /**
        *  Builds a new instance from a pointer.
        *  @param pktITEM Source item.
        *  @exception std::bad_alloc If there is no memory available.
        */
        explicit TSmartPointer (const_pointer pktITEM) :
          TBaseSmartPointer()
        {
          reserveCell (pktITEM);
        }

        ///  Destroys the instance.
        ~TSmartPointer (void)
        {
          release();
        }

        /**
        *  Creates a new cell holding the pointer \a pktITEM.
        *  @param pktITEM Source pointer.
        *  @return This instance.
        */
        TSmartPointer& operator = (const_pointer pktITEM)
        {
          release();
          reserveCell (pktITEM);
          return *this;
        }

        TSmartPointer& operator = (const TSmartPointer& rkqtITEM)
        {
          if ( rkqtITEM.ptSharedCell != ptSharedCell )
          {
            release();
            attachTo (rkqtITEM);
          }
          return *this;
        }

        /**
        *  Decrements reference count and releases the shared cell and the
        *  bound item if this is the last owner.
        */
        void release (void)
        {

          if ( ptSharedCell )
          {
            //
            //  ptSharedCell->pvItem must be different from NULL.
            //  Decrements the reference counter.
            //
            --(ptSharedCell->luiReferenceCount);
            if ( !ptSharedCell->luiReferenceCount )
            {
              //
              //  Warning:  There is no check when  deleting  ptSharedCell->pvItem (for
              //            NULL value) cause  there is no way to assign a NULL pointer
              //            value.
              //
              freeCell();
            }
            
            //
            //  Anyway, clears the pointer to the shared cell.
            //
            ptSharedCell = NULL;
          }

        }  // release()


      public:

        //
        //  S E L E C T O R S
        //

        /**
        *  Function to cast this instance to another non-const.
        *  @return The constant instance copy.
        */
        template <typename TPointer>
        TPointer constCast (void) const throw()
        {
          TPointer   tPointer = NULL;

          if ( ptSharedCell )
          {
            tPointer = const_cast<TPointer> ((pointer) ptSharedCell->pvItem);
          }
          return tPointer;
        }

        template <typename TPointer>
        TPointer dynamicCast (void) const
          throw (TConstraintException)
        {
          TPointer   tPointer = NULL;

          if ( !ptSharedCell )
          {
  // <FIXME id="smart-pointers" comment="this exception must not be thrown">
            throw TConstraintException ("null pointer", __FILE__, __LINE__);
  // </FIXME>
          }
          else
          {
            tPointer = dynamic_cast<TPointer> ((pointer) ptSharedCell->pvItem);
            if ( !tPointer )
            {
              throw TConstraintException ("can not dynamic-cast pointer", __FILE__, __LINE__);
            }
          }
          return tPointer;
        }

        /**
        *  Returns a pointer to the shared item.
        *  @return A pointer to the shared item.
        */
        pointer get (void) const throw()
        {
          register pointer   ptResultItem = NULL;

          if ( ptSharedCell )
          {
            ptResultItem = (pointer) ptSharedCell->pvItem;
          }
          return ptResultItem;
        }

        /**
        *  Returns true if both instances hold the same item or are both null.
        *  @param rkqtITEM Source instance.
        *  @return True if both instances are bound.
        */
        bool operator == (const TSmartPointer& rkqtITEM) const throw()
        {
          return equal (rkqtITEM);
        }

        /**
        *  Returns false if both instances are bound.
        *  @param rkqtITEM Source instance.
        *  @return False if both instances are bound.
        */
        bool operator != (const TSmartPointer& rkqtITEM) const throw()
        {
          return !operator == (rkqtITEM);
        }

        /**
        *  Returns true if this represents a NULL pointer.
        *  @return True if this represents a NULL pointer.
        */
        bool operator ! (void) const throw()
        {
          return ( !ptSharedCell );
        }

        /**
        *  Returns the shared item.
        *  @return The shared item.
        */
        pointer operator -> (void) const
          throw (TConstraintException)
        {
          if ( !ptSharedCell )
          {
            throw TConstraintException ("null pointer", __FILE__, __LINE__);
          }
          return (pointer) ptSharedCell->pvItem;
        }

        /**
        *  Returns the shared item.
        *  @return The shared item.
        */
        reference operator * (void) const
          throw (TConstraintException)
        {
          if ( !ptSharedCell )
          {
            throw TConstraintException ("null pointer", __FILE__, __LINE__);
          }
          return *((pointer) ptSharedCell->pvItem);
        }

    };  // class TSmartPointer

  }  // namespace memory

}  // namespace mpcl


#endif  // not _MPCL_MEMORY_SMART_POINTER__
