/*
*  Name:      config_processor.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   CGI config processor
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_NET_CGI_CONFIG_PROCESSOR__
#define _MPCL_NET_CGI_CONFIG_PROCESSOR__

#include "../../text/html/form.hh"
#include "../../text/string.hh"
#include "../../util/prefs/config_processor.hh"


///  Name-space for MPCL.
namespace mpcl
{
  
  ///  Name-space for NET.
  namespace net
  {

    ///  Name-space for CGI.
    namespace cgi
    {

      using text::TString;
      using util::prefs::TOption;

      /**
      *  CGI configuration processor.   It  parses  input  from  command-line,  and
      *  standard input in CGI fashion.
      *
      *  In CGI programs, the parameters come in this format:
      *
      *    <name_1>=[<value_1>]&<name_2>=[<value_2>]&...<name_n>=[<value_n>]
      *
      *  Furthermore, options can have more than one value.  One of the most useful
      *  techniques used in HTMLL is coding  identifier and value as NAME attribute
      *  of THtmlInputImage (input type=image). This must be done cause to the fact
      *  that,  when that type of input is used,  the value provided by the  server
      *  is:
      *
      *    <name.x>=<position_x>&<name.y>=<position_y>
      *
      *  As  you  can  see,  there is no value attribute, so  HTMLL  bypasses  this
      *  problem encoding name as a pair of identifier and value.  Then, the server
      *  will give us, something like this:
      *
      *    <identifier/value.x>=<position_x>&<identifier/value.y>=<position_y>
      *
      *  Next this is translated to:
      *
      *    <identifier>=<value>&<identifier>=<value>
      */
      class TConfigProcessor : protected util::prefs::TConfigProcessor
      {

        private:

          ///  Imports TOptionVector.
          using util::prefs::TConfigProcessor::TOptionVector;

          /**
          *  True if there is any kind of interactive input (from command-line
          *  or from standard-input).
          */
          bool   gHasInput;

          ///  Vector with HTML form options.
          mutable TOptionVector   tFormOptionVector;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance, and adds all environment variables sets by
          *  the HTTP server.
          *  @param iPARAMETER_COUNT Parameter count.
          *  @param ppkcPARAMETERS   Parameter array.
          */
          TConfigProcessor (int iPARAMETER_COUNT, const char** ppkcPARAMETERS);

          /**
          *  Loads options from the source defined in the CGI environment
          *  variables (from standard-input or from an environment variable).
          */
          void processOptions (void);

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::setCommandLineParameters;

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::setConfigFileName;

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::setCurrentNameSpace;

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::addOption;


        protected:

          //
          //  C O N S T R U C T O R S
          //

          ///  Loads variables from command-line in CGI fashion.
          void processCommandLine (void);

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::processConfigFile;

          /**
          *  Loads variables from standard-input in CGI fashion.
          */
          void readFormData (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);

          /**
          *  Updates  form  variable  named \a rkyNAME with value  \a rkyVALUE.  If
          *  there is no form variable named \a rkyNAME,  then a new form option is
          *  added.
          *  @param rkyNAME  Form variable name.
          *  @param rkyVALUE Form variable value.
          *  @return The form variable.
          */
          TOption& updateFormOption (const TString& rkyNAME, const TString& rkyVALUE);

          TOption& updateOptionWithName ( const TString& rkyNAME  ,
                                          const TString& rkyVALUE );

        public:

          //
          //  S E L E C T O R S
          //

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::environmentValue;

          /**
          *  Gets form option value corresponding to name \a pkcNAME.
          *  @param pkcNAME Option name.
          *  @return The corresponding option.
          */
          const TString& getFormValue (const char* pkcNAME) const;

          /**
          *  Gets form option value corresponding to name \a rkyNAME.
          *  @param rkyNAME Option name.
          *  @return The corresponding option.
          */
          const TString& getFormValue (const std::string& rkyNAME) const
          {
            return getFormValue (rkyNAME.c_str());
          }

          /**
          *  True if there is an form option called \a rkyNAME.
          */
          bool hasFormOption (const TString& rkyNAME) const;

          /**
          *  Returns  true  if there was any  kind of input  (from command-line, or
          *  from standard input).
          */
          bool hasInput (void) const
          {
            return gHasInput;
          }

          /**
          *  True if there is an option called \a rkyNAME.
          */
          bool hasOption (const TString& rkyNAME) const;

          ///  Imports all versions of this operator.
          using util::prefs::TConfigProcessor::operator [];

          ///  Imports all versions of this method.
          using util::prefs::TConfigProcessor::programFileName;

          /**
          *  Updates  form \a qtTARGET_FORM  with values  parsed from  input  (from
          *  command-line, or from standard input).
          */
          void updateForm (text::html::TForm& rtTARGET_FORM) const;

      };  // class TConfigProcessor

    }  // namespace cgi

  }  // namespace net

}  // namespace mpcl


#endif  // not _MPCL_NET_CGI_CONFIG_PROCESSOR__
