/*
*  Name:         cookie.hh
*  Author:       Francisco Rodrigo Escobedo Robles
*  Contributor:  Rafael Jesus Alcantara Perez
*  Summary:      HTTP Cookie handling
*  Date:         $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:     $Revision: 8535 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@pepix.net>
*  Copyright (C) 2001-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_NET_CGI_COOKIE__
#define _MPCL_NET_CGI_COOKIE__

#include "../../io/streamable.hh"
#include "../../text/gmt_date.hh"
#include "../../text/string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for NET.
  namespace net
  {

    ///  Name-space for CGI.
    namespace cgi
    {

      using io::IStreamable;
      using text::IDate;
      using text::TGmtDate;
      using text::TString;

      /**
      *  CGI cookie class. Creates HTTP cookies for session-keeping. Also reads
      *  them from the client.
      */
      class TCookie : public IStreamable<>
      {

        private:

          ///  True if it is secure.
          bool   gSecure;

          ///  Date of expiration.
          TGmtDate   tExpires;

          ///  Domain.
          TString    yDomain;

          ///  Name.
          TString    yName;

          ///  Path.
          TString    yPath;

          ///  Value.
          TString    yValue;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance from another instance.
          *  @param rktCGI_COOKIE Source instance.
          */
          TCookie (const TCookie& rktCGI_COOKIE)   :
            IStreamable<> ()                       ,
            gSecure       (rktCGI_COOKIE.gSecure)  ,
            tExpires      (rktCGI_COOKIE.tExpires) ,
            yDomain       (rktCGI_COOKIE.yDomain)  ,
            yName         (rktCGI_COOKIE.yName)    ,
            yPath         (rktCGI_COOKIE.yPath)    ,
            yValue        (rktCGI_COOKIE.yValue)   {}

          /**
          *  Builds a new instance.
          *  @param rkyNAME    Name.
          *  @param rkyVALUE   Value.
          *  @param rktEXPIRES Date of expiration.
          *  @param rkyPATH    Path.
          *  @param rkyDOMAIN  Domain.
          *  @param kgSECURE   True if it must be used with HTTPS.
          */
          TCookie ( const TString&  rkyNAME                               ,
                    const TString&  rkyVALUE                              ,
                    const TGmtDate& rktEXPIRES = TGmtDate (IDate::eEmpty) ,
                    const TString&  rkyPATH    = ""                       ,
                    const TString&  rkyDOMAIN  = ""                       ,
                    const bool      kgSECURE   = false                    ) :
            IStreamable<> ()                                                ,
            gSecure       (kgSECURE)                                        ,
            tExpires      (rktEXPIRES)                                      ,
            yDomain       (rkyDOMAIN)                                       ,
            yName         (rkyNAME)                                         ,
            yPath         (rkyPATH)                                         ,
            yValue        (rkyVALUE)                                        {}

          /**
          *  Builds a new instance from an input stream.
          *  @param rtSOURCE_ISTREAM Source stream.
          */
          TCookie (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM) :
            IStreamable<> ()                                                     ,
            gSecure       (false)                                                ,
            tExpires      (TGmtDate (IDate::eEmpty))                             ,
            yDomain       ()                                                     ,
            yName         ()                                                     ,
            yPath         ()                                                     ,
            yValue        ()
          {
            read (rtSOURCE_ISTREAM);
          }


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Reads an instance from stream \a rtSOURCE_ISTREAM.
          *  @param rtSOURCE_ISTREAM Source input stream.
          */
          void read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);

          /**
          *  Writes the instance onto stream \a rtTARGET_OSTREAM.
          *  @param rtTARGET_OSTREAM Target output stream.
          */
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;


        public:

          //
          //  S E L E C T O R S
          //

          ///  Gets the name of the cookie.
          TString name (void) const
          {
            return yName;
          }

          ///  Gets the value of the cookie.
          TString value (void) const
          {
            return yValue;
          }

          /**
          *  Returns true if both cookies are equal.
          *  @param rktCOOKIE Second cookie.
          *  @return True if both cookies are equal.
          */
          bool operator == (const TCookie& rktCOOKIE) const
          {
            return ( ( yName == rktCOOKIE.yName ) && ( yValue == rktCOOKIE.yValue ) );
          }

      };  // class TCookie

    }  // namespace cgi

  }  // namespace net

}  // namespace mpcl


#endif  // not _MPCL_NET_CGI_COOKIE__
