/*
*  Name:      session.hh
*  Authors:   Francisco Rodrigo Escobedo Robles
*             Rafael Jesus Alcantara Perez
*  Summary:   HTTP State Management
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 2000-2001  Francisco Rodrigo Escobedo Robles <frer@pepix.net>
*  Copyright (C) 2002       Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_NET_CGI_SESSION__
#define _MPCL_NET_CGI_SESSION__

#include <iostream>
#include <sstream>
#include "../../io/streamable.hh"
#include "../../text/regex/matcher.hh"
#include "../../text/string.hh"
#include "../../util/collection/map.hh"
#include "exceptions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for NET.
  namespace net
  {

    ///  Name-space for CGI.
    namespace cgi
    {

      using text::regex::TMatcher;
      using util::collection::TMap;

      ///  CGI session tracker class. Uses generic cookies for session tracking.
      template <typename TCookie>
      class TSession : public IStreamable<>
      {

        public:

          ///  Map for the subvalues of an structured cookie.
          typedef
            TMap<TString, TString>
            TSubSubCookieValueMap;


        private:

          ///  State information class.
          class TStateInformation
          {

            public:

              ///  True if it has to stand between sessions.
              bool   gKeep;

              ///  Cookie instance.
              TCookie   tCookie;


            public:

              ///  Builds a new instance.
              TStateInformation (const TCookie& rktCOOKIE) :
                gKeep   (false)                            ,
                tCookie (rktCOOKIE)                        {}

              ///  Builds a new instance.
              TStateInformation (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM) :
                gKeep   (false)                                                                ,
                tCookie (rtSOURCE_ISTREAM)                                                     {}

          };  // class TStateInformation

          ///  Map for the simple and final values.
          typedef
            TMap<TString, TStateInformation>
            TStateInformationMap;

          ///  Map for the structured cookies with two selectors.
          typedef
            TMap<TString, TSubSubCookieValueMap>
            TSubSubCookieMap;

          ///  Map for the structured cookies with one selector.
          typedef
            TMap<TString, TSubSubCookieMap>
            TSubCookieMap;

          ///  Map for the structured cookie names.
          typedef
            TMap<TString, TSubCookieMap>
            TStructuredCookieMap;

          ///  Iterators for the maps.
          typedef
            typename TStateInformationMap::iterator
            TStateInformationMapIterator;

          typedef
            typename TSubSubCookieValueMap::iterator
            TSubSubCookieValueMapIterator;

          typedef
            typename TSubCookieMap::iterator
            TSubCookieMapIterator;

          typedef
            typename TSubSubCookieMap::iterator
            TSubSubCookieMapIterator;

          typedef
            typename TStructuredCookieMap::iterator
            TStructuredCookieMapIterator;

          ///  Character constants for the separators.
          static const char   _kcSubCookieNameSeparator    = '(';
          static const char   _kcSubCookieMembersSeparator = ',';
          static const char   _kcSubCookieSeparator        = ')';
          static const char   _kcSubCookieValuesSeparator  = '=';

          //
          //  An alternative notation >:
          //  static const char   _kcSubCookieNameSeparator = '>';
          //  static const char   _kcSubCookieSeparator     = ':';
          //

          ///  Final state of the cookies.
          mutable TStateInformationMap   tStateInformationMap;

          ///  Structured cookie names.
          mutable TStructuredCookieMap   tStructuredCookieMap;

          ///  Structured cookies with one selector.
          mutable TSubCookieMap   tSubCookieMap;

          ///  Structured cookies with two selectors.
          mutable TSubSubCookieMap   tSubSubCookieMap;

          ///  Values of a structured cookie.
          mutable TSubSubCookieValueMap   tSubSubCookieValueMap;


        private:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Decomposes an structured cookie in its parts and store it.
          */
          void decomposeCookie (TCookie tCOOKIE);

          /**
          *  Decompose a subcookie and return a map with the pairs.
          */
          TSubSubCookieValueMap decomposeSubCookie (const TString& ySOURCE_VALUE) const;

          /**
          *  Recomposes all the cookies and store them in tStateInformationMap.
          */
          void recomposeCookies (void) const;

          /**
          *  Sets a cookie in the internal representation.
          */
          void setCookie (const TCookie& rktCOOKIE) const;

          /**
          *  Sets a cookie in the internal representation.
          */
          void setCookie (const TString& rkyCOOKIE_NAME, const TString& rkyCOOKIE_VALUE) const
          {
            setCookie (TCookie (rkyCOOKIE_NAME, rkyCOOKIE_VALUE));
          }


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TSession (void)         :
            tStateInformationMap  () ,
            tStructuredCookieMap  () ,
            tSubCookieMap         () ,
            tSubSubCookieMap      () ,
            tSubSubCookieValueMap () {}

          /**
          *  Builds a new instance from a C-style string.
          *  @param pkcSOURCE_STRING String with cookies definitions.
          */
          TSession (const char* pkcSOURCE_STRING) :
            tStateInformationMap  ()                 ,
            tStructuredCookieMap  ()                 ,
            tSubCookieMap         ()                 ,
            tSubSubCookieMap      ()                 ,
            tSubSubCookieValueMap ()
          {
            std::basic_istringstream<char_type, traits_type>   tIstringstream (pkcSOURCE_STRING);

            read (tIstringstream);
          }

          /**
          *  Builds a new instance from a string.
          *  @param rkySOURCE_STRING String with cookies definitions.
          */
          TSession (const TString& rkySOURCE_STRING) :
            tStateInformationMap  ()                    ,
            tStructuredCookieMap  ()                    ,
            tSubCookieMap         ()                    ,
            tSubSubCookieMap      ()                    ,
            tSubSubCookieValueMap ()
          {
            std::basic_istringstream<char_type, traits_type>   tIstringstream (rkySOURCE_STRING.c_str());

            read (tIstringstream);
          }

          /**
          *  Builds a new instance from a stream.
          *  @param rtSOURCE_ISTREAM Source stream to read instance from.
          */
          TSession (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM) :
            tStateInformationMap  ()                                                 ,
            tStructuredCookieMap  ()                                                 ,
            tSubCookieMap         ()                                                 ,
            tSubSubCookieMap      ()                                                 ,
            tSubSubCookieValueMap ()
          {
            read (rtSOURCE_ISTREAM);
          }

          /**
          *  Erases a simple cookie from its name.
          *  @param rkyCOOKIE_NAME Cookie name.
          */
          void erase (const TString& rkyCOOKIE_NAME)
            throw (TVariableNotFoundException);

          /**
          *  Erases an structured subcookie from its name and its selector.
          *  @param rkyCOOKIE_NAME         Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR Sub-cookie selector.
          */
          void erase ( const TString& rkyCOOKIE_NAME         ,
                       const TString& rkySUB_COOKIE_SELECTOR )
            throw (TVariableNotFoundException);

          /**
          *  Erases an structured sub-cookie from its name and its two selectors.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          */
          void erase ( const TString& rkyCOOKIE_NAME          ,
                       const TString& rkySUB_COOKIE_SELECTOR1 ,
                       const TString& rkySUB_COOKIE_SELECTOR2 )
            throw (TVariableNotFoundException);

          /**
          *  Erases an  structured sub-cookie from its name,  its two selectors and
          *  its subname.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          *  @param rkySUB_COOKIE_NAME      Sub-cookie name.
          */
          void erase ( const TString& rkyCOOKIE_NAME          ,
                       const TString& rkySUB_COOKIE_SELECTOR1 ,
                       const TString& rkySUB_COOKIE_SELECTOR2 ,
                       const TString& rkySUB_COOKIE_NAME      )
            throw (TVariableNotFoundException);

          /**
          *  Sets session transient status.
          *  @param rkyCOOKIE_NAME Cookie name.
          *  @param gKEEP          True if cookie has to stand.
          */
          void keep (const TString& rkyCOOKIE_NAME, bool gKEEP = true)
          {
            TStateInformationMapIterator   I = tStateInformationMap.find (rkyCOOKIE_NAME);

            if ( tStateInformationMap.end() == I )
            {
              throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
            }
            I->second.gKeep = gKEEP;
          }

          /**
          *  Sets a cookie from a given one.
          *  @param rktCOOKIE Cookie to be set.
          */
          void set (const TCookie& rktCOOKIE)
          {
            setCookie (rktCOOKIE);
          }

          /**
          *  Sets a cookie from a name and a value.
          *  @param rkyCOOKIE_NAME  Cookie name.
          *  @param rkyCOOKIE_VALUE Cookie value.
          */
          void set (const TString& rkyCOOKIE_NAME, const TString& rkyCOOKIE_VALUE)
          {
            setCookie (rkyCOOKIE_NAME, rkyCOOKIE_VALUE);
          }

          /**
          *  Sets a cookie from a name, a cookie selector and a map of values.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR  Sub-cookie selector.
          *  @param rktSUB_COOKIE_VALUE_MAP Sub-cookie values map.
          */
          void set ( const TString&               rkyCOOKIE_NAME          ,
                     const TString&               rkySUB_COOKIE_SELECTOR  ,
                     const TSubSubCookieValueMap& rktSUB_COOKIE_VALUE_MAP );

          /**
          *  Sets a cookie from a name, two cookie selectors and a map of values.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          *  @param rktSUB_COOKIE_VALUE_MAP Sub-cookie values map.
          */
          void set ( const TString&               rkyCOOKIE_NAME          ,
                     const TString&               rkySUB_COOKIE_SELECTOR1 ,
                     const TString&               rkySUB_COOKIE_SELECTOR2 ,
                     const TSubSubCookieValueMap& rktSUB_COOKIE_VALUE_MAP );


        protected:

          //
          //  S E L E C T O R S
          //

          /**
          *  Reads an instance from stream \a rtSOURCE_ISTREAM.
          *  @param rtSOURCE_ISTREAM Source input stream.
          */
          void read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM);

          /**
          *  Writes the instance onto stream \a rtTARGET_OSTREAM.
          *  @param rtTARGET_OSTREAM Target output stream.
          */
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns true if there are no cookies.
          *  @return True if there are no cookiesw.
          */
          bool isEmpty (void) const
          {
            return ( tStructuredCookieMap.empty() &&
                     tSubCookieMap.empty()        &&
                     tStateInformationMap.empty()  );
          }

          /**
          *  Gets a simple cookie from its name.
          *  @param rkyCOOKIE_NAME Cookie name.
          *  @return The cookie value.
          */
          TString get (const TString& rkyCOOKIE_NAME) const
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its selector.
          *  @param rkyCOOKIE_NAME         Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR Sub-cookie selector.
          *  @return A constant map with the subcookie values.
          */
          const TSubSubCookieValueMap& get ( const TString& rkyCOOKIE_NAME         ,
                                             const TString& rkySUB_COOKIE_SELECTOR ) const
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its selector.
          *  @param rkyCOOKIE_NAME         Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR Sub-cookie selector.
          *  @return A writable map with the subcookie values.
          */
          TSubSubCookieValueMap& get ( const TString& rkyCOOKIE_NAME         ,
                                       const TString& rkySUB_COOKIE_SELECTOR )
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its selector.
          *  @param rkyCOOKIE_NAME         Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR Sub-cookie selector.
          *  @param rtSUB_COOKIE_VALUES    Sub-cookie value map.
          */
          void get ( const TString&         rkyCOOKIE_NAME         ,
                     const TString&         rkySUB_COOKIE_SELECTOR ,
                     TSubSubCookieValueMap& rtSUB_COOKIE_VALUES    ) const
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its two selectors.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          *  @return A map with the subcookie values.
          */
          const TSubSubCookieValueMap& get ( const TString& rkyCOOKIE_NAME          ,
                                             const TString& rkySUB_COOKIE_SELECTOR1 ,
                                             const TString& rkySUB_COOKIE_SELECTOR2 ) const
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its two selectors.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          *  @return A map with the subcookie values.
          */
          TSubSubCookieValueMap& get ( const TString& rkyCOOKIE_NAME          ,
                                       const TString& rkySUB_COOKIE_SELECTOR1 ,
                                       const TString& rkySUB_COOKIE_SELECTOR2 )
            throw (TVariableNotFoundException);

          /**
          *  Gets a structured subcookie from its name and its two selectors.
          *  @param rkyCOOKIE_NAME          Cookie name.
          *  @param rkySUB_COOKIE_SELECTOR1 Sub-cookie selector.
          *  @param rkySUB_COOKIE_SELECTOR2 Sub-cookie selector.
          *  @param rtSUB_COOKIE_VALUES     Sub-cookie value map.
          */
          void get ( const TString&         rkyCOOKIE_NAME          ,
                     const TString&         rkySUB_COOKIE_SELECTOR1 ,
                     const TString&         rkySUB_COOKIE_SELECTOR2 ,
                     TSubSubCookieValueMap& rtSUB_COOKIE_VALUES     ) const
            throw (TVariableNotFoundException);

          /**
          *  Returns true if there is a cookie named \a rkyCOOKIE_NAME.
          *  @param rkyCOOKIE_NAME Cookie name.
          *  @return True if cookie exists.
          */
          bool has (const TString& rkyCOOKIE_NAME) const
          {
            TStateInformationMapIterator   I = tStateInformationMap.find (rkyCOOKIE_NAME);

            return ( I != tStateInformationMap.end() );
          }

      };  // class TSession

    }  // namespace cgi

  }  // namespace net

}  // namespace mpcl


//
//  C O N S T R U C T O R S
//

template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
decomposeCookie (TCookie tCOOKIE)
{

  using text::Format;
  using text::TString;

  static const TString   _kyDoubleSelectorCookieFormat =
    Format ("%%s%c%%s%c%%t%c"            ,
            _kcSubCookieMembersSeparator ,
            _kcSubCookieNameSeparator    ,
            _kcSubCookieSeparator        );

  static const TString   _kySingleSelectorCookieFormat =
    Format ("%%s%c%%t%c"              ,
            _kcSubCookieNameSeparator ,
            _kcSubCookieSeparator     );

  const TString   kyName (tCOOKIE.name());
  TMatcher        tMatcher (tCOOKIE.value());
  TString         ySelector1;
  TString         ySelector2;
  TString         yValue;

  //
  //  First check for  a cookie with 2 selectors, then  for 1 selector and
  //  then for a simple value.
  //
  if ( tMatcher.match (_kyDoubleSelectorCookieFormat.c_str()) )
  {
    while ( tMatcher.match (_kyDoubleSelectorCookieFormat.c_str()) )
    {
      tMatcher.scan (_kyDoubleSelectorCookieFormat.c_str(), &ySelector1, &ySelector2, &yValue, NULL);
      set (kyName, ySelector1, ySelector2, decomposeSubCookie (yValue));
    }
    return;
  }
  else if ( tMatcher.match (_kySingleSelectorCookieFormat.c_str()) )
  {
    while ( tMatcher.match (_kySingleSelectorCookieFormat.c_str()) )
    {
      tMatcher.scan (_kySingleSelectorCookieFormat.c_str(), &ySelector1, &yValue, NULL);
      set (kyName, ySelector1, decomposeSubCookie (yValue));
    }
    return;
  }
  else
  {
    set (tCOOKIE);
  }

}  // decomposeCookie()


template <typename TCookie>
typename mpcl::net::cgi::TSession<TCookie>::TSubSubCookieValueMap
mpcl::net::cgi::TSession<TCookie>::
decomposeSubCookie (const TString& ySOURCE_VALUE) const
{

  using text::Format;
  using text::TString;

  static const TString   _kySimpleCookieFormat =
    Format
    (
      "%c%%s%c%%s"                 ,
      _kcSubCookieMembersSeparator ,
      _kcSubCookieValuesSeparator
    );

  static const TString   _kySimpleCookieFormatFirst =
    Format ("%%s%c%%s", _kcSubCookieValuesSeparator);

  TMatcher                tMatcher (ySOURCE_VALUE);
  TSubSubCookieValueMap   tRet;
  TString                 yName;
  TString                 yValue;

  if ( tMatcher.match (_kySimpleCookieFormatFirst.c_str()) )
  {
    tMatcher.scan (_kySimpleCookieFormatFirst.c_str(), &yName, &yValue, NULL);
    tRet.bind (yName, yValue);
  }
  while ( tMatcher.match (_kySimpleCookieFormat.c_str()) )
  {
    tMatcher.scan (_kySimpleCookieFormat.c_str(), &yName, &yValue, NULL);
    tRet.bind (yName, yValue);
  }
  return tRet;

}  // decomposeSubCookie()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
erase (const TString& rkyCOOKIE_NAME)
  throw (TVariableNotFoundException)
{

  TStructuredCookieMapIterator   I = tStructuredCookieMap.find (rkyCOOKIE_NAME);
  TSubCookieMapIterator          J = tSubCookieMap.find        (rkyCOOKIE_NAME);
  TStateInformationMapIterator   K = tStateInformationMap.find (rkyCOOKIE_NAME);

  //
  //  First check for a Cookie with 2 selectors, then 1, then a simple one.
  //
  if ( tStructuredCookieMap.end() != I )
  {
    tStructuredCookieMap.erase (I);
  }
  else if ( tSubCookieMap.end() != J )
  {
    tSubCookieMap.erase (J);
  }
  else if ( tStateInformationMap.end() == K )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  tStateInformationMap.erase (K);

}  // erase()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
erase ( const TString& rkyCOOKIE_NAME         ,
        const TString& rkySUB_COOKIE_SELECTOR )
  throw (TVariableNotFoundException)
{

  TSubSubCookieMapIterator   J;
  TSubCookieMapIterator      I = tSubCookieMap.find (rkyCOOKIE_NAME);

  if ( tSubCookieMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  else
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR);
    if ( I->second.end() == J )
    {
      throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
    }
  }
  I->second.erase(J);

}  // erase()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
erase ( const TString& rkyCOOKIE_NAME          ,
        const TString& rkySUB_COOKIE_SELECTOR1 ,
        const TString& rkySUB_COOKIE_SELECTOR2 )
  throw (TVariableNotFoundException)
{

  TSubCookieMapIterator          J;
  TSubSubCookieMapIterator       K;
  TStructuredCookieMapIterator   I = tStructuredCookieMap.find (rkyCOOKIE_NAME);

  if ( tStructuredCookieMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  else
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR1);
    if ( I->second.end() == J )
    {
      throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
    }
    else
    {
      K = J->second.find (rkySUB_COOKIE_SELECTOR2);
      if ( J->second.end() == K )
      {
        throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
      }
    }
  }
  J->second.erase(K);

}  // erase()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
erase ( const TString& rkyCOOKIE_NAME          ,
        const TString& rkySUB_COOKIE_SELECTOR1 ,
        const TString& rkySUB_COOKIE_SELECTOR2 ,
        const TString& rkySUB_COOKIE_NAME      )
  throw (TVariableNotFoundException)
{

  TSubCookieMapIterator           J;
  TSubSubCookieMapIterator        K;
  TSubSubCookieValueMapIterator   L;
  TStructuredCookieMapIterator    I = tStructuredCookieMap.find (rkyCOOKIE_NAME);

  if ( tStructuredCookieMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  else
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR1);
    if ( I->second.end() == J )
    {
      throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
    }
    else
    {
      K = J->second.find (rkySUB_COOKIE_SELECTOR2);
      if ( J->second.end() == K )
      {
        throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
      }
      else
      {
        L = K->second.find (rkySUB_COOKIE_NAME);
        if ( K->second.end() == L )
        {
          throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
        }
      }
    }
  }
  K->second.erase(L);

}  // erase()


//
//  S E L E C T O R S
//

template <typename TCookie>
mpcl::text::TString mpcl::net::cgi::TSession<TCookie>::
get (const text::TString& rkyCOOKIE_NAME) const
  throw (TVariableNotFoundException)
{

  TStateInformationMapIterator   I = tStateInformationMap.find (rkyCOOKIE_NAME);

  if ( tStateInformationMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  return I->second.tCookie.value();

}  // get()


template <typename TCookie>
inline const typename mpcl::net::cgi::TSession<TCookie>::TSubSubCookieValueMap&
mpcl::net::cgi::TSession<TCookie>::
get (const TString& rkyCOOKIE_NAME, const TString& rkySUB_COOKIE_SELECTOR) const
  throw (TVariableNotFoundException)
{

  return (const TSubSubCookieValueMap&) get (rkyCOOKIE_NAME, rkySUB_COOKIE_SELECTOR);

}  // get()


template <typename TCookie>
typename mpcl::net::cgi::TSession<TCookie>::TSubSubCookieValueMap& mpcl::net::cgi::TSession<TCookie>::
get ( const TString& rkyCOOKIE_NAME         ,
      const TString& rkySUB_COOKIE_SELECTOR )
  throw (TVariableNotFoundException)
{

  TSubSubCookieMapIterator   J;
  TSubCookieMapIterator      I = tSubCookieMap.find (rkyCOOKIE_NAME);

  if ( tSubCookieMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  else
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR);
    if ( I->second.end() == J )
    {
      throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
    }
  }
  return J->second;

}  // get()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
get ( const TString&         rkyCOOKIE_NAME         ,
      const TString&         rkySUB_COOKIE_SELECTOR ,
      TSubSubCookieValueMap& rtSUB_COOKIE_VALUES    ) const
  throw (TVariableNotFoundException)
{

  rtSUB_COOKIE_VALUES = get (rkyCOOKIE_NAME, rkySUB_COOKIE_SELECTOR);

}  // get()


template <typename TCookie>
typename mpcl::net::cgi::TSession<TCookie>::TSubSubCookieValueMap& mpcl::net::cgi::TSession<TCookie>::
get ( const TString& rkyCOOKIE_NAME          ,
      const TString& rkySUB_COOKIE_SELECTOR1 ,
      const TString& rkySUB_COOKIE_SELECTOR2 )
  throw (TVariableNotFoundException)
{

  TSubCookieMapIterator          J;
  TSubSubCookieMapIterator       K;
  TStructuredCookieMapIterator   I = tStructuredCookieMap.find (rkyCOOKIE_NAME);

  if ( tStructuredCookieMap.end() == I )
  {
    throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
  }
  else
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR1);
    if ( I->second.end() == J )
    {
      throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
    }
    else
    {
      K = J->second.find (rkySUB_COOKIE_SELECTOR2);
      if ( J->second.end() == K )
      {
        throw (TVariableNotFoundException ("variable not found", __FILE__, __LINE__));
      }
    }
  }
  return K->second;

}  // get()


template <typename TCookie>
inline const typename mpcl::net::cgi::TSession<TCookie>::TSubSubCookieValueMap& mpcl::net::cgi::TSession<TCookie>::
get ( const TString& rkyCOOKIE_NAME          ,
      const TString& rkySUB_COOKIE_SELECTOR1 ,
      const TString& rkySUB_COOKIE_SELECTOR2 ) const
  throw (TVariableNotFoundException)
{

  return (const TSubSubCookieValueMap&) get ( rkyCOOKIE_NAME          ,
                                              rkySUB_COOKIE_SELECTOR1 ,
                                              rkySUB_COOKIE_SELECTOR2 );

}  // get()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
get ( const TString&         rkyCOOKIE_NAME          ,
      const TString&         rkySUB_COOKIE_SELECTOR1 ,
      const TString&         rkySUB_COOKIE_SELECTOR2 ,
      TSubSubCookieValueMap& rtSUB_COOKIE_VALUES     ) const
  throw (TVariableNotFoundException)
{

  rtSUB_COOKIE_VALUES = get (rkyCOOKIE_NAME, rkySUB_COOKIE_SELECTOR1, rkySUB_COOKIE_SELECTOR2);

}  // get()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
read (std::basic_istream<char_type, traits_type>& rtSOURCE_ISTREAM)
{

  TStateInformation   tStateInformation (rtSOURCE_ISTREAM);

// <FIXME id="cookies" comment="there should exists a cookie factory class">
  while ( rtSOURCE_ISTREAM.good() && !tStateInformation.tCookie.name().empty() )
  {
    tStateInformationMap.bind (tStateInformation.tCookie.name(), tStateInformation);
    decomposeCookie (tStateInformation.tCookie);
    rtSOURCE_ISTREAM >> tStateInformation.tCookie;
  }
  if ( !tStateInformation.tCookie.name().empty() )
  {
    tStateInformationMap.bind (tStateInformation.tCookie.name(), tStateInformation);
    decomposeCookie (tStateInformation.tCookie);
  }
// </FIXME>

}  // read()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::recomposeCookies (void) const
{

  TStructuredCookieMapIterator    tStructuredCookieMapIndex;
  TSubCookieMapIterator           tSubCookieMapIndex;
  TSubSubCookieMapIterator        tSubSubCookieMapIndex;
  TSubSubCookieValueMapIterator   tSubSubCookieValueMapIndex;
  TString                         ySubCookie;
  TString                         ySubSubCookie;
  TString                         ySubSubCookieValues;

  //
  //  Structured Cookies with 1 selector.
  //
  for ( tSubCookieMapIndex = tSubCookieMap.begin()    ;
        ( tSubCookieMap.end() != tSubCookieMapIndex ) ;
        ++tSubCookieMapIndex                          )
  {
    for ( tSubSubCookieMapIndex = tSubCookieMapIndex->second.begin(), ySubSubCookie = "" ;
          ( tSubCookieMapIndex->second.end() != tSubSubCookieMapIndex )                  ;
          ++tSubSubCookieMapIndex                                                        )
    {
      for ( tSubSubCookieValueMapIndex = tSubSubCookieMapIndex->second.begin(), ySubSubCookieValues = "" ;
            ( tSubSubCookieMapIndex->second.end() != tSubSubCookieValueMapIndex )                        ;
            ++tSubSubCookieValueMapIndex                                                                 )
      {
        ySubSubCookieValues += tSubSubCookieValueMapIndex->first;
        ySubSubCookieValues += _kcSubCookieValuesSeparator;
        ySubSubCookieValues += tSubSubCookieValueMapIndex->second;
        ySubSubCookieValues += _kcSubCookieMembersSeparator;
      }
      if ( !ySubSubCookieValues.empty() )
      {
        ySubSubCookieValues.resize (ySubSubCookieValues.size() - 1);
      }
      ySubSubCookie += tSubSubCookieMapIndex->first;
      ySubSubCookie += _kcSubCookieNameSeparator;
      ySubSubCookie += ySubSubCookieValues;
      ySubSubCookie += _kcSubCookieSeparator;
    }
    setCookie (tSubCookieMapIndex->first, ySubSubCookie);
  }

  //
  //  Structured Cookies with 2 selectors.
  //
  for ( tStructuredCookieMapIndex = tStructuredCookieMap.begin()    ;
        ( tStructuredCookieMap.end() != tStructuredCookieMapIndex ) ;
        ++tStructuredCookieMapIndex                                 )
  {
    for ( tSubCookieMapIndex = tStructuredCookieMapIndex->second.begin(), ySubCookie = "" ;
          ( tStructuredCookieMapIndex->second.end() != tSubCookieMapIndex )               ;
          ++tSubCookieMapIndex                                                            )
    {
      for ( tSubSubCookieMapIndex = tSubCookieMapIndex->second.begin(), ySubSubCookie = "" ;
            ( tSubCookieMapIndex->second.end() != tSubSubCookieMapIndex )                  ;
            ++tSubSubCookieMapIndex                                                        )
      {
        for ( tSubSubCookieValueMapIndex = tSubSubCookieMapIndex->second.begin(), ySubSubCookieValues = "" ;
              ( tSubSubCookieMapIndex->second.end() != tSubSubCookieValueMapIndex )                        ;
              ++tSubSubCookieValueMapIndex                                                                 )
        {
          ySubSubCookieValues += tSubSubCookieValueMapIndex->first  +
                                 _kcSubCookieValuesSeparator        +
                                 tSubSubCookieValueMapIndex->second +
                                 _kcSubCookieMembersSeparator       ;
        }
        if ( !ySubSubCookieValues.empty() )
        {
          ySubSubCookieValues.resize (ySubSubCookieValues.size() - 1);
        }
        ySubSubCookie += tSubSubCookieMapIndex->first +
                         _kcSubCookieNameSeparator    +
                         ySubSubCookieValues          +
                         _kcSubCookieSeparator        ;
      }
      ySubCookie += tSubCookieMapIndex->first    +
                    _kcSubCookieMembersSeparator +
                    ySubSubCookie                ;
    }
    setCookie (tStructuredCookieMapIndex->first, ySubCookie);
  }

}  // recomposeCookies()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
set ( const TString&               rkyCOOKIE_NAME          ,
      const TString&               rkySUB_COOKIE_SELECTOR  ,
      const TSubSubCookieValueMap& rktSUB_COOKIE_VALUE_MAP )
{

  TSubSubCookieMapIterator   J;
  TSubSubCookieMap           tSubSubCookieMapTmp;
  TSubCookieMapIterator      I = tSubCookieMap.find (rkyCOOKIE_NAME);

  if ( tSubCookieMap.end() != I )
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR);
    if ( I->second.end() != J )
    {
      J->second = rktSUB_COOKIE_VALUE_MAP;
    }
    else
    {
      I->second.bind (rkySUB_COOKIE_SELECTOR, rktSUB_COOKIE_VALUE_MAP);
    }
  }
  else
  {
    tSubSubCookieMapTmp.bind (rkySUB_COOKIE_SELECTOR, rktSUB_COOKIE_VALUE_MAP);
    tSubCookieMap.bind       (rkyCOOKIE_NAME,         tSubSubCookieMapTmp);
  }

}  // set()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
set ( const TString&               rkyCOOKIE_NAME          ,
      const TString&               rkySUB_COOKIE_SELECTOR1 ,
      const TString&               rkySUB_COOKIE_SELECTOR2 ,
      const TSubSubCookieValueMap& rktSUB_COOKIE_VALUE_MAP )
{

  TSubCookieMapIterator          J;
  TSubSubCookieMapIterator       K;
  TSubCookieMap                  tSubCookieMapTmp;
  TSubSubCookieMap               tSubSubCookieMapTmp;
  TStructuredCookieMapIterator   I = tStructuredCookieMap.find (rkyCOOKIE_NAME);

  if ( tStructuredCookieMap.end() != I )
  {
    J = I->second.find (rkySUB_COOKIE_SELECTOR1);
    if ( I->second.end() != J )
    {
      K = J->second.find (rkySUB_COOKIE_SELECTOR2);
      if ( J->second.end() != K )
      {
        K->second = rktSUB_COOKIE_VALUE_MAP;
      }
      else
      {
        J->second.bind (rkySUB_COOKIE_SELECTOR2, rktSUB_COOKIE_VALUE_MAP);
      }
    }
    else
    {
      tSubSubCookieMapTmp.bind (rkySUB_COOKIE_SELECTOR2, rktSUB_COOKIE_VALUE_MAP);
      I->second.bind           (rkySUB_COOKIE_SELECTOR1, tSubSubCookieMapTmp);
    }
  }
  else
  {
    tSubSubCookieMapTmp.bind  (rkySUB_COOKIE_SELECTOR2, rktSUB_COOKIE_VALUE_MAP);
    tSubCookieMapTmp.bind     (rkySUB_COOKIE_SELECTOR1, tSubSubCookieMapTmp);
    tStructuredCookieMap.bind (rkyCOOKIE_NAME,          tSubCookieMapTmp);
  }

}  // set()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
setCookie (const TCookie& rktCOOKIE) const
{

  TStateInformationMapIterator   I = tStateInformationMap.find (rktCOOKIE.name());

  if ( tStateInformationMap.end() != I )
  {
    I->second.tCookie = rktCOOKIE;
  }
  else
  {
    tStateInformationMap.bind (rktCOOKIE.name(), TStateInformation (rktCOOKIE));
  }

}  // setCookie()


template <typename TCookie>
void mpcl::net::cgi::TSession<TCookie>::
write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const
{

  TStateInformationMapIterator   I;

  recomposeCookies();
  for (I = tStateInformationMap.begin(); ( tStateInformationMap.end() != I ) ;++I)
  {
    if ( I->second.gKeep )
    {
      rtTARGET_OSTREAM << I->second.tCookie;
    }
  }

}  // write()


#endif  // not _MPCL_NET_CGI_SESSION__
