/*
*  Name:      orb.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   CORBA ORB class
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2001-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_NET_CORBA_ORB__
#define _MPCL_NET_CORBA_ORB__

#include <algorithm>
#include <list>
#include "../../text/string.hh"
#include CORBA_H


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for NET.
  namespace net
  {

    ///  Name-space for CORBA.
    namespace corba
    {

      using text::TString;

      ///  Base class for CORBA servant applications.
      class TOrb
      {

        private:

          /**
          *  True  when ORB  must put the  client and server  in the same address
          *  space.
          */
          bool   gIsColocated;

          ///  True if there is COS Naming Service support.
          bool   gHasNameService;

          ///  CORBA ORB.
          CORBA::ORB_var   qtOrb;

          ///  CORBA Root POA.
          PortableServer::POA_var   qtPoa;

          ///  CORBA Root POA manager.
          PortableServer::POAManager_var   qtPoaManager;

          ///  COS Naming Service root naming context.
          CosNaming::NamingContext_var   qtRootNamingContext;


        private:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Binds a  new child  context, in context  \a rqtNAMING_CONTEXT, named
          *  \a rkyCONTEXT_NAME.
          *  @param rqtNAMING_CONTEXT Parent context.
          *  @param rkyCONTEXT_NAME   New context name (attribute ID).
          *  @return The new child context.
          *  @warning The new context has empty attribute KIND.
          */
          CosNaming::NamingContext_var bindContext ( CosNaming::NamingContext_var& rqtNAMING_CONTEXT ,
                                                     const TString&                rkyCONTEXT_NAME   );

          /**
          *  Binds a new child context, in the root context ("NameService") named
          *  \a rkyCONTEXT_NAME.
          *  @param rkyCONTEXT_NAME New context name (attribute ID).
          *  @return The new child context.
          *  @warning The new context has empty attribute KIND.
          */
          CosNaming::NamingContext_var bindContext (const TString& rkyCONTEXT_NAME)
          {
            return bindContext (qtRootNamingContext, rkyCONTEXT_NAME);
          }


        public:

          //
          //  C O N S T R U C T O R S
          //

          TOrb (int iPARAMETER_COUNT, const char** ppkcPARAMETERS);

          ~TOrb (void)
          {
            qtOrb->destroy();
          }

          /**
          *  Activates the servant object \a ptOBJECT in the Root POA using.
          *  @param ptOBJECT Servant object to activate.
          *  @return A reference to the incarnated object.
          */
          template <typename TObject, typename TObjectReference>
          TObjectReference enrol (TObject* ptOBJECT)
          {
            TObjectReference   qtObject;

            qtPoa->activate_object (ptOBJECT);
            qtObject = ptOBJECT->_this();
            ptOBJECT->_remove_ref();
            return qtObject;
          }

          /**
          *  Activates the servant object \a ptOBJECT in the Root POA using.
          *  @param ptOBJECT         Servant object to activate.
          *  @param rkyCOMPOUND_NAME Compound  name for  registering  in  the COS
          *                          Naming Service.
          *  @return A reference to the incarnated object.
          */
          template <typename TObject, typename TObjectReference>
          TObjectReference enrol (TObject* ptOBJECT, const TString& rkyCOMPOUND_NAME)
          {
            //
            //  To clarify declarations.
            //
            typedef
              std::list<TString>
              TStringList;

            TStringList::const_iterator    I;
            TObjectReference               qtObject;
            TStringList                    tContextList;
            CosNaming::Name                tObjectName;
            CosNaming::NamingContext_var   qtCurrentContext;
            CosNaming::NamingContext_var   qtNewContext;
            TString                        yObjectId;
            TString                        yObjectIdAndKind;
            TString                        yObjectKind;
            TString::size_type             zDotPosition;

  // <FIXME comment="is this needed?">
            qtObject = enrol<TObject, TObjectReference> (ptOBJECT);
  // </FIXME>
            Split ( rkyCOMPOUND_NAME.begin()                              ,
                    rkyCOMPOUND_NAME.end()                                ,
                    '/'                                                   ,
                    std::back_insert_iterator<TStringList> (tContextList) );
            qtCurrentContext = qtRootNamingContext;
            yObjectIdAndKind = tContextList.back();
            tContextList.pop_back();
            for (I = tContextList.begin(); ( I != tContextList.end() ) ;++I)
            {
              qtNewContext     = bindContext (qtCurrentContext, *I);
              qtCurrentContext = qtNewContext;
            }

            //
            //  Binds the new object into current context.
            //
            zDotPosition = yObjectIdAndKind.find ('.');
            if ( !zDotPosition )
            {
              yObjectId = yObjectIdAndKind;
            }
            else
            {
              yObjectId   = yObjectIdAndKind.substr (0, zDotPosition);
              yObjectKind = yObjectIdAndKind.substr (zDotPosition + 1);
            }
            tObjectName.length(1);
            tObjectName [0].id   = yObjectId.c_str();
            tObjectName [0].kind = yObjectKind.c_str();
            try
            {
              qtCurrentContext->bind (tObjectName, qtObject);
            }
            catch (const CosNaming::NamingContext::AlreadyBound&)
            {
              qtCurrentContext->rebind (tObjectName, qtObject);
            }
            return qtObject;
          }

          void finish (void)
          {
            qtOrb->shutdown (false);
          }

          /**
          *  Requests an object reference to the object  identified by the string
          *  \a rkyOBJECT_IDENTIFIER. This identifier has the same format used in
          *  the CORBA member  function  \a CORBA::ORB::string_to_object().  This
          *  method is used usually in the client side.
          *  @param rkyOBJECT_IDENTIFIER Object identifier.
          *  @return A reference to the requested object.
          *  @see CORBA::ORB::string_to_object().
          */
          template <typename TObject, typename TObjectReference>
          TObjectReference request (const TString& rkyOBJECT_IDENTIFIER)
          {
            CORBA::Object_var   qtObject = qtOrb->string_to_object (rkyOBJECT_IDENTIFIER.c_str());

            return TObject::_narrow (qtObject);
          }

          /**
          *  Requests an object reference to the object  identified by the string
          *  \a pkcOBJECT_IDENTIFIER. This identifier has the same format used in
          *  the CORBA member  function  \a CORBA::ORB::string_to_object().  This
          *  method is used usually in the client side.
          *  @param pkcOBJECT_IDENTIFIER Object identifier.
          *  @return A reference to the requested object.
          *  @see CORBA::ORB::string_to_object().
          */
          template <typename TObject, typename TObjectReference>
          TObjectReference request (const char* pkcOBJECT_IDENTIFIER)
          {
            CORBA::Object_var   qtObject = qtOrb->string_to_object (pkcOBJECT_IDENTIFIER);

            return TObject::_narrow (qtObject);
          }

          /**
          *  Sets colocation state.
          *  @param gCOLOCATED True if object must be colocated (both, client and
          *                    server share the same address space).
          */
          void setColocated (bool gCOLOCATED)
          {
            gIsColocated = gCOLOCATED;
          }

          int start (void)
          {
            qtPoaManager->activate();
            if ( !gIsColocated )
            {
              qtOrb->run();
            }
            return 0;
          }


        public:

          //
          //  S E L E C T O R S
          //

          template <typename TObjectReference>
          TString name (TObjectReference qtOBJECT)
          {
            return qtOrb->object_to_string (qtOBJECT);
          }

      };  // class TOrb

    }  // namespace corba

  }  // namespace net

}  // namespace mpcl


#endif  // not _MPCL_NET_CORBA_ORB__
