/*
*  Name:      condition.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Condition class for POSIX 1003.1c compliant conditions
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_POSIX_1_CONDITION__
#define _MPCL_SYSTEM_POSIX_1_CONDITION__

#include <pthread.h>
#include "../condition.hh"
#include "../exceptions.hh"
#include "mutex.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    /**
    *  Condition for POSIX operating system class.
    *  @see TOperatingSystem.
    */
    class TCondition : public ICondition
    {

      private:

        ///  Condition identifier.
        pthread_cond_t   tPosixId;

        ///  Associated mutex identifier.
        TMutex&          rtMutex;


      public:

        //
        //  C O N S T R U C T O R S
        //

        /**
        *  Builds a new instance.
        *  @param rtSOURCE_MUTEX Lock for entry point to condition variable.
        *  @exception TErrorException If any error occurs.
        */
        TCondition (TMutex& rtSOURCE_MUTEX)
          throw (TErrorException)         :
          ICondition ()                   ,
          tPosixId   ()                   ,
          rtMutex    (rtSOURCE_MUTEX)
        {
          if ( pthread_cond_init (&tPosixId, NULL) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        /**
        *  Destroys the instance.
        *  @exception TErrorException If any error occurs.
        */
        ~TCondition()
          throw (TErrorException)
        {
          if ( pthread_cond_destroy (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        /**
        *  Does the same thing that the parent function but includes exceptions.
        *  @exception TErrorException If any error occurs.
        *  @see #signal(), #broadcast().
        */
        void wait (void)
          throw (TErrorException)
        {
          if ( pthread_cond_wait (&tPosixId, &rtMutex.tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        /**
        *  Does the same thing that the parent function but includes exceptions.
        *  @param zSECONDS     Seconds to wait for.
        *  @param zNANOSECONDS Nanoseconds to wait for.
        *  @return True if time has been reached or exceeded.
        *  @exception TErrorException If any error occurs.
        *  @see #wait().
        */
        bool timedWait (size_t zSECONDS, size_t zNANOSECONDS = 0)
          throw (TErrorException);

        /**
        *  Does the same thing that the parent function but includes exceptions.
        *  @exception TErrorException If any error occurs.
        */
        void signal (void)
          throw (TErrorException)
        {
          if ( pthread_cond_signal (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        /**
        *  Does the same thing that the parent function but includes exceptions.
        *  @exception TErrorException If any error occurs.
        */
        void broadcast (void)
          throw (TErrorException)
        {
          if ( pthread_cond_broadcast (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

    };  // class TCondition

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_POSIX_1_CONDITION__
