/*
*  Name:      mutex.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Mutex class for POSIX 1003.1c compliant mutexes
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_POSIX_1_MUTEX__
#define _MPCL_SYSTEM_POSIX_1_MUTEX__

#include <pthread.h>
#include "../defs.hh"
#include "../exceptions.hh"
#include "../mutex.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    /**
    *  Mutex for POSIX operating system class.
    *  @see TOperatingSystem.
    */
    class TMutex : public IMutex
    {

        ///  For accessing tPosixId from TCondition.
        friend class TCondition;


      private:

        ///  Mutex identifier.
        pthread_mutex_t   tPosixId;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TMutex (void)
          throw (TErrorException) :
          IMutex   ()             ,
          tPosixId () 
        {
          if ( pthread_mutex_init (&tPosixId, NULL) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        ///  Destroys the instance.
        ~TMutex()
          throw (TErrorException)
        {
          if ( pthread_mutex_destroy (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        ///  Lock mutex.
        void lock (void)
          throw (TErrorException)
        {
          if ( pthread_mutex_lock (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        ///  Unlock mutex.
        void unlock (void)
          throw (TErrorException)
        {
          if ( pthread_mutex_unlock (&tPosixId) )
          {
            throw TErrorException (GetErrorMessage(), __FILE__, __LINE__);
          }
        }

        /**
        *  Lock mutex if unlocked, if not, then it returns false.
        *  @return True if unlocked (locking it, if so).
        */
        bool tryLock (void)
        {
          return ( pthread_mutex_trylock (&tPosixId) != 0 );
        }

    };  // class TMutex

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_POSIX_1_MUTEX__
