/*
*  Name:      system.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Wrapper class for POSIX.1 compliant operating systems
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_SYSTEM_POSIX_1_SYSTEM__
#define _MPCL_SYSTEM_POSIX_1_SYSTEM__

#include <syslog.h>
#include "../system.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for SYSTEM.
  namespace system
  {

    ///  Wrapper for POSIX.1 operating system services.
    class TSystem : public ISystem
    {

      public:
        
        ///  Conversion of big endian dependent buffers type.
        typedef TBigEndianBuffer   TBigEndianBuffer;

        ///  Conversion of little endian dependent buffers type.
        typedef TLittleEndianBuffer   TLittleEndianBuffer;

        ///  Machine conversions type.
        typedef TMachineDependent   TMachineDependent;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        TSystem (void)
        {
          openlog (NULL, LOG_PID, LOG_USER);
        }
        
        ~TSystem (void)
        {
          closelog();
        }

        /**
        *  Get current directory.
        *  @return Current directory.
        *  @exception std::bad_alloc If there is no memory available.
        */
        TString getCurrentDirectory (void) const;

        char getDirectorySeparator (void) const;

        std::size_t getFileSize (const char* pkcFILE_NAME) const
          throw (TErrorException);

        char getPathSeparator (void) const;

        int getProcessId (void) const;

        /**
        *  Sleeps \a zSECONDS seconds and \a zNANOSECONDS nanoseconds.
        *  According to POSIX.1, sleep() affect only to the current thread.
        *  @param zSECONDS     Seconds to wait.
        *  @param zNANOSECONDS Nanoseconds to wait.
        */
        void sleep (std::size_t zSECONDS, std::size_t zNANOSECONDS = 0) const;

        void writeLogEntry ( const std::string& rkyMESSAGE          ,
                             ELogLevel          eLEVEL    = eNotice ,
                             ELogFacility       eFACILITY = eUser   ) const;

        void yield (void) const;

    };  // class TSystem


    //
    //  E X P O R T E D   V A R I A B L E S
    //

    ///  The system object.
    extern TSystem   tSystem;

  }  // namespace system

}  // namespace mpcl


#endif  // not _MPCL_SYSTEM_POSIX_1_SYSTEM__
