/*
*  Name:      basic_string.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Extension class for STL string
*  Date:      $Date: 2005-03-08 00:21:07 +0100 (mar, 08 mar 2005) $
*  Revision:  $Revision: 10851 $
*
*  Copyright (C) 1999-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_BASIC_STRING__
#define _MPCL_TEXT_BASIC_STRING__

#include <algorithm>
#include <cstring>
#include <locale>
#include <string>
#include "../exceptions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    /**
    *  Extended string.
    *
    *  \warning This class follows the naming conventions used in STL classes.
    */
    template < typename TChar                                ,
               typename TTraits    = std::char_traits<TChar> ,
               typename TAllocator = std::allocator<TChar>   >
    class TBasicString : public std::basic_string<TChar, TTraits, TAllocator>
    {

      public:

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::allocator_type
          allocator_type;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::const_iterator
          const_iterator;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::const_pointer
          const_pointer;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::const_reference
          const_reference;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::const_reverse_iterator
          const_reverse_iterator;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::difference_type
          difference_type;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::iterator
          iterator;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::pointer
          pointer;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::reference
          reference;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::reverse_iterator
          reverse_iterator;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::size_type
          size_type;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::traits_type
          traits_type;

        ///  Import this type.
        typedef
          typename std::basic_string<TChar, TTraits, TAllocator>::value_type
          value_type;

        using std::basic_string<TChar, TTraits, TAllocator>::npos;


      public:

        //
        //  C O N S T R U C T O R S
        //

        ///  Builds a new instance.
        explicit TBasicString (void)                      :
          std::basic_string<TChar, TTraits, TAllocator>() {}

        ///  Builds a new instance.
        TBasicString (const std::basic_string<TChar, TTraits, TAllocator>& rkySOURCE) :
          std::basic_string<TChar, TTraits, TAllocator> (rkySOURCE)                   {}

        ///  Builds a new instance.
        TBasicString (const TBasicString& rkySOURCE)                :
          std::basic_string<TChar, TTraits, TAllocator> (rkySOURCE) {}

        ///  Builds a new instance.
        TBasicString ( const TBasicString& rkySOURCE      ,
                       size_type           zOFFSET        ,
                       size_type           zLENGTH = npos )                           :
          std::basic_string<TChar, TTraits, TAllocator> (rkySOURCE, zOFFSET, zLENGTH) {}

        /**
        *  Extends the parent constructor with checks for NULL string.
        *  @param pkcSOURCE Source string.
        *  @param zLENGTH   Source string length.
        *  @exception TConstraintException If source string is NULL.
        */
        TBasicString (const TChar* pkcSOURCE, size_type zLENGTH) :
          std::basic_string<TChar, TTraits, TAllocator>()
        {
          if ( !pkcSOURCE )
          {
            throw TConstraintException ("null string", __FILE__, __LINE__);
          }
          assign (pkcSOURCE, zLENGTH);
        }

        /**
        *  Extends the parent constructor with checks for NULL string.
        *  @param pkcSOURCE Source string.
        *  @exception TConstraintException If source string is NULL.
        */
        TBasicString (const TChar* pkcSOURCE) :
          std::basic_string<TChar, TTraits, TAllocator>()
        {
          if ( !pkcSOURCE )
          {
            throw TConstraintException ("null string", __FILE__, __LINE__);
          }
          assign (pkcSOURCE);
        }

        ///  Builds a new instance.
        TBasicString (size_type zTIMES, TChar cSOURCE)                    :
          std::basic_string<TChar, TTraits, TAllocator> (zTIMES, cSOURCE) {}

        ///  Builds a new instance.
        template<typename InputIterator>
        TBasicString(InputIterator tBEGIN_ITER, InputIterator tEND_ITER)         :
          std::basic_string<TChar, TTraits, TAllocator> (tBEGIN_ITER, tEND_ITER) {}

        /**
        *  Copies the instance \a rkySOURCE onto this.
        *  @param rkySOURCE Source string.
        *  @return This instance.
        */
        TBasicString& operator = (const TBasicString& rkySOURCE)
        {
          std::basic_string<TChar, TTraits, TAllocator>::operator = (rkySOURCE);
          return *this;
        }

        /**
        *  Copies the string \a pkcSOURCE onto this.
        *  @param pkcSOURCE Source string.
        *  @return This instance.
        */
        TBasicString& operator = (const TChar* pkcSOURCE)
        {
          if ( !pkcSOURCE )
          {
            throw TConstraintException ("null string", __FILE__, __LINE__);
          }
          std::basic_string<TChar, TTraits, TAllocator>::operator = (pkcSOURCE);
          return *this;
        }

        /**
        *  Copies the character \a cSOURCE onto this.
        *  @param cSOURCE Source character.
        *  @return This instance.
        */
        TBasicString& operator = (TChar cSOURCE)
        {
          std::basic_string<TChar, TTraits, TAllocator>::operator = (cSOURCE);
          return *this;
        }

        /**
        *  Replaces all occurrences of \a rkySOURCE1 with \a rkySOURCE2.
        *  It returns the number of times that rkySOURCE1 is found.
        *  @param rkySOURCE1 Source string.
        *  @param rkySOURCE2 Source string.
        *  @return The number of substitutions.
        */
        size_type replaceAll ( const std::basic_string<TChar, TTraits, TAllocator>& rkySOURCE1 ,
                               const std::basic_string<TChar, TTraits, TAllocator>& rkySOURCE2 )
        {
          return replaceAll (rkySOURCE1.c_str(), rkySOURCE2.c_str());
        }

        /**
        *  Replace all occurrences of \a pkcSOURCE1 with \a pkcSOURCE2.
        *  It returns the number of times that \a pkcSOURCE1 is found.
        *  @param pkcSOURCE1 Source string.
        *  @param pkcSOURCE2 Source string.
        *  @return The number of substitutions.
        */
        size_type replaceAll (const TChar* pkcSOURCE1, const TChar* pkcSOURCE2);

        /**
        *  Replace  all occurrences  of \a cSOURCE1 with \a cSOURCE2.  It returns
        *  the number of times that \a cSOURCE1 is found.
        *  @param cSOURCE1 Source character.
        *  @param cSOURCE2 Source char.
        *  @return The number of substitutions.
        */
        size_type replaceAll (TChar cSOURCE1, TChar cSOURCE2);

        /**
        *  Left  inserts  \a zLENGTH  - length()  characters \a cSOURCE  (only if
        *  zLENGTH > length()).
        *  @param cSOURCE Source character.
        *  @param zLENGTH Total final length of the string.
        *  @return This instance.
        */
        TBasicString& leftFill (TChar cSOURCE, size_type zLENGTH);

        /**
        *  Right  inserts  \a zLENGTH - length()  characters \a cSOURCE  (only if
        *  zLENGTH > length()).
        *  @param cSOURCE Source character.
        *  @param zLENGTH Total final length of the string.
        *  @return This instance.
        */
        TBasicString& rightFill (TChar cSOURCE, size_type zLENGTH);

        /**
        *  Suppress all occurrences of cSOURCE.
        *  @param cSOURCE Source character.
        *  @return This instance.
        */
        TBasicString& suppress (TChar cSOURCE);

        /**
        *  Suppress the first occurrence of cSOURCE.
        *  @param cSOURCE Source character.
        *  @return This instance.
        */
        TBasicString& suppressFirst (TChar cSOURCE);

        /**
        *  Suppress the last occurrence of cSOURCE.
        *  @param cSOURCE Source character.
        *  @return This instance.
        */
        TBasicString& suppressLast (TChar cSOURCE);

        /**
        *  Changes all characters to uppercase (using std::toupper()).
        *  @return This instance.
        *  @see std::toupper().
        */
        TBasicString& uppercase (void);

        /**
        *  Changes all characters to lowercase (using std::tolower()).
        *  @return This instance.
        *  @see std::tolower().
        */
        TBasicString& lowercase (void);


      public:

        ///
        //  S E L E C T O R S
        //

        using std::basic_string<TChar, TTraits, TAllocator>::append;
        using std::basic_string<TChar, TTraits, TAllocator>::begin;
        using std::basic_string<TChar, TTraits, TAllocator>::empty;
        using std::basic_string<TChar, TTraits, TAllocator>::end;
        using std::basic_string<TChar, TTraits, TAllocator>::length;

        /**
        *  Returns the occurrences of the string \a pkcSOURCE in this instance.
        *  @param pkcSOURCE Source string.
        *  @return The number of occurrences.
        */
        size_type occurrences (const TChar* pkcSOURCE) const;

        /**
        *  Returns the occurrences of the character \a cSOURCE in this instance.
        *  @param cSOURCE Source character.
        *  @return The number of occurrences.
        */
        size_type occurrences (TChar cSOURCE) const;

        /**
        *  Stores the strings between characters \a cSOURCE (but excluding it) in
        *  the container pointed by the iterator \a tTARGET_ITER.
        *  @param cSOURCE      Separator character.
        *  @param tTARGET_ITER Target container iterator.
        */
        template <typename TOutputIter>
        const TBasicString& split (TChar cSOURCE, TOutputIter tTARGET_ITER) const
        {
          Split (begin(), end(), cSOURCE, tTARGET_ITER);
          return *this;
        }

    };  // class TBasicString


    template <typename TChar, typename TTraits, typename TAllocator>
    inline typename TBasicString<TChar, TTraits, TAllocator>::size_type TBasicString<TChar, TTraits, TAllocator>::
    replaceAll (const TChar* pkcSOURCE1, const TChar* pkcSOURCE2)
    {

      register size_type   zTimes = 0;

      if ( !pkcSOURCE1 || !pkcSOURCE2 )
      {
        throw TConstraintException ("null string in parameters", __FILE__, __LINE__);
      }
      else
      {
        register size_type   zOffset       = 0;
        register size_type   zSourceLength = std::strlen (pkcSOURCE1);
        register size_type   zTargetLength = std::strlen (pkcSOURCE2);

        while ( npos != (zOffset = find (pkcSOURCE1, zOffset)) )
        {
          erase (zOffset, zSourceLength);
          insert (zOffset, pkcSOURCE2);
          zOffset += zTargetLength;
          ++zTimes;
        }
      }
      return zTimes;

    }  // replaceAll()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline typename TBasicString<TChar, TTraits, TAllocator>::size_type TBasicString<TChar, TTraits, TAllocator>::
    replaceAll (TChar cSOURCE1, TChar cSOURCE2)
    {

      register size_type   zTimes  = 0;
      register size_type   zOffset = 0;

      while ( npos != (zOffset = find (cSOURCE1, zOffset)) )
      {
        erase (zOffset, 1);
        insert (zOffset, 1, cSOURCE2);
        ++zOffset;
        ++zTimes;
      }
      return zTimes;

    }  // replaceAll()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    leftFill (TChar cSOURCE, size_type zLENGTH)
    {

      size_type   zThisLength = length();

      if ( zLENGTH > zThisLength )
      {
        assign (std::string (zLENGTH - zThisLength, cSOURCE) + *this);
      }
      return *this;

    }  // leftFill()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    rightFill (TChar cSOURCE, size_type zLENGTH)
    {

      size_t   zThisLength = length();

      if ( zLENGTH > zThisLength )
      {
        append (std::string (zLENGTH - zThisLength, cSOURCE));
      }
      return *this;

    }  // rightFill()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    suppress (TChar cSOURCE)
    {

      iterator   tIter = std::remove (begin(), end(), cSOURCE);

      erase (tIter, end());
      return *this;

    }  // suppress()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    suppressFirst (TChar cSOURCE)
    {

      if ( !empty() )
      {
        const_iterator   tEnd  = end();
        iterator         tIter = begin();

        while ( tIter != tEnd )
        {
          if ( *tIter != cSOURCE )
          {
            if ( tIter != begin() )
            {
              erase (0, tIter - begin());
            }
            break;
          }
          ++tIter;
        }
      }
      return *this;

    }  // suppressFirst()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    suppressLast (TChar cSOURCE)
    {

      if ( !empty() )
      {
        iterator   tBegin = begin();
        iterator   tIter  = end();

        while ( tBegin != tIter )
        {
          --tIter;
          if ( *tIter != cSOURCE )
          {
            if ( tIter != (end() - 1) )
            {
              erase (tIter - tBegin + 1);
            }
            break;
          }
        }
      }
      return *this;

    }  // suppressLast()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    uppercase (void)
    {

      std::locale                TLocale;
      const std::ctype<TChar>&   rktCtype (std::use_facet<std::ctype<TChar> >(TLocale));

      for (iterator I = begin(); ( I != end() ) ;++I)
      {
        *I = rktCtype.toupper (*I);
      }
      return *this;

    }  // uppercase()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline TBasicString<TChar, TTraits, TAllocator>& TBasicString<TChar, TTraits, TAllocator>::
    lowercase (void)
    {

      using std::locale;

      std::locale                TLocale;
      const std::ctype<TChar>&   rktCtype (std::use_facet<std::ctype<TChar> >(TLocale));

      for (iterator I = begin(); ( I != end() ) ;++I)
      {
        *I = rktCtype.tolower (*I);
      }
      return *this;

    }  // lowercase()


    //
    //  S E L E C T O R S
    //

    template <typename TChar, typename TTraits, typename TAllocator>
    inline typename TBasicString<TChar, TTraits, TAllocator>::size_type TBasicString<TChar, TTraits, TAllocator>::
    occurrences (const TChar* pkcSOURCE) const
    {

      register size_type   zOccurrences = 0;

      if ( !pkcSOURCE )
      {
        throw TConstraintException ("null string in parameters", __FILE__, __LINE__);
      }
      else
      {
        register size_type   zOffset       = 0;
        register size_type   zTargetLength = std::strlen (pkcSOURCE);

        while ( npos != (zOffset = find (pkcSOURCE, zOffset)) )
        {
          ++zOccurrences;
          zOffset += zTargetLength;
        }
      }
      return zOccurrences;

    }  // occurrences()


    template <typename TChar, typename TTraits, typename TAllocator>
    inline typename TBasicString<TChar, TTraits, TAllocator>::size_type TBasicString<TChar, TTraits, TAllocator>::
    occurrences (TChar cTARGET) const
    {

      register size_type   zOffset      = 0;
      register size_type   zOccurrences = 0;

      while ( npos != (zOffset = find (cTARGET, zOffset)) )
      {
        ++zOccurrences;
        ++zOffset;
      }
      return zOccurrences;

    }  // occurrences()

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_BASIC_STRING__
