/*
*  Name:      variable_string.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Variable string class for CODEGEN
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_TEXT_CODEGEN_VARIABLE_STRING__
#define _MPCL_TEXT_CODEGEN_VARIABLE_STRING__

#include "../string.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for TEXT.
  namespace text
  {

    ///  Name-space for CODEGEN.
    namespace codegen
    {

      /**
      *  Variable string for pattern based code generator.
      *  @see TPatternBasedCodeGenerator.
      */
      class TVariableString : public TString
      {

        public:

          ///  Variable type type.
          enum EType
          {
            eNormal   , ///<  Variables that must be present in the clause body.
            eOptional , ///<  These variables doesn't need to be present.
            eFile       ///<  Variables that are going to hold file data.
          };


        protected:

          ///  Variable type.
          EType   eType;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TVariableString (void) :
            TString ()           ,
            eType   (eNormal)    {}

          /**
          *  Builds a new instance from \a pkcTEXT.
          *  @param pkcTEXT Source text.
          */
          TVariableString (const char* pkcTEXT) :
            TString (pkcTEXT)                   ,
            eType   (eNormal)                   {}

          /**
          *  Builds a new instance from \a pkcTEXT with maximum \a zMAX_LENGTH
          *  length.
          *  @param pkcTEXT     Source text.
          *  @param zMAX_LENGTH Maximum length readable from \a pkcTEXT.
          */
          TVariableString (const char* pkcTEXT, size_t zMAX_LENGTH) :
            TString (pkcTEXT, zMAX_LENGTH)                          ,
            eType   (eNormal)                                       {}

          /**
          *  Builds a new instance from \a rkySOURCE with type \a eSOURCE_TYPE.
          *  @param rkySOURCE    Source string.
          *  @param eSOURCE_TYPE Source variable type.
          */
          TVariableString (const TString& rkySOURCE, EType eSOURCE_TYPE = eNormal) :
            TString (rkySOURCE)                                                    ,
            eType   (eSOURCE_TYPE)                                                 {}

          /**
          *  Builds a new instance from \a rkySOURCE_VARIABLE.
          *  @param rkySOURCE_VARIABLE Source instance.
          */
          TVariableString (const TVariableString& rkySOURCE_VARIABLE) :
            TString (rkySOURCE_VARIABLE)                              ,
            eType   (rkySOURCE_VARIABLE.eType)                        {}

          /**
          *  Sets variable type.
          *  @param eSOURCE_TYPE Source variable type.
          */
          void setType (EType eSOURCE_TYPE);

          /**
          *  Copies \a rkySOURCE_VARIABLE instance onto this.
          *  @param rkySOURCE_VARIABLE Source instance.
          *  @return This instance.
          */
          TVariableString& operator = (const TVariableString& rkySOURCE_VARIABLE);

          /**
          *  Copies \a rkySOURCE string onto this.
          *  @param rkySOURCE Source string.
          *  @return This instance.
          */
          TVariableString& operator = (const TString& rkySOURCE);


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns variable type.
          *  @return Variable type.
          */
          EType type (void) const;

      };  // class TVariableString

    }  // namespace codegen

  }  // namespace text

}  // namespace mpcl


#endif  // not _MPCL_TEXT_CODEGEN_VARIABLE_STRING__
