/*
*  Name:      general_tree.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   General tree class
*  Date:      $Date: 2004-08-02 19:20:39 +0200 (lun, 02 ago 2004) $
*  Revision:  $Revision: 9838 $
*
*  Copyright (C) 2000-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_COLLECTION_GENERAL_TREE__
#define _MPCL_UTIL_COLLECTION_GENERAL_TREE__

#include <algorithm>
#include <cstddef>
#include <functional>
#include <iterator>
#include <memory>
#include <utility>
#include "../../exceptions.hh"
#include "../../memory/alloc_smart_pointer.hh"
#include "../../memory/smart_pointer.hh"
#include "../general-functions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    ///  Name-space for COLLECTION.
    namespace collection
    {

      ///  Tree node class.
      template < typename TItem                                               ,
                 template <typename TItemA> class TAllocator = std::allocator >
      class TTreeNode
      {

        public:

          ///  Smart pointer to this class.
          typedef
            memory::TAllocSmartPointer<TTreeNode, TAllocator>
            alloc_smart_pointer;

          ///  Const reference type.
          typedef
            const TItem&
            const_reference;

          ///  Pointer type.
          typedef
            TItem*
            pointer;

          ///  Reference type.
          typedef
            TItem&
            reference;

          ///  Value type.
          typedef
            TItem
            value_type;

          ///  Size type.
          typedef
            std::size_t
            size_type;


        public:

          ///  Node item.
          TItem   tItem;

          ///  Pointer to the left node (if any).
          TTreeNode*   ptLeftNode;

          ///  Pointer to the parent node (if any).
          TTreeNode*   ptParentNode;

          ///  Pointer to the first child node.
          alloc_smart_pointer   qtFirstChildNode;

          ///  Pointer to the right sibling node.
          alloc_smart_pointer   qtRightNode;


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Copies source node \a rkqtNODE onto this instance.
          *  @param rkqtNODE     Source node.
          *  @param rptNODE_HEAP The node heap.
          *  @return Node copy.
          */
          static alloc_smart_pointer _copy ( const alloc_smart_pointer& rkqtNODE     ,
                                             TTreeNode*&                rptNODE_HEAP );

          /**
          *  Copies source node \a rkqtNODE onto this instance.
          *  @param rqtTARGET_NODE Target node.
          *  @param rkqtNODE       Source node.
          *  @param rptNODE_HEAP   Node heap.
          *  @return Node copy.
          *  @exception TConstraintException if target \a rkqtNODE is NULL.
          *  @pre ( rqtTARGET_NODE != NULL )
          */
          static alloc_smart_pointer _copy ( alloc_smart_pointer&       rqtTARGET_NODE ,
                                             const alloc_smart_pointer& rkqtNODE       ,
                                             TTreeNode*&                rptNODE_HEAP   )
            throw (TConstraintException);

          /**
          *  Returns a copy of source sibling  nodes in \a rkqtNODE (including it).
          *  @param rkqtNODE       Source node to copy from.
          *  @param pktPARENT_NODE Parent node to attach new nodes.
          *  @param rptNODE_HEAP   Node heap.
          *  @return Target node.
          */
          static alloc_smart_pointer _copySiblings ( const alloc_smart_pointer& rkqtNODE       ,
                                                     const TTreeNode*           pktPARENT_NODE ,
                                                     TTreeNode*&                rptNODE_HEAP   );

          /**
          *  Creates a new node.
          *  @param rktITEM     Source Item.
          *  @param ptNODE_HEAP The node heap.
          *  @return Node copy.
          *  @post ( ptTARGET_NODE->tItem == rktITEM )
          */
          static TTreeNode* _create (const TItem& rktITEM, TTreeNode* ptNODE_HEAP)
          {
            TAllocator<TTreeNode>().construct (ptNODE_HEAP, rktITEM);
            return ptNODE_HEAP;
          }

          /**
          *  Links node \a qtRIGHT_NODE to node \a qtLEFT_NODE.
          *  @param qtLEFT_NODE  Left node.
          *  @param qtRIGHT_NODE Node to connect at right \a qtLEFT_NODE.
          *  @post ( qtLEFT_NODE->parent()    == qtRIGHT_NODE->parent() )
          *  @post ( qtLEFT_NODE->qtRightNode == qtRIGHT_NODE )
          *  @post ( qtRIGHT_NODE->ptLeftNode == qtLEFT_NODE )
          */
          static void _link ( alloc_smart_pointer& qtLEFT_NODE  ,
                              alloc_smart_pointer& qtRIGHT_NODE )
          {
            qtRIGHT_NODE->ptParentNode = qtLEFT_NODE->ptParentNode;
            qtRIGHT_NODE->ptLeftNode   = qtLEFT_NODE;
            qtLEFT_NODE->qtRightNode   = qtRIGHT_NODE;
          }

          /**
          *  Links node \a qtMIDDLE_NODE between \a qtLEFT_NODE and \a qtRIGHT_NODE
          *  updating parent node from \a qtLEFT_NODE.
          *  @param qtLEFT_NODE   Node at left.
          *  @param qtMIDDLE_NODE Node at middle.
          *  @param qtRIGHT_NODE  Node at right.
          *  @post ( ptMIDDLE_NODE->parent()  == ptLEFT_NODE->parent() )
          *  @post ( qtLEFT_NODE->qtRightNode == qtMIDDLE_NODE )
          *  @post ( qtRIGHT_NODE->ptLeftNode == qtMIDDLE_NODE )
          */
          static void _link ( alloc_smart_pointer& qtLEFT_NODE   ,
                              alloc_smart_pointer& qtMIDDLE_NODE ,
                              alloc_smart_pointer& qtRIGHT_NODE  )
          {
            qtMIDDLE_NODE->ptParentNode = qtLEFT_NODE->ptParentNode;
            qtMIDDLE_NODE->ptLeftNode   = qtLEFT_NODE;
            qtMIDDLE_NODE->qtRightNode  = qtRIGHT_NODE;
            qtLEFT_NODE->qtRightNode    = qtMIDDLE_NODE;
            qtRIGHT_NODE->ptLeftNode    = qtMIDDLE_NODE;
          }

          /**
          *  Unlinks  the  node \a ptTARGET_NODE  (and its child  nodes)  from  its
          *  parent node.
          *  @param ptTARGET_NODE Node to unlink.
          *  @post ( !ptTARGET_NODE->parent() )
          *  @post ( ptTARGET_NODE->qtRightNode == old (ptTARGET_NODE->ptLeftNode->qtRightNode) )
          */
          static void _unlink (TTreeNode* ptTARGET_NODE)
            throw (TConstraintException);


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance with item \a rktITEM.
          *  @param rktITEM Source item to contain.
          */
          TTreeNode (const TItem& rktITEM) :
            tItem            (rktITEM)     ,
            ptLeftNode       (NULL)        ,
            ptParentNode     (NULL)        ,
            qtFirstChildNode ()            ,
            qtRightNode      ()            {}

          /**
          *  Inserts node  \a rqtTARGET_NODE  (but not its right sibling nodes)
          *  as the last child of this node.
          *  @param rqtTARGET_NODE Target node to insert.
          *  @exception TConstraintException If \a qtTARGET_NODE is NULL.
          */
          void insert (alloc_smart_pointer& rqtTARGET_NODE)
            throw (TConstraintException);


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns the height of the tree.
          *  @return The height of the tree.
          */
          size_type height (void) const;

          /**
          *  Returns a pointer to the parent node (if any).
          *  @return A pointer to the parent node (if any).
          */
          const TTreeNode* parent (void) const
          {
            return ptParentNode;
          }

          /**
          *  Returns the number of nodes attached (including this).
          *  @return The number of nodes.
          */
          size_type size (void) const;

      };  // class TTreeNode

      /**
      *  General (unsorted) tree.
      *  @invariant ( size() >= 0 )
      */
      template < typename TItem                                                 ,
                 template <typename TItemA> class TAllocator = std::allocator   ,
                 typename TNode                              = TTreeNode<TItem> >
      class TGeneralTree;

      ///  Forwarded declaration.
      template <typename TItem, template <typename TItemA> class TAllocator = std::allocator>
      class TGeneralTreeSideIterator;

      ///  Forwarded declaration.
      template <typename TItem, template <typename TItemA> class TAllocator = std::allocator>
      class TGeneralTreePreIterator;

      ///  Pre-order iterator for trees.
      template <typename TItem, template <typename TItemA> class TAllocator>
      class TGeneralTreePreIterator :
        public std::iterator<std::forward_iterator_tag, TItem>
      {

          ///  Friend class.
          template < typename TItemB                              ,
                     template <typename TItemC> class TAllocatorB ,
                     typename TNodeB                              >
          friend class TGeneralTree;

          ///  Friend class.
          template <typename TItemB, template <typename TItemC> class TAllocatorB>
          friend class TGeneralTreeSideIterator;


        private:

          ///  Pointer to the current node.
          TTreeNode<TItem, TAllocator>*   ptCurrentNode;


        public:

          ///  Imports difference type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::difference_type
            difference_type;

          ///  Imports category tag type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::iterator_category
            iterator_category;

          ///  Imports pointer type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::pointer
            pointer;

          ///  Imports reference type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::reference
            reference;

          ///  Imports value type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::value_type
            value_type;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance pointing to 'past the end'.
          TGeneralTreePreIterator (void)                           :
            std::iterator<std::forward_iterator_tag, TItem> ()     ,
            ptCurrentNode                                   (NULL) {}

          /**
          *  Builds a new instance pointing to \a pktNODE.
          *  @param pktNODE Node to point to.
          */
          TGeneralTreePreIterator (const TTreeNode<TItem, TAllocator>* pktNODE)                                   :
            std::iterator<std::forward_iterator_tag, TItem> ()                                                    ,
            ptCurrentNode                                   (const_cast<TTreeNode<TItem, TAllocator>*> (pktNODE)) {}

          /**
          *  Builds a new instance pointing  to the same location that the iterator
          *  \a rktSIDE_ITERATOR points to.
          *  @param rktSIDE_ITERATOR Source side-order iterator.
          */
          TGeneralTreePreIterator (const TGeneralTreeSideIterator<TItem, TAllocator>& rktSIDE_ITERATOR) :
            std::iterator<std::forward_iterator_tag, TItem> ()                                          ,
            ptCurrentNode                                   (rktSIDE_ITERATOR.ptCurrentNode)            {}

          /**
          *  Post-increments the iterator position to the next position. The
          *  position follows a pre-order iteration.
          *  @return This instance.
          *  @exception TConstraintException If iterator is past the end.
          */
          TGeneralTreePreIterator& operator ++ (void)
            throw (TConstraintException);

          /**
          *  Post-increments the iterator position to the next position. The
          *  position follows a pre-order iteration.
          *  @return old (*this)
          */
          TGeneralTreePreIterator operator ++ (int)
          {
            TGeneralTreePreIterator   tCopy (*this);

            operator ++();
            return tCopy;
          }

          /**
          *  Copies the position pointed by \a rktPRE_ITERATOR.
          *  @param rktPRE_ITERATOR Source pre-order iterator.
          *  @return This instance.
          */
          TGeneralTreePreIterator&
          operator = (const TGeneralTreePreIterator<TItem, TAllocator>& rktPRE_ITERATOR)
          {
            ptCurrentNode = rktPRE_ITERATOR.ptCurrentNode;
            return *this;
          }

          /**
          *  Copies the position pointed by \a rktSIDE_ITERATOR.
          *  @param rktSIDE_ITERATOR Source right sibling iterator.
          *  @return This instance.
          */
          TGeneralTreePreIterator&
          operator = (const TGeneralTreeSideIterator<TItem, TAllocator>& rktSIDE_ITERATOR)
          {
            ptCurrentNode = rktSIDE_ITERATOR.ptCurrentNode;
            return *this;
          }


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns a tree pre-iterator pointing to the left child node.
          *  @return A tree pre-iterator pointing to the left child node.
          *  @pre ( !empty() )
          */
          TGeneralTreePreIterator leftChild (void) const
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return TGeneralTreePreIterator (ptCurrentNode->qtFirstChildNode.get());
          }

          pointer operator -> (void) const
            throw (TConstraintException)
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return &(ptCurrentNode->tItem);
          }

          bool operator == (const TGeneralTreePreIterator& rktITERATOR) const
          {
            return ( ptCurrentNode == rktITERATOR.ptCurrentNode );
          }

          /**
          *  Tests if iterator \a rktSIDE_ITERATOR points to the same position that
          *  this instance.
          *  @param rktSIDE_ITERATOR Source pre-order iterator.
          *  @return True if equal.
          */
          bool operator == (const TGeneralTreeSideIterator<TItem, TAllocator>& rktSIDE_ITERATOR) const
          {
            return ( ptCurrentNode == rktSIDE_ITERATOR.ptCurrentNode );
          }

          /**
          *  Tests  if  iterator \a rktSIDE_ITERATOR  does not  point  to the same
          *  position that this instance.
          *  @param rktSIDE_ITERATOR Source pre-order iterator.
          *  @return True if equal.
          */
          bool operator != (const TGeneralTreeSideIterator<TItem, TAllocator>& rktSIDE_ITERATOR) const
          {
            return ( ptCurrentNode != rktSIDE_ITERATOR.ptCurrentNode );
          }

          reference operator * (void) const
            throw (TConstraintException)
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return ptCurrentNode->tItem;
          }

          /**
          *  Returns a tree pre iterator pointing to the right sibling node.
          *  @return A tree pre iterator pointing to the right sibling node.
          *  @pre ( !empty() )
          */
          TGeneralTreePreIterator rightSibling (void) const
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return TGeneralTreePreIterator (ptCurrentNode->qtRightNode);
          }

      };  // class TGeneralTreePreIterator

      ///  Right sibling iterator for trees.
      template <typename TItem, template <typename TItemA> class TAllocator>
      class TGeneralTreeSideIterator :
        public std::iterator<std::forward_iterator_tag, TItem>
      {

          ///  Friend class.
          template < typename TItemB                              ,
                     template <typename TItemA> class TAllocatorB ,
                     typename TNodeB                              >
          friend class TGeneralTree;

          ///  Friend class.
          template <typename TItemB, template <typename TItemC> class TAllocatorB>
          friend class TGeneralTreePreIterator;


        public:

          ///  Imports difference type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::difference_type
            difference_type;

          ///  Imports category tag type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::iterator_category
            iterator_category;

          ///  Imports pointer type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::pointer
            pointer;

          ///  Imports reference type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::reference
            reference;

          ///  Imports value type.
          typedef
            typename std::iterator<std::forward_iterator_tag, TItem>::value_type
            value_type;


        private:

          ///  Pointer to the current node.
          TTreeNode<TItem, TAllocator>*   ptCurrentNode;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance pointing to 'past the end'.
          TGeneralTreeSideIterator (void)                          :
            std::iterator<std::forward_iterator_tag, TItem> ()     ,
            ptCurrentNode                                   (NULL) {}

          /**
          *  Builds a new instance pointing to \a pktNODE.
          *  @param pktNODE Node to point to.
          */
          TGeneralTreeSideIterator (const TTreeNode<TItem, TAllocator>* pktNODE)                                  :
            std::iterator<std::forward_iterator_tag, TItem> ()                                                    ,
            ptCurrentNode                                   (const_cast<TTreeNode<TItem, TAllocator>*> (pktNODE)) {}

          /**
          *  Builds a new instance pointing  to the same location that the iterator
          *  \a rktPRE_ITERATOR points to.
          *  @param rktPRE_ITERATOR Source pre-order iterator.
          */
          TGeneralTreeSideIterator (const TGeneralTreePreIterator<TItem, TAllocator>& rktPRE_ITERATOR) :
            std::iterator<std::forward_iterator_tag, TItem> ()                                         ,
            ptCurrentNode                                   (rktPRE_ITERATOR.ptCurrentNode)            {}

          /**
          *  Post-increments the iterator position to the next position. That
          *  position follows a right sibling node iteration.
          *  @return This instance.
          *  @exception TConstraintException If iterator is past the end.
          */
          TGeneralTreeSideIterator& operator ++ (void)
            throw (TConstraintException);

          /**
          *  Post-increments the iterator position to the next position. The
          *  position follows a right sibling node iteration.
          *  @return old (*this)
          */
          TGeneralTreeSideIterator operator ++ (int)
          {
            TGeneralTreeSideIterator   tCopy (*this);

            operator ++();
            return tCopy;
          }

          /**
          *  Copies the position pointed by \a rktSIDE_ITERATOR.
          *  @param rktSIDE_ITERATOR Source right sibling iterator.
          *  @return This instance.
          */
          TGeneralTreeSideIterator&
          operator = (const TGeneralTreeSideIterator<TItem, TAllocator>& rktSIDE_ITERATOR)
          {
            ptCurrentNode = rktSIDE_ITERATOR.ptCurrentNode;
            return *this;
          }

          /**
          *  Copies the position pointed by \a rktPRE_ITERATOR.
          *  @param rktPRE_ITERATOR Source pre-order iterator.
          *  @return This instance.
          */
          TGeneralTreeSideIterator& operator = (const TGeneralTreePreIterator<TItem, TAllocator>& rktPRE_ITERATOR)
          {
            ptCurrentNode = rktPRE_ITERATOR.ptCurrentNode;
            return *this;
          }


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns a tree side-iterator pointing to the left child node.
          *  @return A tree side-iterator pointing to the left child node.
          *  @pre ( !empty() )
          */
          TGeneralTreeSideIterator leftChild (void) const
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return TGeneralTreeSideIterator (ptCurrentNode->qtFirstChildNode.get());
          }

          pointer operator -> (void) const
            throw (TConstraintException)
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return &(ptCurrentNode->tItem);
          }

          bool operator == (const TGeneralTreeSideIterator& rktITERATOR) const
          {
            return ( ptCurrentNode == rktITERATOR.ptCurrentNode );
          }

          /**
          *  Tests if iterator \a rktPRE_ITERATOR points  to the same position that
          *  this instance.
          *  @param rktPRE_ITERATOR Source pre-order iterator.
          *  @return True if equal.
          */
          bool operator == (const TGeneralTreePreIterator<TItem, TAllocator>& rktPRE_ITERATOR) const
          {
            return ( ptCurrentNode == rktPRE_ITERATOR.ptCurrentNode );
          }

          /**
          *  Tests  if  iterator \a rktPRE_ITERATOR  does  not  point  to the  same
          *  position that this instance.
          *  @param rktPRE_ITERATOR Source pre-order iterator.
          *  @return True if equal.
          */
          bool operator != (const TGeneralTreePreIterator<TItem, TAllocator>& rktPRE_ITERATOR) const
          {
            return ( ptCurrentNode != rktPRE_ITERATOR.ptCurrentNode );
          }

          reference operator * (void) const
            throw (TConstraintException)
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return ptCurrentNode->tItem;
          }

          /**
          *  Returns a tree side iterator pointing to the right sibling node.
          *  @return A tree side iterator pointing to the right sibling node.
          *  @pre ( !empty() )
          */
          TGeneralTreeSideIterator rightSibling (void) const
          {
            if ( !ptCurrentNode )
            {
              throw TConstraintException ("empty node", __FILE__, __LINE__);
            }
            return TGeneralTreeSideIterator (ptCurrentNode->qtRightNode);
          }

      };  // class TGeneralTreeSideIterator

      /**
      *  General (unsorted) tree.
      *  @invariant ( size() >= 0 )
      */
      template < typename TItem                              ,
                 template <typename TItemA> class TAllocator ,
                 typename TNode                              >
      class TGeneralTree : private virtual TAllocator<TNode>
      {

        public:

          ///  Alloc-smart-pointer to node type.
          typedef
            typename TNode::alloc_smart_pointer
            QTNode;

          ///  Smart-pointer to general tree type.
          typedef
            memory::TAllocSmartPointer<TGeneralTree, TAllocator>
            QTTree;

          ///  Const iterator type.
          typedef
            TGeneralTreePreIterator<const TItem, TAllocator>
            const_iterator;

          ///  Const side iterator type.
          typedef
            TGeneralTreeSideIterator<const TItem, TAllocator>
            const_side_iterator;

          ///  Iterator type.
          typedef
            TGeneralTreePreIterator<TItem, TAllocator>
            iterator;

          ///  Iterator type.
          typedef
            TGeneralTreeSideIterator<TItem, TAllocator>
            side_iterator;

          ///  Size type.
          typedef
            typename TNode::size_type
            size_type;

          ///  Value type.
          typedef
            TItem
            value_type;


        private:

          ///  Smart-pointer to the root node (if any).
          QTNode   qtRootNode;


        private:

          /**
          *  Builds a new instance with root node pointing to \a rkqtNODE.
          *  @param rkqtNODE Source node.
          */
          TGeneralTree (const QTNode& rkqtNODE) :
            TAllocator<TNode> ()                ,
            qtRootNode        (rkqtNODE)        {}


        public:

          //
          //  C O N S T R U C T O R S
          //

          /**
          *  Builds a new instance of empty tree.
          *  @post ( this->empty() )
          */
          TGeneralTree (void)    :
            TAllocator<TNode> () ,
            qtRootNode        () {}

          /**
          *  Builds a new instance with  a  root node that is a copy of \a rktITEM.
          *  @param rktITEM Source root node.
          *  @post ( this->getRoot() == rktITEM )
          */
          TGeneralTree (const TItem& rktITEM)                          :
            TAllocator<TNode> ()                                       ,
            qtRootNode        (TNode::_create (rktITEM, allocate (1))) {}

          /**
          *  Builds a new instance copying from tree \a rktGENERAL_TREE.
          *  @param rktGENERAL_TREE Source tree.
          *  @post ( *this == rktGENERAL_TREE )
          */
          TGeneralTree (const TGeneralTree& rktGENERAL_TREE) :
            TAllocator<TNode> ()                             ,
            qtRootNode        ()
          {
            TNode*               ptNodeHeap;
            register size_type   zSourceTreeSize = rktGENERAL_TREE.size();

            if ( zSourceTreeSize )
            {
              ptNodeHeap = allocate (zSourceTreeSize);
              qtRootNode = TNode::_copy (rktGENERAL_TREE.qtRootNode, ptNodeHeap);
            }
          }

          ///  Destroys the instance. Complexity is like of clear().
          virtual ~TGeneralTree (void)
          {
            clear();
          }

          /**
          *  Clears all the sub-trees. Complexity is constant.
          *  @post ( empty() )
          */
          void clear (void)
          {
            qtRootNode = NULL;
          }

          /**
          *  Copies another tree onto this instance. Complexity is linear.
          *  @param rktGENERAL_TREE Source tree.
          *  @return This instance.
          */
          TGeneralTree& copy (const TGeneralTree& rktGENERAL_TREE)
          {
            if ( this != &rktGENERAL_TREE )
            {
              TNode*               ptNodeHeap;
              register size_type   zSourceTreeSize = rktGENERAL_TREE.size();

              if ( !qtRootNode )
              {
                if ( zSourceTreeSize )
                {
                  ptNodeHeap = allocate (zSourceTreeSize);
                  qtRootNode = TNode::_copy (rktGENERAL_TREE.qtRootNode, ptNodeHeap);
                }
              }
              else
              {
                if ( !zSourceTreeSize )
                {
                  qtRootNode = NULL;
                }
                else
                {
                  ptNodeHeap = ( zSourceTreeSize > 1 ) ? allocate (zSourceTreeSize - 1) : NULL;
                  TNode::_copy (qtRootNode, rktGENERAL_TREE.qtRootNode, ptNodeHeap);
                }
              }
            }
            return *this;
          }

          /**
          *  Erases sub-tree pointed by \a tPOSITION. Complexity is constant.
          *  @param tPOSITION Iterator pointing to the element to be erased.
          *  @return An  iterator that points to the element immediately  following
          *          \a tPOSITION  prior  to the element being  erased.  If no such
          *          element exists, end() is returned.
          *  @post ( old (size()) > size() )
          */
          template <typename TIterator>
          TIterator erase (TIterator tPOSITION)
          {
            TIterator   tNEXT_POSITION (tPOSITION);

            ++tNEXT_POSITION;
            TNode::_unlink (tPOSITION.ptCurrentNode);
            return tNEXT_POSITION;
          }

          /**
          *  Erases  sub-trees  from position \a tBEGIN  to \a tEND.  Complexity is
          *  linear.
          *  @param tBEGIN Iterator pointing to the first element to be erased.
          *  @param tEND   Iterator pointing past the last element to be erased.
          *  @return An  iterator that points to the element immediately  following
          *          \a tEND prior to the element being erased.  If no such element
          *          exists, end() is returned.
          */
          template <typename TIterator>
          TIterator erase (TIterator tBEGIN, TIterator tEND)
          {
            for (; ( tBEGIN != tEND ) ;)
            {
              //
              //  Post-increments the iterator so, firstly,  it gets the pointer to
              //  the node,  then it increments the  value of the iterator,  and at
              //  last, it unlinks the former node.
              //
              TNode::_unlink ((tBEGIN++).ptCurrentNode);
            }
            return tEND;
          }

          /**
          *  Inserts, as the last sub-tree, the tree \a rqtGENERAL_TREE. Complexity
          *  is like of TNode::insert().
          *  @param rqtGENERAL_TREE Smart-pointer to the tree to insert.
          *  @exception TConstraintException ( empty() )
          *  @pre  ( !empty() )
          *  @pre  ( !rqtGENERAL_TREE.empty() )
          *  @post ( size() > old (size()) )
          */
          void insert (QTTree& rqtGENERAL_TREE)
            throw (TConstraintException)
          {
            if ( !qtRootNode || !rqtGENERAL_TREE )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            qtRootNode->insert (rqtGENERAL_TREE->qtRootNode);
          }

          /**
          *  Inserts, as the last position, the item \a rktITEM. Complexity is like
          *  of TNode::insert().
          *  @param rktITEM Item to be inserted.
          *  @exception TConstraintException ( empty() )
          *  @pre  ( !empty() )
          *  @post ( size() > old (size()) )
          */
          void insert (const TItem& rktITEM)
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            else
            {
              QTNode   qtNewNode (TNode::_create (rktITEM, allocate (1)));

              qtRootNode->insert (qtNewNode);
            }
          }

          /**
          *  Inserts the  sub-tree  \a rqtGENERAL_TREE  at  position  \a tPOSITION.
          *  Complexity is like of TNode::insert().
          *  @param tPOSITION       Iterator  pointing at  the  position where  the
          *                         tree \a rqtGENERAL_TREE must be inserted.
          *  @param rqtGENERAL_TREE Smart-pointer to the tree to be inserted.
          *  @pre ( !empty() )
          *  @pre ( !rqtGENERAL_TREE.empty() )
          *  @post ( size() > old (size()) )
          *  @exception TConstraintException ( rqtGENERAL_TREEE.empty() )
          *  @exception TConstraintException ( tPOSITION == end() )
          */
          template <class TIterator>
          void insert ( TIterator tPOSITION       ,
                        QTTree&   rqtGENERAL_TREE )
            throw (TConstraintException)
          {
            if ( !qtRootNode || !rqtGENERAL_TREE )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            if ( !tPOSITION.ptCurrentNode )
            {
              throw TConstraintException ("null iterator", __FILE__, __LINE__);
            }
            tPOSITION.ptCurrentNode->insert (rqtGENERAL_TREE->qtRootNode);
          }

          /**
          *  Inserts  as the  last child node,  the item \a rktITEM.  Complexity is
          *  like of TNode::insert().
          *  @param tPOSITION
          *         Iterator at the position where the tree \a rqtGENERAL_TREE must
          *         be inserted.
          *  @param rktITEM
          *         Item to be inserted.
          *  @pre ( !empty() )
          *  @post ( size() > old (size()) )
          *  @exception TConstraintException ( rqtGENERAL_TREEE.empty() )
          *  @exception TConstraintException ( tPOSITION == end() )
          */
          template <class TIterator>
          void insert ( TIterator    tPOSITION ,
                        const TItem& rktITEM   )
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            else if ( !tPOSITION.ptCurrentNode )
            {
              throw TConstraintException ("null iterator", __FILE__, __LINE__);
            }
            else
            {
              QTNode   qtNewNode (TNode::_create (rktITEM, allocate (1)));

              tPOSITION.ptCurrentNode->insert (qtNewNode);
            }
          }

          /**
          *  Copies another tree onto this instance. Complexity is linear.
          *  @param rktGENERAL_TREE Source tree.
          *  @return This instance.
          *  @post ( parent() == old (parent()) )
          */
          TGeneralTree&
          operator = (const TGeneralTree& rktGENERAL_TREE)
          {
            return copy (rktGENERAL_TREE);
          }

          /**
          *  Sets the root node item to \a rktITEM. Complexity is constant.
          *  @param rktITEM Source item.
          *  @post ( size() >= old (size()) )
          */
          void setRoot (const TItem& rktITEM)
          {
            if ( !qtRootNode )
            {
              qtRootNode = TNode::_create (rktITEM, allocate (1));
            }
            else
            {
              qtRootNode->tItem = rktITEM;
            }
          }


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Returns the iterator at first sub-tree. Complexity is constant.
          *  @return The iterator at first sub-tree.
          */
          iterator begin (void)
          {
            return iterator (qtRootNode.get());
          }

          /**
          *  Returns the iterator at first sub-tree. Complexity is constant.
          *  @return The iterator at first sub-tree.
          */
          const_iterator begin (void) const
          {
            return const_iterator (qtRootNode.get());
          }

          /**
          *  Returns the iterator at first sub-tree. Complexity is constant.
          *  @return The iterator at first sub-tree.
          *  @pre ( !empty() )
          */
          side_iterator beginChild (void)
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return side_iterator (qtRootNode->qtFirstChildNode.get());
          }

          /**
          *  Returns the iterator at first sub-tree. Complexity is constant.
          *  @return The iterator at first sub-tree.
          *  @pre ( !empty() )
          */
          const_side_iterator beginChild (void) const
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return const_side_iterator (qtRootNode->qtFirstChildNode);
          }

          /**
          *  Returns true if tree is empty. Complexity is constant.
          *  @return True if tree is empty (has no root node).
          */
          bool empty (void) const
          {
            return ( !qtRootNode );
          }

          /**
          *  Returns the iterator at position past the last sub-tree. Complexity is
          *  constant.
          *  @return The iterator at position past the last sub-tree.
          */
          iterator end (void)
          {
            return iterator();
          }

          /**
          *  Returns the iterator at position past the last sub-tree. Complexity is
          *  constant.
          *  @return The iterator at position past the last sub-tree.
          */
          const_iterator end (void) const
          {
            return const_iterator();
          }

          /**
          *  Returns the iterator at position past the last sub-tree. Complexity is
          *  constant.
          *  @return The iterator at position past the last sub-tree.
          */
          side_iterator endChild (void)
          {
            return side_iterator();
          }

          /**
          *  Returns the iterator at position past the last sub-tree. Complexity is
          *  constant.
          *  @return The iterator at position past the last sub-tree.
          */
          const_side_iterator endChild (void) const
          {
            return const_side_iterator();
          }

          /**
          *  Returns the root node item. Complexity is constant.
          *  @return The root node item.
          *  @pre ( !empty() )
          */
          const TItem& getRoot (void) const
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return qtRootNode->tItem;
          }

          /**
          *  Returns the root node item. Complexity is constant.
          *  @return The root node item.
          *  @pre ( !empty() )
          */
          TItem& getRoot (void)
            throw (TConstraintException)
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return qtRootNode->tItem;
          }

          /**
          *  Returns the height of the tree. Complexity is like TNode::height().
          *  @return The height of the tree.
          */
          size_type height (void) const
          {
            return ( qtRootNode.isNull() ) ? 0 : qtRootNode->height();
          }

          /**
          *  Returns  a tree  pointing to the  left child  sub-tree.  Complexity is
          *  constant.
          *  @return A tree pointing to the left child sub-tree.
          *  @pre ( !empty() )
          */
          TGeneralTree leftChild (void) const
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return TGeneralTree (qtRootNode->qtFirstChildNode);
          }

          /**
          *  Returns maximum size of the tree.
          *  @return Maximum size of the tree.
          */
          size_type max_size (void) const
          {
            return size_type (-1);
          }

          /**
          *  Returns  a tree pointing to the right sibling sub-tree.  Complexity is
          *  constant.
          *  @return A tree pointing to the right sibling sub-tree.
          *  @pre ( !empty() )
          */
          TGeneralTree rightSibling (void) const
          {
            if ( !qtRootNode )
            {
              throw TConstraintException ("empty tree", __FILE__, __LINE__);
            }
            return TGeneralTree (qtRootNode->qtRightNode);
          }

          /**
          *  Returns the number of nodes. Complexity is like TNode::size().
          *  @return The number of nodes.
          */
          size_type size (void) const
          {
            register size_type   zSize = 0;

            if ( !qtRootNode.isNull() )
            {
              zSize = qtRootNode->size();
            }
            return zSize;
          }

          /**
          *  Swaps tree with \a rtTREE.
          *  @param rtTREE Tree to swap to.
          *  @post ( old (*this)   == rtTREE )
          *  @post ( old (*rtTREE) == *this )
          */
          void swap (TGeneralTree& rtTREE)
          {
            std::swap<QTNode> (qtRootNode, rtTREE.qtRootNode);
          }

      };  // class TGeneralTree


      //
      //  C O N S T R U C T O R S
      //

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline typename TTreeNode<TItem, TAllocator>::alloc_smart_pointer TTreeNode<TItem, TAllocator>::
      _copy (const alloc_smart_pointer& rkqtNODE, TTreeNode*& rptNODE_HEAP)
      {

        alloc_smart_pointer   qtNewNode;

        if ( rkqtNODE.get() )
        {
          //
          //  Takes the space for the node from the heap, and the construct the new
          //  instance on it. Finally, it copies the child nodes.
          //
          qtNewNode = rptNODE_HEAP;
          TAllocator<TTreeNode>().construct (rptNODE_HEAP++, rkqtNODE->tItem);
          qtNewNode->qtFirstChildNode =
            _copySiblings (rkqtNODE->qtFirstChildNode, qtNewNode.get(), rptNODE_HEAP);
        }
        return qtNewNode;

      }  // TTreeNode::_copy()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline typename TTreeNode<TItem, TAllocator>::alloc_smart_pointer TTreeNode<TItem, TAllocator>::
      _copy ( alloc_smart_pointer&       rqtTARGET_NODE ,
              const alloc_smart_pointer& rkqtNODE       ,
              TTreeNode*&                rptNODE_HEAP   )
        throw (TConstraintException)
      {

        if ( !rqtTARGET_NODE )
        {
          throw TConstraintException ("target node is null", __FILE__, __LINE__);
        }
        if ( !rkqtNODE.isNull() )
        {
          TTreeNode*   ptChildNode = rqtTARGET_NODE->qtFirstChildNode.get();

          //
          //  Detaches target node children from the parent node.
          //
          while ( ptChildNode )
          {
            ptChildNode->ptParentNode = NULL;
            ptChildNode               = ptChildNode->qtRightNode.get();
          }

          //
          //  Makes a copy of the source nodes.
          //
          rqtTARGET_NODE->tItem            = rkqtNODE->tItem;
          rqtTARGET_NODE->qtFirstChildNode =
            _copySiblings (rkqtNODE->qtFirstChildNode, rqtTARGET_NODE.get(), rptNODE_HEAP);
        }
        return rqtTARGET_NODE;

      }  // TTreeNode::_copy()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline typename TTreeNode<TItem, TAllocator>::alloc_smart_pointer TTreeNode<TItem, TAllocator>::
      _copySiblings ( const alloc_smart_pointer& rkqtNODE       ,
                      const TTreeNode*           pktPARENT_NODE ,
                      TTreeNode*&                rptNODE_HEAP   )
      {

        alloc_smart_pointer   qtFirstNode;

        if ( !rkqtNODE.isNull() )
        {
          alloc_smart_pointer   qtNewNode;
          alloc_smart_pointer   qtOldNode;
          alloc_smart_pointer   qtSourceNode (rkqtNODE);

          //
          //  Copies the first node of source sibling nodes.
          //
          qtFirstNode               = _copy (qtSourceNode, rptNODE_HEAP);
          qtFirstNode->ptParentNode = const_cast<TTreeNode*> (pktPARENT_NODE);
          qtSourceNode              = qtSourceNode->qtRightNode;

          //
          //  Copies rest of source siblings nodes.
          //
          for (qtOldNode = qtFirstNode; ( !qtSourceNode.isNull() ) ;)
          {
            //
            //  Creates  a new node and  attaches  to the parent node,  then it
            //  connects left and right newly created nodes.
            //
            qtNewNode               = _copy (qtSourceNode, rptNODE_HEAP);
            qtNewNode->ptParentNode = const_cast<TTreeNode*> (pktPARENT_NODE);
            qtNewNode->ptLeftNode   = qtOldNode.get();
            qtOldNode->qtRightNode  = qtNewNode;
            qtOldNode               = qtNewNode;
            qtSourceNode            = qtSourceNode->qtRightNode;
          }
        }
        return qtFirstNode;

      }  // TTreeNode::_copySiblings()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline void TTreeNode<TItem, TAllocator>::_unlink (TTreeNode* ptTARGET_NODE)
        throw (TConstraintException)
      {

        if ( !ptTARGET_NODE )
        {
          throw TConstraintException ("node is null", __FILE__, __LINE__);
        }
        if ( ptTARGET_NODE->ptParentNode )
        {
          ///  Holds the target node while it is operating over that node.
          alloc_smart_pointer   qtTargetNode;

          //
          //  Unlinks  from parent node. There is no need to  unlink  from  sibling
          //  nodes because if there is no parent, then there are no sibling nodes.
          //
          if ( ptTARGET_NODE->ptLeftNode )
          {
            //
            //  This is not the first sibling node.
            //
            //  Warning:  Cause  node  ptTARGET_NODE  is  holded  by  (possibly) an
            //            unique  smart-pointer,  (left node  smart-pointer to  its
            //            right sibling  node)  it must  point  to  another  smart-
            //            pointer instance pointing to the former node.
            //
            qtTargetNode = ptTARGET_NODE->ptLeftNode->qtRightNode;
            if ( !ptTARGET_NODE->qtRightNode )
            {
              //
              //  This is the last node.
              //
              ptTARGET_NODE->ptLeftNode->qtRightNode = NULL;
            }
            else
            {
              //
              //  This is an inner node.
              //
              ptTARGET_NODE->qtRightNode->ptLeftNode = ptTARGET_NODE->ptLeftNode;
              ptTARGET_NODE->ptLeftNode->qtRightNode = ptTARGET_NODE->qtRightNode;
            }
          }
          else
          {
            //
            //  Node ptTARGET_NODE is the first sibling node.
            //
            //  Warning:  Cause  node   ptTARGET_NODE  is   holded by (possibly) an
            //            unique smart-pointer,  (parent node  smart-pointer to its
            //            first child node) it must point  to another smart-pointer
            //            instance pointing to the former node.
            //
            qtTargetNode = ptTARGET_NODE->ptParentNode->qtFirstChildNode;
            if ( !ptTARGET_NODE->qtRightNode )
            {
              //
              //  This is the unique child node,  so it removes the pointer to this
              //  node.   If  there   is  no   other   smart-pointer   pointing  to
              //  ptTARGET_NODE, then it may  automatically destroyed before we can
              //  finish all operations over it.
              //
              ptTARGET_NODE->ptParentNode->qtFirstChildNode = NULL;
            }
            else
            {
              //
              //  There are sibling nodes,  so it  re-connects the parent node with
              //  its right node.
              //
              ptTARGET_NODE->qtRightNode->ptLeftNode        = NULL;
              ptTARGET_NODE->ptParentNode->qtFirstChildNode = ptTARGET_NODE->qtRightNode;
              ptTARGET_NODE->qtRightNode                    = NULL;
            }
          }
          //
          //  Erases pointer to parent node.
          //
          ptTARGET_NODE->ptParentNode = NULL;
        }

      }  // TTreeNode::_unlink()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline void TTreeNode<TItem, TAllocator>::
      insert (alloc_smart_pointer& rqtTARGET_NODE)
        throw (TConstraintException)
      {

        if ( !rqtTARGET_NODE )
        {
          throw TConstraintException ("empty node", __FILE__, __LINE__);
        }
        if ( !qtFirstChildNode )
        {
          //
          //  There is no child nodes.
          //
          qtFirstChildNode           = rqtTARGET_NODE;
          rqtTARGET_NODE->ptLeftNode = NULL;
        }
        else
        {
          alloc_smart_pointer   qtNode (qtFirstChildNode);

          //
          //  Iterates through the nodes to the last one.
          //
          while ( !qtNode->qtRightNode.isNull() )
          {
            qtNode = qtNode->qtRightNode;
          }
          qtNode->qtRightNode        = rqtTARGET_NODE;
          rqtTARGET_NODE->ptLeftNode = qtNode.get();
        }

        //
        //  Isolates  the  new node as the only node at last,  and  then makes this
        //  instance to 'adopt' that node.
        //
        rqtTARGET_NODE->qtRightNode  = NULL;
        rqtTARGET_NODE->ptParentNode = this;

      }  // TTreeNode::insert()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline TGeneralTreePreIterator<TItem, TAllocator>& TGeneralTreePreIterator<TItem, TAllocator>::
      operator ++ (void)
        throw (TConstraintException)
      {

        if ( !ptCurrentNode )
        {
          throw TConstraintException ("iterator past the end", __FILE__, __LINE__);
        }
        if ( !ptCurrentNode->qtFirstChildNode.isNull() )
        {
          //
          //  The node has at least one child, so take the that node.
          //
          ptCurrentNode = ptCurrentNode->qtFirstChildNode.get();
        }
        else
        {
          //
          //  Node has no child nodes.
          //
          if ( !ptCurrentNode->qtRightNode.isNull() )
          {
            //
            //  The node has at least one sibling node.
            //
            ptCurrentNode = ptCurrentNode->qtRightNode.get();
          }
          else
          {
            //
            //  The Node has no  sibling nodes,  so the searches  for the first
            //  parent node with sibling nodes.
            //
            while ( ptCurrentNode && !ptCurrentNode->qtRightNode )
            {
              ptCurrentNode = ptCurrentNode->ptParentNode;
            }
            if ( ptCurrentNode )
            {
              //
              //  There  has been found a node with  right sibling node,  so it
              //  points to this one.
              //
              ptCurrentNode = ptCurrentNode->qtRightNode.get();
            }
          }
        }
        return *this;

      }  // TGeneralTreePreIterator::operator ++()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline TGeneralTreeSideIterator<TItem, TAllocator>& TGeneralTreeSideIterator<TItem, TAllocator>::
      operator ++ (void)
        throw (TConstraintException)
      {

        if ( !ptCurrentNode )
        {
          throw TConstraintException ("empty node", __FILE__, __LINE__);
        }
        if ( !ptCurrentNode->qtRightNode )
        {
          //
          //  Node is empty.
          //
          ptCurrentNode = NULL;
        }
        else
        {
          //
          //  The node has at least one sibling node.
          //
          ptCurrentNode = ptCurrentNode->qtRightNode.get();
        }
        return *this;

      }  // TGeneralTreeSideIterator::operator ++()


      //
      //  S E L E C T O R S
      //

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline typename TTreeNode<TItem, TAllocator>::size_type TTreeNode<TItem, TAllocator>::
      height (void) const
      {

        alloc_smart_pointer   qtNode  = qtFirstChildNode;
        size_type             zHeight = 0;

        //
        //  Computes the maximum height.
        //
        while ( qtNode.get() )
        {
          zHeight = std::max<size_type> (zHeight, qtNode->height());
          qtNode  = qtNode->qtRightNode;
        }
        return (1 + zHeight);

      }  // TTreeNode::height()

      template <typename TItem, template <typename TItemA> class TAllocator>
      inline typename TTreeNode<TItem, TAllocator>::size_type TTreeNode<TItem, TAllocator>::
      size (void) const
      {

        alloc_smart_pointer   qtNode = qtFirstChildNode;
        register size_type    zSize  = 1;

        while ( !qtNode.isNull() )
        {
          zSize  += qtNode->size();
          qtNode  = qtNode->qtRightNode;
        }
        return zSize;

      }  // TTreeNode::size()

    }  // namespace collection

  }  // namespace util

}  // namespace mpcl


///  Name-space for STD.
namespace std
{

  using mpcl::util::collection::TGeneralTree;

  template < typename TItem                              ,
             template <typename TItemA> class TAllocator ,
             typename TNode                              >
  inline void swap ( TGeneralTree<TItem, TAllocator, TNode>& rtTREE1 , 
                     TGeneralTree<TItem, TAllocator, TNode>& rtTREE2 )
  {
    rtTREE1.swap (rtTREE2);
  }

}  // namespace std


#endif  // not _MPCL_UTIL_COLLECTION_GENERAL_TREE__
