/*
*  Name:      log.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Definitons for logging C++ programs
*  Date:      $Date: 2004-07-26 13:59:16 +0200 (lun, 26 jul 2004) $
*  Revision:  $Revision: 9795 $
*
*  Copyright (C) 2000-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_LOGGING_LOG__
#define _MPCL_UTIL_LOGGING_LOG__

#include <iomanip>
#include <iostream>
#include <vector>
#include "../../exceptions.hh"
#include "../../system/all.hh"
#include "../../text/string.hh"
#include "../general-functions.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    ///  Name-space for LOGGING.
    namespace logging
    {

      using system::ISystem;
      using text::TString;

      ///  Forwarded declaration.
      class TBaseLog;

      ///  Forwarded declaration.
      template <typename TChar, typename TTraits>
      class TLog;

      ///  Manipulator type.
      typedef TBaseLog& (*MManipulator) (TBaseLog&);

      ///  Writes a header telling what level is active.
      template <typename TChar, typename TTraits>
      TLog<TChar, TTraits>& header (TLog<TChar, TTraits>& rtBASE_LOG);

      //@{
      /**
      *  @name log_level
      *  Manipulators for setting the log level in \a TBaseLog.
      *  @see system::ISystem::eEmerg
      *  @see system::ISystem::eAlert
      *  @see system::ISystem::eCrit
      *  @see system::ISystem::eErr
      *  @see system::ISystem::eWarning
      *  @see system::ISystem::eNotice
      *  @see system::ISystem::eInfo
      *  @see system::ISystem::eDebug
      */
      TBaseLog& emerg   (TBaseLog& rtBASE_LOG);
      TBaseLog& alert   (TBaseLog& rtBASE_LOG);
      TBaseLog& crit    (TBaseLog& rtBASE_LOG);
      TBaseLog& err     (TBaseLog& rtBASE_LOG);
      TBaseLog& warning (TBaseLog& rtBASE_LOG);
      TBaseLog& notice  (TBaseLog& rtBASE_LOG);
      TBaseLog& info    (TBaseLog& rtBASE_LOG);
      TBaseLog& debug   (TBaseLog& rtBASE_LOG);
      //@}

      //@{
      /**
      *  @name log_facility
      *  Manipulators for setting the log facility in \a TBaseLog.
      */
      TBaseLog& authpriv (TBaseLog& rtBASE_LOG); ///<  @see system::ISystem::eAuthpriv.
      TBaseLog& cron (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eCron.
      TBaseLog& daemon (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eDaemon.
      TBaseLog& kern (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eKern.
      TBaseLog& local0 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal0.
      TBaseLog& local1 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal1.
      TBaseLog& local2 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal2.
      TBaseLog& local3 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal3.
      TBaseLog& local4 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal4.
      TBaseLog& local5 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal5.
      TBaseLog& local6 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal6.
      TBaseLog& local7 (TBaseLog& rtBASE_LOG);   ///<  @see system::ISystem::eLocal7.
      TBaseLog& lpr (TBaseLog& rtBASE_LOG);      ///<  @see system::ISystem::eLpr.
      TBaseLog& mail (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eMail.
      TBaseLog& news (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eNews.
      TBaseLog& user (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eUser.
      TBaseLog& uucp (TBaseLog& rtBASE_LOG);     ///<  @see system::ISystem::eUucp.
      //@}

      ///  Base class for logs.
      class TBaseLog
      {

        public:

          /**
          *  Imports all versions of system::ISystem::ELogLevel.
          *  @see system::ISystem#ELogLevel.
          */
          typedef
            ISystem::ELogFacility
            ELogFacility;

          /**
          *  Imports all versions of system::ISystem::ELogLevel.
          *  @see system::ISystem#ELogFacility.
          */
          typedef
            ISystem::ELogLevel
            ELogLevel;


        public:

          ///  Sets log facility.
          virtual void setFacility (ELogFacility eLOG_FACILITY) = 0;

          ///  Sets log level.
          virtual void setLevel (ELogLevel eLOG_LEVEL) = 0;


        public:

          //
          //  S E L E C T O R S
          //

          ///  Gets log level.
          virtual ELogLevel getLevel (void) const = 0;

          ///  Gets log facility.
          virtual ELogFacility getFacility (void) const = 0;

      };  // class TBaseLog


      /**
      *  Streambuf for log class \a TLog.
      *  @see system::ISystem#writeLogMessage().
      */
      template < typename TChar                             ,
                 typename TTraits = std::char_traits<TChar> >
      class TLogStreambuf                           :
        public TBaseLog                             ,
        public std::basic_streambuf<TChar, TTraits>
      {

        private:

          ///  Output stream vector type.
          typedef
            std::vector<std::basic_ostream<TChar, TTraits>*>
            TOstreamVector;


        private:

          ///  Log facility.
          ELogFacility   eLogFacility;

          ///  Log level.
          ELogLevel   eLogLevel;

          ///  Ostream vector for logging information also in this streams.
          TOstreamVector   tOstreamVector;

          ///  Line buffer.
          TString   yLine;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TLogStreambuf (void)                                      :
            TBaseLog                             ()                 ,
            std::basic_streambuf<TChar, TTraits> ()                 ,
            eLogFacility                         (ISystem::eUser)   ,
            eLogLevel                            (ISystem::eNotice) ,
            tOstreamVector                       ()                 ,
            yLine                                ()                 {}

          /**
          *  Enables  \a rtOSTREAM for  receiving  the  information written in this
          *  instance.
          *  @param rtOSTREAM Target std::basic_ostream.
          *  @return This instance.
          */
          void also (std::basic_ostream<TChar, TTraits>& rtOSTREAM)
          {
            tOstreamVector.push_back (&rtOSTREAM);
          }

          /**
          *  Disables \a rktOSTREAM for receiving  the  information written in this
          *  instance.
          *  @param rktOSTREAM Target std::basic_ostream.
          *  @return This instance.
          *  @exception TNotFoundException If \a rktOSTREAM is not included.
          */
          void erase (const std::basic_ostream<TChar, TTraits>& rktOSTREAM)
            throw (TNotFoundException);

          /**
          *  Appends the character designated by iCHAR to the  output sequence,  if
          *  possible.
          *  @return If possible, iCHAR if not, EOF.
          *  @see std::streambuf#overflow().
          */
          int overflow (int iCHAR = EOF);

          ///  Sets log facility.
          void setFacility (ELogFacility eLOG_FACILITY)
          {
            eLogFacility = eLOG_FACILITY;
          }

          ///  Sets log level.
          void setLevel (ELogLevel eLOG_LEVEL)
          {
            eLogLevel = eLOG_LEVEL;
          }

          /**
          *  Appends \a zSIZE characters  from \a pkcSTRING to the log.  Whenever a
          *  newline appears,  the string until that character is logged  and then,
          *  it start again with the rest of the string in \a pkcSTRING.
          *  @param pkcSTRING Source string.
          *  @param zSIZE     Size of the source string.
          *  @return The number of characters written.
          *  @see std::streambuf#xsputn().
          */
          std::streamsize xsputn (const char* pkcSTRING, std::streamsize zSIZE);


        public:

          //
          //  S E L E C T O R S
          //

          ///  Gets log level.
          ELogLevel getLevel (void) const
          {
            return eLogLevel;
          }

          ///  Gets log facility.
          ELogFacility getFacility (void) const
          {
            return eLogFacility;
          }

      };  // class TLogStreambuf

      /**
      *  Log stream class.  It uses the method of the class \a system::ISystem
      *  system::ISystem::writeLogEntry() and thus the operating system syslog
      *  facility.
      *  @see system::ISystem#writeLogEntry().
      */
      template < typename TChar                             ,
                 typename TTraits = std::char_traits<TChar> >
      class TLog                                  :
        public TBaseLog                           ,
        public std::basic_ostream<TChar, TTraits>
      {

        public:

          ///  Import this type.
          using TBaseLog::ELogLevel;


        private:

          ///  Streambuf for logging.
          TLogStreambuf<TChar, TTraits>   tLogStreambuf;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Builds a new instance.
          TLog (void)                                           :
            TBaseLog                           ()               ,
            std::basic_ostream<TChar, TTraits> (&tLogStreambuf) ,
            tLogStreambuf                      ()               {}

          /**
          *  Enables  \a rtOSTREAM for  receiving  the  information written in this
          *  instance.
          *  @param rtOSTREAM Target std::basic_ostream.
          *  @return This instance.
          */
          TLog& also (std::basic_ostream<TChar, TTraits>& rtOSTREAM)
          {
            tLogStreambuf.also (rtOSTREAM);
            return *this;
          }

          /**
          *  Writes \a tITEM  using  debug log  level  and user  log facility.  The
          *  currents log and facility levels do not change.
          *  @param tITEM Item to write on this log instance.
          */
          template <typename TItem>
          void debug (TItem tITEM)
          {
            ELogLevel      eCurrentLogLevel    = getLevel();
            ELogFacility   eCurrentLogFacility = getFacility();

            setLevel (ISystem::eDebug);
            setFacility (ISystem::eUser);
            *this << header << tITEM << std::endl;
            setLevel (eCurrentLogLevel);
            setFacility (eCurrentLogFacility);
          }

          /**
          *  Disables \a rktOSTREAM for  receiving  the information written in this
          *  instance.
          *  @param rktOSTREAM Target std::basic_ostream.
          *  @return This instance.
          */
          TLog& erase (const std::basic_ostream<TChar, TTraits>& rktOSTREAM)
          {
            tLogStreambuf.erase (rktOSTREAM);
            return *this;
          }

          /**
          *  Calls manipulator \a mMANIP_FUNCTION over this instance.
          *  @param mMANIP_FUNCTION Manipulator function.
          *  @return This instance.
          *  @see MManipulator.
          */
          TLog& operator << (MManipulator mMANIP_FUNCTION)
          {
            (*mMANIP_FUNCTION) (*this);
            return *this;
          }

          /**
          *  Calls manipulator \a mMANIP_FUNCTION over this instance.
          *  @param mMANIP_FUNCTION Manipulator function.
          *  @return This instance.
          *  @see MManipulator.
          */
          TLog& operator << (TLog& (*mMANIP_FUNCTION) (TLog&))
          {
            return (*mMANIP_FUNCTION) (*this);
          }

          /**
          *  Sets log facility.
          *  @param eLOG_FACILITY Log facility.
          *  @see system::ISystem#ELogFacility.
          */
          void setFacility (ELogFacility eLOG_FACILITY)
          {
            tLogStreambuf.setFacility (eLOG_FACILITY);
          }

          /**
          *  Sets log level.
          *  @param eLOG_LEVEL Log level.
          *  @see system::ISystem#ELogLevel.
          */
          void setLevel (ELogLevel eLOG_LEVEL)
          {
            tLogStreambuf.setLevel (eLOG_LEVEL);
          }


        public:

          //
          //  S E L E C T O R S
          //

          ///  Gets log level.
          ELogLevel getLevel (void) const
          {
            return tLogStreambuf.getLevel();
          }

          ///  Gets log facility.
          ELogFacility getFacility (void) const
          {
            return tLogStreambuf.getFacility();
          }

      };  // class TLog


      //
      //  E X P O R T E D   V A R I A B L E S
      //

      ///  System logger.
      extern TLog<char>   tLog;

    }  // namespace logging

  }  // namespace util

}  // namespace mpcl


//
//  E X P O R T E D   F U N C T I O N S
//

template <typename TChar, typename TTraits>
mpcl::util::logging::TLog<TChar, TTraits>& mpcl::util::logging::
header (TLog<TChar, TTraits>& rtBASE_LOG)
{

  using system::ISystem;

  switch (rtBASE_LOG.getLevel())
  {
    case ISystem::eEmerg:   rtBASE_LOG << "#EMERG: ";   break;
    case ISystem::eAlert:   rtBASE_LOG << "#ALERT: ";   break;
    case ISystem::eCrit:    rtBASE_LOG << "#CRIT: ";    break;
    case ISystem::eErr:     rtBASE_LOG << "#ERR: ";     break;
    case ISystem::eWarning: rtBASE_LOG << "#WARNING: "; break;
    case ISystem::eNotice:  rtBASE_LOG << "#NOTICE: ";  break;
    case ISystem::eInfo:    rtBASE_LOG << "#INFO: ";    break;
    case ISystem::eDebug:   rtBASE_LOG << "#DEBUG: ";   break;
  }
  return rtBASE_LOG;

}  // header()


//
//  C O N S T R U C T O R S
//

template <typename TChar, typename TTraits>
void mpcl::util::logging::TLogStreambuf<TChar, TTraits>::
erase (const std::basic_ostream<TChar, TTraits>& rktOSTREAM)
  throw (TNotFoundException)
{
 
  typename TOstreamVector::iterator   I;

  I = std::find (tOstreamVector.begin(), tOstreamVector.end(), &rktOSTREAM);
  if ( I == tOstreamVector.end() )
  {
    throw TNotFoundException ("stream not found", __FILE__, __LINE__);
  }
  else
  {
    tOstreamVector.erase (I);
  }

}  // TLogStreambuf::erase()


template <typename TChar, typename TTraits>
int mpcl::util::logging::TLogStreambuf<TChar, TTraits>::
overflow (int iCHAR)
{

  using std::for_each;
  using std::isprint;
  using std::string;
  using system::tSystem;
  using text::Format;

  switch (iCHAR)
  {
    case '\x0':
    case EOF:
    {
      break;
    }
    case '\n':
    {
      tSystem.writeLogEntry (yLine, eLogLevel, eLogFacility);
      for_each ( tOstreamVector.begin()                                              ,
                 tOstreamVector.end()                                                ,
                 MAppend<std::basic_ostream<TChar, TTraits>*, string> (yLine + '\n') );
      yLine.erase();
      break;
    }
    default:
    {
      if ( isprint (iCHAR) )
      {
        yLine += iCHAR;
      }
      else
      {
        yLine += Format ("\\x%x", (unsigned int) iCHAR);
      }
      break;
    }
  }
  return iCHAR;

}  // TLogStreambuf::overflow()


template <typename TChar, typename TTraits>
std::streamsize mpcl::util::logging::TLogStreambuf<TChar, TTraits>::
xsputn (const char* pkcSTRING, std::streamsize zSIZE)
{

  using std::basic_ostream;
  using std::for_each;
  using std::isprint;
  using std::streamsize;
  using std::string;
  using system::tSystem;
  using text::Format;

  if ( !pkcSTRING )
  {
    throw TConstraintException ("null string", __FILE__, __LINE__);
  }
  else
  {
    for (streamsize I = 0; ( ( I < zSIZE ) && *pkcSTRING ) ;++I, ++pkcSTRING)
    {
      if ( *pkcSTRING != '\n' )
      {
        if ( isprint (*pkcSTRING) )
        {
          yLine += *pkcSTRING;
        }
        else
        {
          yLine += Format ("\\x%x", (int) *pkcSTRING);
        }
      }
      else
      {
        tSystem.writeLogEntry (yLine, eLogLevel, eLogFacility);
        for_each ( tOstreamVector.begin()                                         ,
                   tOstreamVector.end()                                           ,
                   MAppend<basic_ostream<TChar, TTraits>*, string> (yLine + '\n') );
        yLine.erase();
      }
    }
  }
  return zSIZE;

}  // TLogStreambuf::xsputn()


#endif  // not _MPCL_UTIL_LOGGING_LOG__
