/*
*  Name:      option.hh
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Option for PREFS
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#ifndef _MPCL_UTIL_PREFS_OPTION__
#define _MPCL_UTIL_PREFS_OPTION__

#include <iostream>
#include <vector>
#include "../../io/output_streamable.hh"
#include "../../text/string.hh"
#include "defs.hh"


///  Name-space for MPCL.
namespace mpcl
{

  ///  Name-space for UTIL.
  namespace util
  {

    ///  Name-space for PREFS.
    namespace prefs
    {

      using io::IOutputStreamable;
      using text::TString;

      /**
      *  Option for configuration processor.
      *  @see TConfigProcessor.
      */
      class TOption : public IOutputStreamable<>
      {

        private:

          /// Default string for option if no other is provided.
          static const TString   _kyDefaultHelpString;

          ///  Long name. This is the one used in configuration file.
          TString   yName;

          ///  Short name.
          TString   yShortcut;

          ///  Default Value. It is valid only if there is no input value(s).
          TString   yDefaultValue;

          ///  Help string, if any.
          TString   yHelp;

          ///  Vector with values.
          std::vector<TString>   tValueVector;

          ///  True if its value has been passed from command-line.
          bool   gIsAtCmdLine;

          ///  True if it Can contain parameters.
          bool   gIsParameterAble;


        public:

          typedef
            std::vector<TString>::size_type
            size_type;


        protected:

          //
          //  S E L E C T O R S
          //
          void write (std::basic_ostream<char_type, traits_type>& rtTARGET_OSTREAM) const;


        public:

          //
          //  C O N S T R U C T O R S
          //

          ///  Build a new instance.
          TOption (void)                               :
            IOutputStreamable<> ()                     ,
            yName               ()                     ,
            yShortcut           ()                     ,
            yDefaultValue       ()                     ,
            yHelp               (_kyDefaultHelpString) ,
            tValueVector        ()                     ,
            gIsAtCmdLine        (false)                ,
            gIsParameterAble    (true)                 {}

          /**
          *  Build a new instance from another.
          *  @param rktSOURCE_OPTION Source instance.
          */
          TOption (const TOption& rktSOURCE_OPTION)                 :
            IOutputStreamable<> ()                                  ,
            yName               (rktSOURCE_OPTION.yName)            ,
            yShortcut           (rktSOURCE_OPTION.yShortcut)        ,
            yDefaultValue       (rktSOURCE_OPTION.yDefaultValue)    ,
            yHelp               (rktSOURCE_OPTION.yHelp)            ,
            tValueVector        (rktSOURCE_OPTION.tValueVector)     ,
            gIsAtCmdLine        (rktSOURCE_OPTION.gIsAtCmdLine)     ,
            gIsParameterAble    (rktSOURCE_OPTION.gIsParameterAble) {}

          /**
          *  Builds a new instance. Now this is an option with no value.
          *  @param rkyNAME     Source name.
          *  @param rkySHORTCUT Shortcut of the option.
          */
          TOption (const TString& rkyNAME, const TString& rkySHORTCUT) :
            IOutputStreamable<> ()                                     ,
            yName               (rkyNAME)                              ,
            yShortcut           (rkySHORTCUT)                          ,
            yDefaultValue       ()                                     ,
            yHelp               (_kyDefaultHelpString)                 ,
            tValueVector        ()                                     ,
            gIsAtCmdLine        (false)                                ,
            gIsParameterAble    (false)                                {}

          /**
          *  Builds a new instance. Now this is an option that can contain a value.
          *  @param rkyNAME          Source name.
          *  @param rkySHORTCUT      Shortcut of the option.
          *  @param rkyDEFAULT_VALUE Default value.
          */
          TOption ( const TString& rkyNAME          ,
                    const TString& rkySHORTCUT      ,
                    const TString& rkyDEFAULT_VALUE )  :
            IOutputStreamable<> ()                     ,
            yName               (rkyNAME)              ,
            yShortcut           (rkySHORTCUT)          ,
            yDefaultValue       (rkyDEFAULT_VALUE)     ,
            yHelp               (_kyDefaultHelpString) ,
            tValueVector        ()                     ,
            gIsAtCmdLine        (false)                ,
            gIsParameterAble    (true)                 {}

          ///  Destroys the instance.
          virtual ~TOption (void) {}

          /**
          *  Adds \a rkyVALUE as the last value of this instance.
          *  @param rkyVALUE Value to add.
          */
          void addValue (const TString& rkyVALUE)
          {
            if ( !gIsParameterAble )
            {
              throw TNotParameterAbleException (yName.c_str(), __FILE__, __LINE__);
            }
            else
            {
              tValueVector.push_back (rkyVALUE);
            }
          }

          ///  Copies \a rktSOURCE_OPTION onto this instance.
          TOption& operator = (const TOption& rktSOURCE_OPTION)
          {
            yName            = rktSOURCE_OPTION.yName;
            yShortcut        = rktSOURCE_OPTION.yShortcut;
            yDefaultValue    = rktSOURCE_OPTION.yDefaultValue;
            yHelp            = rktSOURCE_OPTION.yHelp;
            tValueVector     = rktSOURCE_OPTION.tValueVector;
            gIsAtCmdLine     = rktSOURCE_OPTION.gIsAtCmdLine;
            gIsParameterAble = rktSOURCE_OPTION.gIsParameterAble;
            return *this;
          }

          ///  Obliges this option to appear in command-line.
          void setCmdLinePresence (void)
          {
            gIsAtCmdLine = true;
          }

          /**
          *  Sets \a rkyVALUE as the help string associated to this option.
          *  @param rkyVALUE Help string.
          */
          void setUsageString (const TString& rkyVALUE)
          {
            yHelp = rkyVALUE;
          }

          /**
          *  Sets \a rkyVALUE as the unique value of this instance.
          *  @param rkyVALUE Value to assign.
          */
          void setValue (const TString& rkyVALUE)
          {
            tValueVector.clear();
            addValue (rkyVALUE);
          }


        public:

          //
          //  S E L E C T O R S
          //

          /**
          *  Get last value in value collection.
          *  @return Last value.
          */
          const TString& getValue (void) const
            throw (TNotParameterAbleException)
          {
            if ( !gIsParameterAble )
            {
              throw TNotParameterAbleException (yName.c_str(), __FILE__, __LINE__);
            }
            return ( tValueVector.empty() ) ? yDefaultValue : tValueVector.back();
          }

          /**
          *  Gets a vector with input values  (from command-line,  or from standard
          *  input).
          *  @return All values.
          */
          const std::vector<TString>& getValues (void) const
            throw (TNotParameterAbleException)
          {
            if ( !gIsParameterAble )
            {
              throw TNotParameterAbleException (yName.c_str(), __FILE__, __LINE__);
            }
            return tValueVector;
          }

          /**
          *  Returns true if it has \a rkySOURCE_VALUE as value.
          *  @param rkySOURCE_VALUE Source value.
          *  @return True if it has \a rkySOURCE_VALUE as value.
          */
          bool hasValue (const TString& rkySOURCE_VALUE) const;

          /**
          *  Get help string
          *  @return Help.
          */
          TString help (void) const
          {
            return yHelp;
          }

          /**
          *  Returns true if option is at command line.
          *  @return True if option is at command line.
          */
          bool isAtCmdLine (void) const
          {
            return gIsAtCmdLine;
          }

          /**
          *  Returns true if it has no value from any source.
          *  @return True if it has no value from any source.
          */
          bool isEmpty (void) const
          {
            return ( yName.empty() );
          }

          /**
          *  Returns true if option can have parameters.
          *  @return True if option can have parameters.
          */
          bool isParameterAble (void) const
          {
            return gIsParameterAble;
          }

          /**
          *  Get name.
          *  @return Name.
          */
          TString name (void) const
          {
            return yName;
          }

          /**
          *  Gets value at position \a zINDEX.
          *  @param zINDEX Index.
          *  @return Value at position \a zINDEX.
          */
          const TString& operator [] (size_type zINDEX) const
          {
            return tValueVector [zINDEX];
          }

          /**
          *  Gets value at position \a zINDEX.
          *  @param zINDEX Index.
          *  @return Value at position \a zINDEX.
          */
          TString& operator [] (size_type zINDEX)
          {
            return tValueVector [zINDEX];
          }

          /**
          *  Returns true if this instance and \a rktSOURCE_OPTION have equal value.
          *  This method is required by container classes.
          *  @param rktSOURCE_OPTION Source instance.
          *  @return True if this instance and \a rktSOURCE_OPTION have equal value.
          */
          bool operator == (const TOption& rktSOURCE_OPTION) const
          {
            return ( yName == rktSOURCE_OPTION.yName );
          }

          /**
          *  Get short name alias for command-line.
          *  @return Shortcut.
          */
          TString shortcut (void) const
          {
            return yShortcut;
          }

          /**
          *  Gets number of values.
          *  @return Value count.
          */
          size_type valueCount (void) const
          {
            return tValueVector.size();
          }

      };  // class TOption

    }  // namespace prefs

  }  // namespace util

}  // namespace mpcl


#endif  // not _MPCL_UTIL_PREFS_OPTION__
