/*
*  Name:      test-matcher.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for regular expression matcher
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <fstream>
#include <iostream>
#include <mpcl/test.h>
#include <mpcl/text/regex/matcher.hh>
#include <sstream>


using mpcl::text::regex::TMatcher;
using mpcl::text::TString;

  
//
//  L O C A L   F U N C T I O N S
//

static int _TestAgents (void)
{

  using std::size_t;

  TMatcher   tMatcher;
  TString    yAgent;
  TString    yVersion;
  TString    ySubVersion;
  size_t     zSize;

  TEST_INIT ("test for web browser agents identifiers");

  //
  //  Netscape Navigator 4.75.
  //
  yAgent = "Mozilla/4.75 [en] (X11; U; Linux 2.2.16-22 i686; Nav)";
  tMatcher.setInput (yAgent);
  zSize = tMatcher.scan ("Mozilla/%t [.]*<<EOF>>", &yVersion, NULL);
  TEST_NUMBERS (true, ( zSize > 0 ));
  TEST_STRINGS ("4.75", yVersion.c_str());

  //
  //  Microsoft Internet Explorer 5.0.
  //
  yAgent = "Mozilla/4.0 (compatible; MSIE 5.0; Windows 98; DigExt)";
  tMatcher.setInput (yAgent);
  zSize = tMatcher.scan ( "Mozilla/%t (compatible; MSIE %t; [.]*<<EOF>>" ,
                          &yVersion                                      ,
                          &ySubVersion                                   ,
                          NULL                                           );
  TEST_NUMBERS (true, ( zSize > 0 ));
  TEST_STRINGS ("4.0", yVersion.c_str());
  TEST_STRINGS ("5.0", ySubVersion.c_str());

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestAgents()


static int _TestCookies (void)
{

  TString    yName;
  TString    ySelector1;
  TString    ySelector2;
  TString    yValue;
  TString    ySource ( "cookie1=0aabn7z3,monday(009=1,700=90)" \
                       "0aabn7z3,friday(003=9,090=1)"          \
                       "0zzabbn7,january(009=1,030=7)"         );
  TMatcher   tMatcher (ySource);

  TEST_INIT ("tests the local stream mode for class 'TMatcher'");

  //
  //  Overwrites 'ySource'  to  be  sure  that 'tMatcher' is running over a copy of
  //  'ySource'.
  //
  ySource = "fake-value";
  tMatcher.scan ("%s=%t", &yName, &yValue, NULL);
  TEST_STRINGS ("cookie1", yName.c_str());
  TEST_STRINGS ( "0aabn7z3,monday(009=1,700=90)"
                 "0aabn7z3,friday(003=9,090=1)"
                 "0zzabbn7,january(009=1,030=7)" ,
                 yValue.c_str()                  );

  //
  //  Reads from last value in 'yValue'.
  //
  tMatcher.setInput (yValue);
  tMatcher.scan ("%s,%s(%t)", &ySelector1, &ySelector2, &yValue, NULL);
  TEST_STRINGS ("0aabn7z3",     ySelector1.c_str());
  TEST_STRINGS ("monday",       ySelector2.c_str());
  TEST_STRINGS ("009=1,700=90", yValue.c_str());
  
  tMatcher.scan ("%s,%s(%t)", &ySelector1, &ySelector2, &yValue, NULL);
  TEST_STRINGS ("0aabn7z3",    ySelector1.c_str());
  TEST_STRINGS ("friday",      ySelector2.c_str());
  TEST_STRINGS ("003=9,090=1", yValue.c_str());
  
  tMatcher.scan ("%s,%s(%t)", &ySelector1, &ySelector2, &yValue, NULL);
  TEST_STRINGS ("0zzabbn7",    ySelector1.c_str());
  TEST_STRINGS ("january",     ySelector2.c_str());
  TEST_STRINGS ("009=1,030=7", yValue.c_str());

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestCookies()


static int _TestSgmlComments (void)
{

  TMatcher   tMatcher;
  TString    ySource;
  TString    yText;

  TEST_INIT ("tests the SGML comments parsing");

  ySource =
    "<!doctype cdml public \"-//MPCL//DTD CDML 1.0//EN\">\n"
    "<cdml>\n"
    "  <!---->\n"
    "  <!-- sample text <sample symbols> -->\n"
    "  <!-- sample text <sample symbols> -->\n"
    "  <!---->\n"
    "  <instruct name=\"alter_table_statement\">\n"
    "    <clause name=\"body\">\n"
    "code code %variable% %variable% code\n"
    "    </clause>\n"
    "  </instruct>\n"
    "</cdml>\n";
  tMatcher.setInput (ySource);
  tMatcher.scan ("<!%t>[[:space:]]*<cdml>[[:space:]]*", &yText, NULL);
  TEST_STRINGS ("doctype cdml public \"-//MPCL//DTD CDML 1.0//EN\"", yText.c_str());

  tMatcher.scan ("<!--%t-->[[:space:]]*", &yText, NULL);
  TEST_NUMBERS (true, yText.empty());

  yText.erase();
  tMatcher.scan ("<!--%t-->[[:space:]]*", &yText, NULL);
  TEST_STRINGS (" sample text <sample symbols> ", yText.c_str());

  yText.erase();
  tMatcher.scan ("<!--%t-->[[:space:]]*", &yText, NULL);
  TEST_STRINGS (" sample text <sample symbols> ", yText.c_str());

  yText.erase();
  tMatcher.scan ("<!--%t-->[[:space:]]*", &yText, NULL);
  TEST_NUMBERS (true, yText.empty());

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestSgmlComments()


static int _TestTransactions (void)
{

  using std::basic_stringstream;

  int                             iAmmount;
  TString                         yCard;
  TString                         yCode;
  TString                         yMessage;
  TString                         yName;
  TString                         yResult;
  std::basic_stringstream<char>   tStringstream;
  TMatcher                        tMatcher;
  
  TEST_INIT ("another test");
  tStringstream << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=12:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=23123123:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=23:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0728199711135914458:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=another_mall:result=KO:reference=0728199711135914459:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n"
                << "commerce=big_mall:result=KO:reference=0728199711135914459:cost=3164:code=TRANSACTION DENIED:card=0609:message= Thanks for using this service:name=John Doe:\n";
  tMatcher.setInput (tStringstream);
  TEST_NUMBERS ( true                                                   ,
                 ( tMatcher.scan ( "[.]*commerce=big_mall:"
                                   "result=%s:"
                                   "reference=0728199711135914459:"
                                   "cost=%d:"
                                   "code=%t:"
                                   "card=%s:"
                                   "message=%t:"
                                   "name=%t:"                       ,
                                   &yResult                         ,
                                   &iAmmount                        ,
                                   &yCode                           ,
                                   &yCard                           ,
                                   &yMessage                        ,
                                   &yName                           ,
                                   NULL                             ) > 0 ) );
  TEST_STRINGS (yResult.c_str(), "KO");
  TEST_STRINGS (yName.c_str(),   "John Doe");
  TEST_STRINGS (yCode.c_str(),   "TRANSACTION DENIED");
  TEST_STRINGS (yCard.c_str(),   "0609");
  TEST_NUMBERS (iAmmount,        3164);

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // _TestTransactions()


///  Main.
int main (void)
{

  using std::ifstream;

  TEST_INIT ("tests for class 'TMatcher'");

  TString                          yTest;
  TString                          yTest2;
  TString                          yTest4 ("company=acme");
  ifstream                         tTestIfstream ("test-matcher.input");
  std::basic_istringstream<char>   tTestStringstream2 ("--option=value 1\n<clause>\nHello\n</clause>\n?");
  std::basic_istringstream<char>   tTestStringstream3 ("login=r12323oot&password=&s1=sex+machine");
  std::basic_istringstream<char>   tTestStringstream4 (yTest4.c_str());
  std::basic_istringstream<char>   tTestStringstream5 (" \n Hello \n\t");
  TMatcher                         tMatcher1 (tTestIfstream);
  TMatcher                         tMatcher2 (tTestStringstream2);
  TMatcher                         tMatcher3 (tTestStringstream3);
  TMatcher                         tMatcher4 (tTestStringstream4);
  TMatcher                         tMatcher5 (tTestStringstream5);

  tMatcher1.define ("{clause_tag}", "[[:space:]]*<clause>[[:space:]]*");
  tMatcher2.define ("{sep}+",       "[[:space:]]+");
  tMatcher2.define ("{sep}*",       "[[:space:]]*");

  //
  //  Input: file.
  //
  TEST_NUMBERS (true, ( tMatcher1.scan ("[[:space:]]+<clause>[[:space:]]*%s", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "Line1");
  TEST_NUMBERS (true, ( tMatcher1.scan ("[[:space:]]+<clause>[[:space:]]*%q", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "Line2 Line2");
  TEST_NUMBERS (true, ( tMatcher1.scan ("[[:space:]]+<clause>[[:space:]]*%t</clause>[[:space:]]*", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "Line3\nLine4\nLine5\n\n");
  TEST_NUMBERS (true, ( tMatcher1.scan ("{clause_tag}%t</clause>[[:space:]]*", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "Line6\nLine7\nLine8\n");
  TEST_NUMBERS (true, ( tMatcher1.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "login");
  TEST_STRINGS (yTest2.c_str(), "r12323oot");
  TEST_NUMBERS (true, ( tMatcher1.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "password");
  TEST_STRINGS (yTest2.c_str(), "");
  TEST_NUMBERS (false, ( tMatcher1.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  yTest  = "-";
  yTest2 = "-";
  TEST_NUMBERS (true, ( tMatcher1.scan ("%s=%t", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "s1");
  TEST_STRINGS (yTest2.c_str(), "sex+machine");

  //
  //  Input: string.
  //
  TEST_NUMBERS (true, ( tMatcher2.scan ("{sep}*--option{sep}*=%t\n", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "value 1");
  TEST_NUMBERS (true, ( tMatcher3.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "login");
  TEST_STRINGS (yTest2.c_str(), "r12323oot");
  TEST_NUMBERS (true, ( tMatcher3.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "password");
  TEST_STRINGS (yTest2.c_str(), "");
  TEST_NUMBERS (false, ( tMatcher3.scan ("%s=%t&", &yTest, &yTest2, NULL) > 0 ));
  yTest  = "";
  yTest2 = "";
  TEST_NUMBERS (true,           ( tMatcher3.scan ("%s=%t", &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "s1");
  TEST_STRINGS (yTest2.c_str(), "sex+machine");
  TEST_NUMBERS (false,          ( tMatcher4.scan ("%t=%t&", &yTest, &yTest2, NULL) > 0 ));
  TEST_NUMBERS (true,           ( tMatcher4.scan ("%t=%t",  &yTest, &yTest2, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "company");
  TEST_STRINGS (yTest2.c_str(), "acme");
  TEST_NUMBERS (true,           ( tMatcher5.scan ("[[:space:]]*%t[[:space:]]*<<EOF>>", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(),  "Hello");
  
  tMatcher2.setCaseSensitiveness (false);
  TEST_NUMBERS (true,          ( tMatcher2.scan ("{sep}*<CLAUSE>{sep}*%s{sep}*</ClAuSe>", &yTest, NULL) > 0 ));
  TEST_STRINGS (yTest.c_str(), "Hello");

  TEST_FUNCTION (_TestAgents());
  TEST_FUNCTION (_TestCookies());
  TEST_FUNCTION (_TestSgmlComments());
  TEST_FUNCTION (_TestTransactions());
  
  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
