/*
*  Name:      test-config_processor.cc
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Test for config processor
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 1994-2002  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

#include <iostream>
#include <mpcl/util/prefs/config_processor.hh>
#include <mpcl/test.h>


using mpcl::util::prefs::TConfigProcessor;
using mpcl::util::prefs::TException;
using mpcl::util::prefs::TNotParameterAbleException;
using mpcl::util::prefs::TOptionAlreadyDefinedException;


/**
*  Main.
*
*  \warning:  this test must be executed like this:
*
*    ./test-config_processor arg_0 arg_1 arg_2
*/
int main (int argc, const char* argv[])
{

  TEST_INIT ("tests for class 'TConfigProcessor'");

  TConfigProcessor   tConfigProcessor;

  //
  //  Define string constants names for options.
  // 
  tConfigProcessor.addOption ("first_option",  "a", "");
  tConfigProcessor.addOption ("second_option", "b", "");
  tConfigProcessor.addOption ("third_option",  "c", "");
  tConfigProcessor.addOption ("fourth_option", "d");
  tConfigProcessor.setCurrentNameSpace ("first_name_space");
  tConfigProcessor.addOption ("first_option",  "A", "");
  tConfigProcessor.addOption ("second_option", "B", "");
  tConfigProcessor.addOption ("third_option",  "C", "");
  tConfigProcessor.setCurrentNameSpace ("second_name_space");
  tConfigProcessor.addOption ("first_option",  "D", "");
  tConfigProcessor.addOption ("second_option", "E", "");
  tConfigProcessor.addOption ("third_option",  "F", "");

  //
  //  Checks for already defined options.
  //
  tConfigProcessor.setRootNameSpace();
  try
  {
    //
    //  Shortcut already defined.
    //
    tConfigProcessor.addOption ("fifth_option", "A");
    TEST_FAIL;
  }
  catch (const TOptionAlreadyDefinedException&)
  {
    TEST_PASS;
  }
  try
  {
    //
    //  Name already defined.
    //
    tConfigProcessor.addOption ("fourth_option", "d");
    TEST_FAIL;
  }
  catch (const TOptionAlreadyDefinedException&)
  {
    TEST_PASS;
  }

  tConfigProcessor.setConfigFileName ("test-config_processor.configuration");
  tConfigProcessor.setCommandLineParameters (argc, argv);
  try
  {
    tConfigProcessor.processOptions();
  }
  catch (const TException& rktEXCEPTION)
  {
    if ( !TEST_IN_SILENT_MODE )
    {
      fprintf ( hptTestTargetFile                          ,
                "#ERROR: %s:%d:%s; %s\n"                   ,
                rktEXCEPTION.fileName().c_str()            ,
                rktEXCEPTION.lineNumber()                  ,
                rktEXCEPTION.generalDescription().c_str()  ,
                rktEXCEPTION.specificDescription().c_str() );
    }
    TEST_FAIL;
  }
  try
  {
    TEST_STRINGS (tConfigProcessor ["fourth_option"].getValue().c_str(), "");  
    TEST_FAIL;
  }
  catch (const TNotParameterAbleException&)
  {
    TEST_PASS;
  }

  TEST_STRINGS ("1",   tConfigProcessor ["first_option"].getValue().c_str());
  TEST_STRINGS ("2",   tConfigProcessor ["second_option"].getValue().c_str());
  TEST_STRINGS ("3",   tConfigProcessor ["third_option"].getValue().c_str());
  TEST_NUMBERS (false, tConfigProcessor ["fourth_option"].isAtCmdLine());
  TEST_STRINGS ("1.1", tConfigProcessor ["first_name_space.first_option"].getValue().c_str());
  TEST_STRINGS ("1.2", tConfigProcessor ["first_name_space.second_option"].getValue().c_str());
  TEST_STRINGS ("1.3", tConfigProcessor ["first_name_space.third_option"].getValue().c_str());
  TEST_STRINGS ("2.1", tConfigProcessor ["second_name_space.first_option"].getValue().c_str());
  TEST_STRINGS ("2.2", tConfigProcessor ["second_name_space.second_option"].getValue().c_str());
  TEST_STRINGS ("2.3", tConfigProcessor ["second_name_space.third_option"].getValue().c_str());

  //
  //  Only make argument checking when not in 'silent_mode'.
  //
  if ( !TEST_IN_SILENT_MODE )
  {
    TEST_NUMBERS (true, tConfigProcessor.hasArguments());
    TEST_NUMBERS (true, ( tConfigProcessor.numberOfArguments() == 3 ));  
    if ( tConfigProcessor.numberOfArguments() >= 3 )
    {
      TEST_STRINGS ("arg_0", tConfigProcessor.argumentValue (0).c_str());
      TEST_STRINGS ("arg_1", tConfigProcessor.argumentValue (1).c_str());  
      TEST_STRINGS ("arg_2", tConfigProcessor.argumentValue (2).c_str());
    }
  }
  tConfigProcessor.saveConfigFile();

  TEST_MEMORY_STATUS;
  TEST_RETURN_CODE;

}  // main()
