/*
*  Name:      TMutable.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Mutable class
*  Date:      $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:  $Revision: 8535 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for BEHAVIORAL design patterns.
package org.mpcl.designpatterns.behavioral;

import java.util.Iterator;
import java.util.Vector;


/**
*  Mutable class.
*  This class represents a mutable object. It can be subclassed to represent an
*  object that the application wants to have mutated. A mutable object can have
*  one or more mutators.  A mutator may be any object that implements interface
*  \a IMutator. After a  mutable instance  needs  to be  mutated (changed),  an
*  application  calling the \a TMutable's \a notifyMutators method,  causes all
*  of its mutators to be notified of the need to  be mutated by a call to their
*  \a update method.  The order  in which  notifications will  be delivered  is
*  unspecified.  The default implementation provided in  the  \a TMutable class
*  will notify mutators  in the order in  which they  registered interest,  but
*  subclasses  may  change  this  order,  use  no  guaranteed  order,   deliver
*  notifications  on separate  threads,  or may  guarantee that  their subclass
*  follows this order, as they choose.
*
*  When an mutable object is newly created,  its set of mutators is empty.  Two
*  mutators  are considered  the same if and  only if the equals method returns
*  true for them.
*/
public class TMutable implements IMutable
{

  ///  True if the instance need to be mutated.
  private boolean   gNeedMutation;

  ///  Vector for mutators.
  protected Vector   tMutatorVector;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Indicates that this object has no longer need for mutation, or that it has
  *  already  notified all of its mutators of its  most recent change,  so that
  *  the \a needMutation method will now return false.
  */
  protected void clearNeedMutation()
  {
    gNeedMutation = false;
  }


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @post ( countMutators() == 0 )
  */
  public TMutable()
  {
    gNeedMutation  = false;
    tMutatorVector = new Vector();
  }

  /**
  *  Adds an mutator to the  set of mutators for this object,  provided that it
  *  is not the same as some mutator already in the set.
  *  @param tMUTATOR The mutator to be added.
  */
  public void addMutator (IMutator tMUTATOR)
  {
    tMutatorVector.add (tMUTATOR);
  }

  /**
  *  Deletes an mutator from the set of mutators of this object.
  *  @param tMUTATOR The mutator to be removed.
  */
  public void deleteMutator (IMutator tMUTATOR)
  {
    tMutatorVector.remove (tMUTATOR);
  }

  ///  Clears the mutator list so that this object no longer has any mutators.
  public void deleteMutators()
  {
    tMutatorVector.removeAllElements();
  }

  /**
  *  Marks this  mutable object  as having  been changed;  the  \a needMutation
  *  method will now return true.
  */
  public void setNeedMutation()
  {
    gNeedMutation = true;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the number of mutators of this mutable object.
  *  @return The number of mutators.
  */
  public int countMutators()
  {
    return tMutatorVector.size();
  }

  /**
  *  Tests if this object need to be mutated.
  *  @return True if this object need to be mutated.
  */
  public boolean needMutation()
  {
    return gNeedMutation;
  }

  /**
  *  If  this object  need  a mutation,  as  indicated  by  the \a needMutation
  *  method,   then  it   notifies   all  of   its  mutators   and   calls  the
  *  \a clearNeedMutation method  to indicate  that this  object has  no longer
  *  need for mutation.
  */
  public void notifyMutators()
  {
    notifyMutators (null);
  }

  /**
  *  If  this object  need  a mutation,  as  indicated  by  the \a needMutation
  *  method,   then  it   notifies   all  of   its  mutators   and   calls  the
  *  \a clearNeedMutation method  to indicate  that this  object has  no longer
  *  need for mutation.
  *  @param tARGUMENT An argument passed to the mutator update method.
  *  @see IMutator
  */
  public void notifyMutators (Object tARGUMENT)
  {

    if ( needMutation() )
    {
      IMutator   tMutator;
      Iterator   I = tMutatorVector.iterator();
      
      while ( I.hasNext() )
      {
        tMutator = (IMutator) I.next();
        tMutator.update (this, tARGUMENT);
      }
      clearNeedMutation();
    }

  }  // notifyMutators()

}  // class TMutable
