/*
*  Name:      TAbstractModelToContainerAdapter.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Abstract model to Swing container adapter class
*  Date:      $Date: 2004-11-24 16:41:53 +0100 (mié, 24 nov 2004) $
*  Revision:  $Revision: 10491 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import javax.swing.ComboBoxModel;
import javax.swing.ListModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import org.mpcl.designpatterns.mvc.model.TTuple;


/**
*  Model to container  adapter class.  It  adapts a  model to be displayed in a
*  container like \a TModelComboBox or \a TModelList.  To do so,  it implements
*  the 'observer' pattern  for listening to  the changes happened in the  model
*  being displayed.  It also implements the 'observable'  pattern for notifying
*  to  its  own 'observers'  for changes  in  this  instance which  need to  be
*  redisplayed.
*  @see javax.swing.ComboBoxModel
*  @see javax.swing.ListModel
*/
public abstract class TAbstractModelToContainerAdapter
  extends    Observable
  implements ComboBoxModel, ListModel, Observer
{

  ///  Selected index.
  protected int   iSelectedIndex;

  ///  The event listener list.
  protected EventListenerList   tEventListenerList;

  ///  Instance array.
  protected List   tTupleList;


  //
  //  C O N S T R U C T O R S
  //

  ///  Builds a new instance.
  public TAbstractModelToContainerAdapter()
  {
    super();
    iSelectedIndex     = 0;
    tEventListenerList = new EventListenerList();
    tTupleList         = new ArrayList();
  }

  /**
  *  Builds a new instance. It attaches to the object \a tOBSERVABLE.
  *  @param tOBSERVABLE Subject instance to be observed (usually a model instance).
  */
  public TAbstractModelToContainerAdapter (Observable tOBSERVABLE)
  {
    super();
    iSelectedIndex     = 0;
    tEventListenerList = new EventListenerList();
    tTupleList         = new ArrayList();
    tOBSERVABLE.addObserver (this);
  }

  /**
  *  Adds a listener  to the list  that is notified  each time a  change to the
  *  data model occurs.
  *  @param tLIST_DATA_LISTENER The \a ListDataListener object to be added.
  *  @see javax.swing.ListModel#addListDataListener (ListDataListener)
  */  
  public void addListDataListener (ListDataListener tLIST_DATA_LISTENER)
  {
    tEventListenerList.add (ListDataListener.class, tLIST_DATA_LISTENER);
  }

  /**
  *  Removes a listener from the list that's notified each time a change to the
  *  data model occurs.
  *  @param tLIST_DATA_LISTENER The \a ListDataListener object to be removed.
  *  @see javax.swing.ListModel#removeListDataListener (ListDataListener)
  */  
  public void removeListDataListener (ListDataListener tLIST_DATA_LISTENER)
  {
    tEventListenerList.remove(ListDataListener.class, tLIST_DATA_LISTENER);
  }

  /**
  *  Sets the value of the selected item. The selected item may be null.
  *  @param tOBJECT The combo box value or null for no selection.
  *  @see javax.swing.ComboBoxModel#setSelectedItem (Object)
  */
  public void setSelectedItem (Object tOBJECT)
  {
    int   iSize = getSize();

    iSelectedIndex = -1;
    if ( tOBJECT != null )
    {
      for (int I = 0; ( I < iSize ) ;++I)
      {
        if ( tOBJECT.equals (getTextAt (I)) )
        {
          iSelectedIndex = I;
          break;
        }
      }
    }
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Subclasses must call  this method  after one or  more elements of the list
  *  change.  The  changed  elements  are  specified  by  the  closed  interval
  *  \a iINDEX_0  and  \a iINDEX_1  (the end  points are  included).  Note that
  *  \a iINDEX_0 need not be less than or equal to \a iINDEX_1.
  *  @param tSOURCE  The \a ListModel that changed, typically "this".
  *  @param iINDEX_0 One end of the changed interval.
  *  @param iINDEX_1 The other end of the changed interval.
  */
  protected void fireContentsChanged (Object tSOURCE, int iINDEX_0, int iINDEX_1)
  {
    Object[]        atListenerPair = tEventListenerList.getListenerList();
    ListDataEvent   tListDataEvent = null;
    
    for (int I = atListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( atListenerPair [I] == ListDataListener.class )
      {
        if ( tListDataEvent == null )
        {
          tListDataEvent = new ListDataEvent(tSOURCE, ListDataEvent.CONTENTS_CHANGED, iINDEX_0, iINDEX_1);
        }
        ((ListDataListener) atListenerPair [I + 1]).contentsChanged (tListDataEvent);
      }	       
    }
  }

  /**
  *  Subclasses must call this method  after one or  more elements are added to
  *  the  model.  The  new  elements  are  specified  by  the  closed  interval
  *  \a iINDEX_0  and  \a iINDEX_1 (the end  points are  included).  Note  that
  *   \a iINDEX_0 need not be less than or equal to \a iINDEX_1.
  *  @param tSOURCE  The \a ListModel that changed, typically "this".
  *  @param iINDEX_0 One end of the new interval.
  *  @param iINDEX_1 The other end of the new interval.
  */
  protected void fireIntervalAdded (Object tSOURCE, int iINDEX_0, int iINDEX_1)
  {
    Object[]        atListenerPair = tEventListenerList.getListenerList();
    ListDataEvent   tListDataEvent = null;
    
    for (int I = atListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( atListenerPair [I] == ListDataListener.class )
      {
        if ( tListDataEvent == null )
        {
          tListDataEvent = new ListDataEvent(tSOURCE, ListDataEvent.INTERVAL_ADDED, iINDEX_0, iINDEX_1);
        }
        ((ListDataListener) atListenerPair [I + 1]).intervalAdded (tListDataEvent);
      }	       
    }
  }

  /**
  *  Subclasses must call  this method  after one or  more elements are removed
  *  from the model.  \a iINDEX_0  and  \a iINDEX_1 are  the end  points of the
  *  interval that is being  removed  (the end points are included).  Note that
  *   \a iINDEX_0 need not be less than or equal to \a iINDEX_1.
  *  @param tSOURCE  The \a ListModel that changed, typically "this".
  *  @param iINDEX_0 One end of the removed interval.
  *  @param iINDEX_1 The other end of the removed interval.
  */
  protected void fireIntervalRemoved (Object tSOURCE, int iINDEX_0, int iINDEX_1)
  {
    Object[]        atListenerPair = tEventListenerList.getListenerList();
    ListDataEvent   tListDataEvent = null;
    
    for (int I = atListenerPair.length - 2; ( I >= 0 ) ;I -= 2)
    {
      if ( atListenerPair [I] == ListDataListener.class )
      {
        if ( tListDataEvent == null )
        {
          tListDataEvent = new ListDataEvent (tSOURCE, ListDataEvent.INTERVAL_REMOVED, iINDEX_0, iINDEX_1);
        }
        ((ListDataListener) atListenerPair [I + 1]).intervalRemoved (tListDataEvent);
      }	       
    }
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the code of the model instance at index \a iINDEX.
  *  @return The code of the model instance.
  */
  public abstract Integer getCodeAt (int iINDEX);

  /**
  *  Returns the code of the model instance at index \a iINDEX.
  *  @return The code of the model instance.
  *  @see javax.swing.ListModel#getElementAt (int)
  */
  public Object getElementAt (int iINDEX)
  {
    return getTextAt (iINDEX);
  }

  /**
  *  Returns the selected item index.
  *  @return The selected item index or -1 if there is no selection.
  */
  public int getSelectedIndex()
  {
    return iSelectedIndex;
  }

  /**
  *  Returns the selected item.
  *  @return The selected item or null if there is no selection.
  *  @see javax.swing.ComboBoxModel#getSelectedItem()
  */
  public Object getSelectedItem()
  {
    Object   tObject = null;

    if ( getSelectedIndex() >= 0 )
    {
      tObject = getElementAt (getSelectedIndex());
    }
    return tObject;
  }

  /**
  *  Returns the number of model instances.
  *  @return The number of model instances.
  *  @see javax.swing.ListModel#getSize()
  */
  public int getSize()
  {
    return tTupleList.size();
  }

  /**
  *  Returns the description of the model instance at index \a iINDEX.  This is
  *  the text shown combo-box entries.
  *  @return The description of the model instance.
  */
  public abstract String getTextAt (int iINDEX);

  /**
  *  Returns the tool-tip text of the model  instance at index \a iINDEX.  This
  *  is the tool-tip text shown when that model instance is selected.
  *  @return The description of the model instance.
  */
  public String getToolTipTextAt (int iINDEX)
  {
    return null;
  }

  /**
  *  Returns the tuple of the model at index \a iINDEX.
  *  @return The tuple of the model.
  */
  public TTuple getTupleAt (int iINDEX)
  {
    TTuple   tTuple = null;

    if ( ( iINDEX >= 0 ) && ( iINDEX < tTupleList.size() ) )
    {
      tTuple = (TTuple) tTupleList.get (iINDEX);
    }
    return tTuple;
  }

}  // class TAbstractModelToContainerAdapter
