/*
*  Name:      TModelComboBox.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Combo-box for model instances
*  Date:      $Date: 2004-11-24 06:04:57 +0100 (mié, 24 nov 2004) $
*  Revision:  $Revision: 10489 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for NUI (Native User Interface).
package org.mpcl.nui;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.JComboBox;
import org.mpcl.designpatterns.mvc.model.TTuple;


/**
*  Combo-box for selecting an  instance from  all the instances hold by a model
*  adapter.
*/
public class TModelComboBox extends JComboBox implements ItemListener
{

  /**
  *  True if it must show model tool-tips.
  *  @see TAbstractModelToContainerAdapter#getToolTipText()
  */
  private boolean   gShowModelToolTips;

  ///  Model adapter.
  private TAbstractModelToContainerAdapter   tModelToContainerAdapter;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tMODEL_TO_CONTAINER_ADAPTER Model to container adapter instance.
  *  @param gSHOW_MODEL_TOOL_TIPS       True if it must show model tool tips.
  */
  public TModelComboBox ( TAbstractModelToContainerAdapter tMODEL_TO_CONTAINER_ADAPTER ,
                          boolean                          gSHOW_MODEL_TOOL_TIPS       )
  {
    super (tMODEL_TO_CONTAINER_ADAPTER);
    gShowModelToolTips       = gSHOW_MODEL_TOOL_TIPS;
    tModelToContainerAdapter = tMODEL_TO_CONTAINER_ADAPTER;
    addItemListener (this);
  }

  /**
  *  Invoked when an item has been selected or deselected by the user. The
  *  code written for this method performs the operations that need to occur
  *  when an item is selected (or deselected).
  *  @param tITEM_EVENT The item event.
  *  @see javax.swing.event.ItemListener#itemStateChanged (ItemEvent)
  */
  public void itemStateChanged (ItemEvent tITEM_EVENT)
  {
    if ( getShowModelToolTips() )
    {
      setToolTipText (tModelToContainerAdapter.getToolTipTextAt (getSelectedIndex()));
    }
  }

  /**
  *  Selects the instance with code \a tCODE  (it is not the item index).  That
  *  code can be null.
  *  @param tCODE Code of the instance.
  */
  public void setSelectedCode (Integer tCODE)
  {

    Integer   tCurrentCode;

    if ( tCODE == null )
    {
      setSelectedIndex (-1);
    }
    else
    {
      for (int I = 0; ( I < tModelToContainerAdapter.getSize() ) ;++I)
      {
        tCurrentCode = tModelToContainerAdapter.getCodeAt (I);
        if ( tCODE.equals (tCurrentCode) )
        {
          setSelectedIndex (I);
          break;
        }
      }
    }

  }  // setSelectedCode()

  /**
  *  Selects the instance with code \a iCODE (it is not the item index).
  *  @param iCODE Code of the instance.
  */
  public void setSelectedCode (int iCODE)
  {
    setSelectedCode (new Integer (iCODE));
  }

  /**
  *  Sets the gShowModelToolTips value.
  *  @param gSHOW_MODEL_TOOL_TIPS True if it must show model tool tips.
  */
  public void setShowModelToolTips (boolean gSHOW_MODEL_TOOL_TIPS)
  {
    gShowModelToolTips = gSHOW_MODEL_TOOL_TIPS;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the  current selected  instance  with code \a iCODE (it is not the
  *  item index).
  *  @return The code of the current selected instance.
  */
  public Integer getSelectedCode()
  {
    Integer   tSelectedCode = null;

    if ( getSelectedIndex() >= 0 )
    {
      tSelectedCode = tModelToContainerAdapter.getCodeAt (getSelectedIndex());
    }
    return tSelectedCode;
  }

  /**
  *  Returns the index of the current selected instance.
  *  @return The index of the current selected instance.
  */
  public int getSelectedIndex()
  {
    int   iSelectedIndex = -1;

    if ( tModelToContainerAdapter != null )
    {
      iSelectedIndex = tModelToContainerAdapter.getSelectedIndex();
    }
    return iSelectedIndex;
  }

  /**
  *  Returns the index of the current selected instance.
  *  @return The index of the current selected instance.
  */
  public TTuple getSelectedTuple()
  {
    int      iSelectedIndex = -1;
    TTuple   tTuple         = null;

    if ( tModelToContainerAdapter != null )
    {
      iSelectedIndex = tModelToContainerAdapter.getSelectedIndex();
      tTuple         = tModelToContainerAdapter.getTupleAt (iSelectedIndex);
    }
    return tTuple;
  }

  /**
  *  Returns true if it must set the tool-tip from selected model object.
  *  @return True if it must set the tool-tip from selected model object.
  */
  public boolean getShowModelToolTips()
  {
    return gShowModelToolTips;
  }

}  // class TModelComboBox
