/*
*  Name:      TTextAreaCellEditor.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Table cell editor using a JTextArea in a popup dialog
*  Date:      $Date: 2004-09-13 04:10:22 +0200 (lun, 13 sep 2004) $
*  Revision:  $Revision: 10107 $
*
*  Copyright (C) 2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TABLE (table support classes).
package org.mpcl.nui.table;

import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableCellEditor;


/**
*  Table cell editor that uses a \a JTextArea (inside of a \a JScrollPane) in a
*  \a JDialog for editing.  This way,  the text area will have the user defined
*  size for the editor.
*  @see javax.swing.JTextArea
*  @see javax.swing.JScrollPane
*  @see javax.swing.JScrollPane
*/
public class TTextAreaCellEditor
  extends    AbstractCellEditor
  implements FocusListener, KeyListener, TableCellEditor
{

  ///  The number of clicks to start editing.
  private int   iClickCountToStart;

  ///  The popup dialog holding the text area component.
  private JDialog   tPopupDialog;

  ///  The text area.
  private JTextArea   tPopupTextArea;

  ///  The text editor (used only to initially show the data).
  private JTextArea   tTextArea;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tOWNER_FRAME The frame owner.
  *  @param iROWS        The number of rows of the pop-up text area.
  *  @param iCOLUMNS     The number of columns of the pop-up text area.
  */
  public TTextAreaCellEditor (JFrame tOWNER_FRAME, int iROWS, int iCOLUMNS)
  {
    this (tOWNER_FRAME, iROWS, iCOLUMNS, null);
  }

  /**
  *  Builds a new instance.
  *  @param tOWNER_FRAME The frame owner.
  *  @param iROWS        The number of rows of the pop-up text area.
  *  @param iCOLUMNS     The number of columns of the pop-up text area.
  *  @param yTOOL_TIP    The tool-tip message.
  */
  public TTextAreaCellEditor ( JFrame       tOWNER_FRAME ,
                               int          iROWS        ,
                               int          iCOLUMNS     ,
                               final String yTOOL_TIP    )
  {

    JScrollPane   tScrollPane;

    iClickCountToStart = 2;
    tPopupDialog       = new JDialog (tOWNER_FRAME, true);
    tPopupTextArea     = new JTextArea (iROWS, iCOLUMNS);
    tTextArea          = new JTextArea (1, 8);
    tScrollPane        = new JScrollPane (tPopupTextArea);

    //
    //  Layouts the components.
    //  It makes the text area not editable to avoid input, even not having the
    //  focus. It makess 
    //
    tPopupTextArea.setToolTipText (yTOOL_TIP);
    tPopupTextArea.addKeyListener (this);
    tScrollPane.setBorder (BorderFactory.createEtchedBorder (EtchedBorder.RAISED));
    tTextArea.setEditable (false);
    tTextArea.addFocusListener (this);
    tPopupDialog.setUndecorated (true);
    tPopupDialog.getContentPane().add (tScrollPane);
    tPopupDialog.pack();

  }  // TTextAreaCellEditor()

  /**
  *  Invoked when a component gains the keyboard focus.
  *  @param tFOCUS_EVENT The focus event.
  *  @see java.awt.event.FocusListener#focusGained()
  */
  public void focusGained (FocusEvent tFOCUS_EVENT)
  {
    tPopupTextArea.setText (tTextArea.getText());
    tPopupDialog.setLocation (tTextArea.getLocationOnScreen());
    tTextArea.setVisible    (false);
    tPopupDialog.setVisible (true);
  }

  /**
  *  Invoked when a component loses the keyboard focus.
  *  @param tFOCUS_EVENT The focus event.
  *  @see java.awt.event.FocusListener#focusLost()
  */
  public void focusLost (FocusEvent tFOCUS_EVENT) {}

  /**
  *  Invoked when a key has been  pressed.  It handles  the  key events for the
  *  popup text area.
  *  @param tKEY_EVENT The key event.
  *  @see java.awt.event.KeyListener#keyPressed()
  */
  public void keyPressed (KeyEvent tKEY_EVENT)
  {

    //
    //  Handles the popup text area key events.
    //
    if ( tKEY_EVENT.getKeyCode() == KeyEvent.VK_ESCAPE )
    {
      //
      //  When the user types ESCAPE,  it cancels the  editor changes and hides
      //  the dialog.
      //
      tKEY_EVENT.consume();
      tPopupDialog.setVisible (false);
      tTextArea.setVisible    (true);
      fireEditingCanceled();
    }
    else if ( ( tKEY_EVENT.isControlDown() )                   &&
              ( tKEY_EVENT.getKeyCode() == KeyEvent.VK_ENTER )  )
    {
      //
      //  When the user types CTRL+ENTER,  it commits  the  editor  changes and
      //  hides the dialog.
      //
      tKEY_EVENT.consume();
      tTextArea.setText (tPopupTextArea.getText());
      tPopupDialog.setVisible (false);
      tTextArea.setVisible    (true);
      fireEditingStopped();
    }

  }  // keyPressed()

  /**
  *  Invoked when a key has been typed.
  *  @param tKEY_EVENT The key event.
  *  @see java.awt.event.KeyListener#keyTyped()
  */
  public void keyTyped (KeyEvent tKEY_EVENT) {}

  /**
  *  Invoked when a key has been released.
  *  @param tKEY_EVENT The key event.
  *  @see java.awt.event.KeyListener#keyReleased()
  */
  public void keyReleased (KeyEvent tKEY_EVENT) {}

  /**
  *  Specifies the number of clicks needed to start editing.
  *  @param iCLICK_COUNT The number of clicks needed to start editing.
  *  @see #getClickCountToStart()
  */
  public void setClickCountToStart (int iCLICK_COUNT)
  {
    iClickCountToStart = iCLICK_COUNT;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the value contained in the editor.
  *  @return The value contained in the editor.
  *  @see javax.swing.CellEditor#getCellEditorValue()
  */
  public Object getCellEditorValue()
  {
    return tTextArea.getText();
  }

  /**
  *  Returns the number of clicks needed to start editing. It defaults to 2.
  *  @return The number of clicks needed to start editing.
  */
  public int getClickCountToStart()
  {
    return iClickCountToStart;
  }

  /**
  *  Asks the editor if it can start editing using \a tEVENT_OBJECT. 
  *  @param tEVENT_OBJECT An event object.
  *  @see javax.swing.CellEditor#isCellEditable()
  */
  public boolean isCellEditable (EventObject tEVENT_OBJECT)
  {
    int       iMouseClickCount;
    boolean   gIsCellEditable = true;

    if ( tEVENT_OBJECT instanceof MouseEvent )
    {
      iMouseClickCount = ((MouseEvent) tEVENT_OBJECT).getClickCount();
      gIsCellEditable  = ( iMouseClickCount >= getClickCountToStart() );
    }
    return gIsCellEditable;
  }

  /**
  *  Sets an initial value for the  editor.  This will cause the editor to stop
  *  editing (\a stopEditing) and lose any partially edited value if the editor
  *  is editing when this method is called.  Returns the  component that should
  *  be added  to the  client's  Component hierarchy.  Once  installed  in  the
  *  client's hierarchy  this component will then  be able to draw  and receive
  *  user input. 
  *  @param tTABLE
  *         The JTable object that is asking the editor to edit; can be null.
  *  @param tVALUE
  *         The value of the cell to be edited; it is up to the specific editor
  *         to interpret and draw the value. For example, if \a tVALUE is the
  *         string "true", it could be rendered as a string or it could be
  *         rendered as a check-box that is checked. null is a valid value.
  *  @param gIS_SELECTED
  *         True if the cell is to be rendered with highlighting.
  *  @param iROW
  *         The row of the cell being edited.
  *  @param iCOLUMN
  *         The column of the cell being edited.
  *  @return The component for editing.
  *  @see javax.swing.table.TableCellEditor#getTableCellEditorComponent()
  */
  public Component getTableCellEditorComponent ( JTable  tTABLE       ,
                                                 Object  tVALUE       ,
                                                 boolean gIS_SELECTED ,
                                                 int     iROW         ,
                                                 int     iCOLUMN      )
  {
    tTextArea.setText ((String) tVALUE);
    return tTextArea;
  }

}  // class TTextAreaCellEditor
