/*
*  Name:      TDecoratorTreeTableCellRenderer.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Tree-table cell renderer that decorates another tree-table cell renderer
*  Date:      $Date: 2004-10-01 02:01:50 +0200 (vie, 01 oct 2004) $
*  Revision:  $Revision: 10189 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import java.awt.Color;
import java.awt.Component;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import org.mpcl.nui.TTreeTable;


/**
*  Tree and  table cell renderer  that decorates  a delegate cell renderer.  It
*  implements the Decorator design pattern.
*/
public class TDecoratorTreeTableCellRenderer implements ITreeTableCellRenderer
{

  ///  Delegate table cell renderer.
  private ITreeTableCellRenderer   tDelegateTreeTableCellRenderer;

  ///  Table cell decorator.
  private ITreeTableCellDecorator   tTreeTableCellDecorator;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tDELEGATE_TREE_TABLE_CELL_RENDERER Delegate tree table cell renderer.
  *  @param tTREE_TABLE_CELL_DECORATOR         Tree table cell decorator.
  */
  public TDecoratorTreeTableCellRenderer
  (
    ITreeTableCellRenderer  tDELEGATE_TREE_TABLE_CELL_RENDERER ,
    ITreeTableCellDecorator tTREE_TABLE_CELL_DECORATOR
  )
  {
    tDelegateTreeTableCellRenderer = tDELEGATE_TREE_TABLE_CELL_RENDERER;
    tTreeTableCellDecorator        = tTREE_TABLE_CELL_DECORATOR;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the  component  used for  drawing  the cell got from the  delegate
  *  table cell renderer, and modified by its decorator.
  *  @param tTABLE       Table where the cell belongs to.
  *  @param tVALUE       Value contained in the cell.
  *  @param gIS_SELECTED True if the cell is selected.
  *  @param gHAS_FOCUS   True if the cell has the focus.
  *  @param iROW         Row of the cell.
  *  @param iCOLUMN      Column of the cell.
  *  @return The \a Component that the renderer uses to draw the value.
  */
  public Component getTableCellRendererComponent ( JTable  tTABLE       ,
                                                   Object  tVALUE       ,
                                                   boolean gIS_SELECTED ,
                                                   boolean gHAS_FOCUS   ,
                                                   int     iROW         ,
                                                   int     iCOLUMN      )
  {

    Component   tResultComponent;

    tResultComponent =
      tDelegateTreeTableCellRenderer.getTableCellRendererComponent ( tTABLE       ,
                                                                     tVALUE       ,
                                                                     gIS_SELECTED ,
                                                                     gHAS_FOCUS   ,
                                                                     iROW         ,
                                                                     iCOLUMN      );
    tResultComponent =
      tTreeTableCellDecorator.getTableCellComponent ( tResultComponent ,
                                                      tTABLE           ,
                                                      tVALUE           ,
                                                      gIS_SELECTED     ,
                                                      gHAS_FOCUS       ,
                                                      iROW             ,
                                                      iCOLUMN          );
    return tResultComponent;

  }  // getTableCellRendererComponent()

  /**
  *  Returns the  component  used for  drawing  the cell got from the  delegate
  *  tree-table cell renderer, and modified by its decorator.
  *  @param tTREE        Tree where the cell belongs to.
  *  @param tVALUE       Value contained in the cell.
  *  @param gIS_SELECTED True if the cell is selected.
  *  @param gIS_EXPANDED True if the node is expanded.
  *  @param gIS_LEAF     True if the node represents a leaf.
  *  @param iROW         Row of the cell.
  *  @param gHAS_FOCUS   True if the cell has the focus.
  *  @return The \a Component that the renderer uses to draw the value.
  */
  public Component getTreeCellRendererComponent ( JTree   tTREE        ,
                                                  Object  tVALUE       ,
                                                  boolean gIS_SELECTED ,
                                                  boolean gIS_EXPANDED ,
                                                  boolean gIS_LEAF     ,
                                                  int     iROW         ,
                                                  boolean gHAS_FOCUS   )
  {

    Component   tResultComponent;

    tResultComponent =
      tDelegateTreeTableCellRenderer.getTreeCellRendererComponent ( tTREE        ,
                                                                    tVALUE       ,
                                                                    gIS_SELECTED ,
                                                                    gIS_EXPANDED ,
                                                                    gIS_LEAF     ,
                                                                    iROW         ,
                                                                    gHAS_FOCUS   );
    tResultComponent =
      tTreeTableCellDecorator.getTreeCellRendererComponent ( tResultComponent ,
                                                             tTREE            ,
                                                             tVALUE           ,
                                                             gIS_SELECTED     ,
                                                             gIS_EXPANDED     ,
                                                             gIS_LEAF         ,
                                                             iROW             ,
                                                             gHAS_FOCUS       );
    return tResultComponent;

  }  // getTreeCellRendererComponent()

}  // class TDecoratorTreeTableCellRenderer
