/*
*  Name:        TTree.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Tree used to render tree columns.
*  Date:        $Date: 2004-08-04 03:41:54 +0200 (mié, 04 ago 2004) $
*  Revision:    $Revision: 9855 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.util.Enumeration;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.table.TableCellRenderer;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import org.mpcl.nui.TTreeTable;


///  A \a JTree used for displaying the tree in the table cells.
public class TTree extends JTree
{

  ///  Last table/tree row asked to renderer.
  protected int iVisibleRow;

  ///  The tree-table.
  protected TTreeTable   tTreeTable;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tTREE_TABLE The tree-table.
  *  @param tTREE_MODEL The tree model.
  *  @pre ( tTREE_TABLE != null )
  */
  public TTree (TTreeTable tTREE_TABLE, ITreeTableModel tTREE_MODEL)
  {
    super (tTREE_MODEL);
    tTreeTable = tTREE_TABLE;
    setCellRenderer (new TTreeTableCellRenderer (tTREE_TABLE));
  }

  /**
  *  Collapses all nodes in the tree path.
  *  @param tPARENT_TREE_PATH Path that has to be collapseed.
  */
  public void collapseAll (TreePath tPARENT_TREE_PATH)
  {

    Enumeration   I;
    TreeNode      tChildNode;
    TreePath      tChildPath;
    TreeNode      tLastNode = (TreeNode) tPARENT_TREE_PATH.getLastPathComponent();

    if ( tLastNode.getChildCount() > 0 )
    {
      //
      //  Collapses recursively all child nodes.
      //
      I = tLastNode.children();
      while ( I.hasMoreElements() )
      {
        tChildNode = (TreeNode) I.nextElement();
        tChildPath = tPARENT_TREE_PATH.pathByAddingChild (tChildNode);
        collapseAll (tChildPath);
      }
    }

    //
    //  Collapse must be done bottom-up.
    //
    collapsePath (tPARENT_TREE_PATH);

  }  // collapseAll()

  /**
  *  Expands all nodes in the tree path.
  *  @param tPARENT_TREE_PATH Path that has to be expanded.
  */
  public void expandAll (TreePath tPARENT_TREE_PATH)
  {

    Enumeration   I;
    TreeNode      tChildNode;
    TreePath      tChildPath;
    TreeNode      tLastNode = (TreeNode) tPARENT_TREE_PATH.getLastPathComponent();

    if ( tLastNode.getChildCount() > 0 )
    {
      //
      //  Expands recursively all child nodes.
      //
      I = tLastNode.children();
      while ( I.hasMoreElements() )
      {
        tChildNode = (TreeNode) I.nextElement();
        tChildPath = tPARENT_TREE_PATH.pathByAddingChild (tChildNode);
        expandAll (tChildPath);
      }
    }

    //
    //  Expansion must be done bottom-up.
    //
    expandPath (tPARENT_TREE_PATH);

  }  // expandAll()

  /**
  *  Moves and resizes this component.  The new location of the top-left corner
  *  is  specified  by \a iX  and \a iY,  and  the  new  size is  specified  by
  *  \a iWIDTH  and \a iHEIGHT.  This is overridden to set the height  to match
  *  that of the \a JTable.
  *  iX      The new x-coordinate of this component.
  *  iY      The new y-coordinate of this component.
  *  iWIDTH  The new width of this component.
  *  iHEIGHT The new height of this component.
  */
  public void setBounds (int iX, int iY, int iWIDTH, int iHEIGHT)
  {
    super.setBounds (iX, 0, iWIDTH, tTreeTable.getHeight());
  }

  /**
  *  Notification from the \a UIManager that the L&F has changed.  Replaces the
  *  current  UI object with the  latest version  from the \a UIManager.  It is
  *  overridden  to set the colors of the Tree's renderer  to match that of the
  *  table.
  */
  public void updateUI()
  {
    super.updateUI();

    //
    //  Make the tree's cell renderer use the table's cell selection colors.
    //

    DefaultTreeCellRenderer   tDefaultTreeCellRenderer; 
    TreeCellRenderer          tTreeCellRenderer = getCellRenderer();

    if ( tTreeCellRenderer instanceof DefaultTreeCellRenderer )
    {
      tDefaultTreeCellRenderer = (DefaultTreeCellRenderer) tTreeCellRenderer;
      tDefaultTreeCellRenderer.setTextSelectionColor (UIManager.getColor ("Table.selectionForeground"));
      tDefaultTreeCellRenderer.setBackgroundSelectionColor (UIManager.getColor ("Table.selectionBackground"));
    }
  }

  /**
  *  Sets the last visible row for the tree renderer.
  *  @param iROW The tree last visible row.
  */
  public void setLastVisibleRow (int iROW)
  {
    iVisibleRow = iROW;
  }

  /**
  *  Sets the row height of the tree, and forwards the row height to the table.
  *  @param iROW_HEIGHT Row height.
  */
  public void setRowHeight (int iROW_HEIGHT)
  {
    if ( iROW_HEIGHT > 0 )
    {
      super.setRowHeight (iROW_HEIGHT);

      //
      //  Forwards the row height  to the tree-table only  if the tree-table is
      //  constructed  (called after the 'TTreeTable' constructor has finished)
      //  and the both row heights are not equal.
      //
      if ( ( tTreeTable                != null )        &&
           ( tTreeTable.getRowHeight() != iROW_HEIGHT )  )
      {
        tTreeTable.setRowHeight (getRowHeight()); 
      }
    }
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns true if the last node in the path is editable.
  *  @param tTREE_PATH The tree-path object.
  *  @see javax.swing.JTree#isPathEditable()
  *  @see org.mpcl.nui.treetable.ITreeTableModel#isCellEditable()
  */
  public boolean isPathEditable (TreePath tTREE_PATH)
  {
    ITreeTableModel   tTreeTableModel;

    tTreeTableModel = (ITreeTableModel) getModel();
    return tTreeTableModel.isPathEditable (tTREE_PATH);
  }

  /**
  *  Paints this component.  It has been  subclassed to  translate the graphics
  *  such that the last visible row will be drawn at 0,0.
  *  @param tGRAPHICS The graphics context to use for painting.
  */
  public void paint (Graphics tGRAPHICS)
  {
    tGRAPHICS.translate (0, -iVisibleRow * getRowHeight());
    super.paint (tGRAPHICS);
  }

}  // class TTree
