/*
*  Name:        TTreeSelectionModel.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Tree selection model.
*  Date:        $Date: 2003-11-27 12:04:28 +0100 (jue, 27 nov 2003) $
*  Revision:    $Revision: 8535 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreePath;


/**
*  This class extends \a DefaultTreeSelectionModel to listen for changes in the
*  \a ListSelectionModel    it    maintains.     Once   a    change   in    the
*  \a ListSelectionModel   happens,     the   paths    are   updated   in   the
*  \a DefaultTreeSelectionModel.
*/
public class TTreeSelectionModel extends DefaultTreeSelectionModel
{

  /**
  *  Class  responsible  for  calling  \a updateSelectedPathsFromSelectedRows()
  *  when the selection of the list changes.
  */
  protected class TListSelectionListener implements ListSelectionListener
  {
    /**
    *  Called whenever the value of the selection changes.
    *  @param tLIST_SELECTION_EVENT The event that characterizes the change.
    *  @see javax.swing.event.ListSelectionListener#valueChanged()
    */
    public void valueChanged (ListSelectionEvent tLIST_SELECTION_EVENT)
    {
      updateSelectedPathsFromSelectedRows();
    }
  }

  ///  True when it is updating the \a ListSelectionModel.
  protected boolean   gIsUpdatingListSelectionModel;

  ///  The tree.
  protected TTree   tTree;


  //
  //  C O N S T R U C T O R S
  //

  ///  Builds a new instance.
  public TTreeSelectionModel (TTree tTREE)
  {
    super();
    tTree = tTREE;
    getListSelectionModel().addListSelectionListener (createListSelectionListener());
  }

  /**
  *  This is  overridden to set  \a gIsUpdatingListSelectionModel  and  message
  *  super.  This  is the only  place \a DefaultTreeSelectionModel  alters  the
  *  \a ListSelectionModel.
  */
  public void resetRowSelection()
  {

    //
    //  Notice how it does not  message super if  gIsUpdatingListSelectionModel
    //  is true.  If  gIsUpdatingListSelectionModel  is true,   it implies  the
    //  ListSelectionModel has already been  updated and the paths are the only
    //  thing that needs to be updated.
    //
    if ( !gIsUpdatingListSelectionModel )
    {
      gIsUpdatingListSelectionModel = true;
      try
      {
        super.resetRowSelection();
      }
      finally
      {
        gIsUpdatingListSelectionModel = false;
      }
    }

  }  // resetRowSelection()

  /**
  *  If \a gIsUpdatingListSelectionModel is false, this will reset the selected
  *  paths from the selected rows in the list selection model.
  */
  protected void updateSelectedPathsFromSelectedRows()
  {

    if ( !gIsUpdatingListSelectionModel )
    {
      gIsUpdatingListSelectionModel = true;
      try
      {
        //
        //  This is way expensive, ListSelectionModel needs an enumerator
        //  for iterating.
        //

        TreePath   tSelectedTreePath;
        int        iMax = listSelectionModel.getMaxSelectionIndex();
        int        iMin = listSelectionModel.getMinSelectionIndex();

        clearSelection();
        if ( ( iMin != -1 ) && ( iMax != -1 ) )
        {
          for (int I = iMin; ( I <= iMax ) ;++I)
          {
            if ( listSelectionModel.isSelectedIndex (I) )
            {
              tSelectedTreePath = tTree.getPathForRow (I);
              if ( tSelectedTreePath != null )
              {
                addSelectionPath (tSelectedTreePath);
              }
            }
          }
        }
      }
      finally
      {
        gIsUpdatingListSelectionModel = false;
      }
    }

  }  // updateSelectedPathsFromSelectedRows()


  //
  //  S E L E C T O R S
  //

  /**
  *  Creates and returns an instance of TListSelectionListener.
  *  @return A new \a TListSelectionListener.
  */
  protected ListSelectionListener createListSelectionListener()
  {
    return new TListSelectionListener();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Returns the list selection  model.  \a TTreeSelectionModel
  *  listens  for changes to  this  model and  updates the  selected  paths
  *  accordingly.
  *  @return The list selection model.
  */
  public ListSelectionModel getListSelectionModel()
  {
    return listSelectionModel;
  }

}  // class TTreeSelectionModel
