/*
*  Name:        TTreeTableCellEditor.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Tree-table cell editor.
*  Date:        $Date: 2004-08-04 03:41:54 +0200 (mié, 04 ago 2004) $
*  Revision:    $Revision: 9855 $
*
*  Copyright (C) 2003  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import javax.swing.DefaultCellEditor;
import javax.swing.Icon;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.TableCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;
import org.mpcl.nui.TTreeTable;


/**
*  An editor that can be used to edit the tree column. This extends
*  \a DefaultCellEditor and uses a \a JTextField (actually, \a TTextField) to
*  perform the actual editing. To support editing of the tree column we can not
*  make the tree editable. The reason this doesn't work is that you can not use
*  the same component for editing and renderering.  The table may have the need
*  to paint cells,  while a cell  is being edited.  If the same  component were
*  used for the rendering and editing the component would be moved around,  and
*  the contents would change.  When editing,  this is undesirable, the contents
*  of the  text field must stay  the same,  including the caret  blinking,  and
*  selections  persisting.   For  this  reason  the   editing  is  done  via  a
*  \a TableCellEditor.
*  Another  interesting thing to be aware  of is how  tree positions its render
*  and editor. The render/editor is responsible for drawing the icon indicating
*  the type of node (leaf, branch...).  The tree is responsible for drawing any
*  other indicators,  perhaps an additional +/- sign,  or lines connecting  the
*  various nodes.  So, the renderer is positioned based on depth.  On the other
*  hand, table always makes  its editor fill the  contents of the cell.  To get
*  the allusion that the table cell editor is part of the tree,  we do not want
*  the table cell  editor to fill  the cell bounds.  We want it to be placed in
*  the same manner as tree places it editor, and have table message the tree to
*  paint any decorations  the tree wants.  Then,  we would  only have to  worry
*  about the editing part.  The approach taken here is to determine  where tree
*  would  place the editor,  and to  override  the \a reshape()  method in  the
*  \a JTextField  component to nudge the text field to the location  tree would
*  place  it.  Since  \a JTreeTable  will  paint  the tree  behind  the  editor
*  everything should just work. So, that is what we are doing here. Determining
*  of the  icon  position  will only  work  if  the \a TreeCellRenderer  is  an
*  instance    of    \a DefaultTreeCellRenderer.    If    you    need    custom
*  \a TreeCellRenderers,  that do not descend  from \a DefaultTreeCellRenderer,
*  and you  want to  support  editing  in \a JTreeTable,  you will  have to  do
*  something similiar.
*/
public class TTreeTableCellEditor extends DefaultCellEditor
{

  ///  Tree-table.
  protected TTreeTable   tTreeTable;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tTREE_TABLE The tree-table.
  */
  public TTreeTableCellEditor (TTreeTable tTREE_TABLE)
  {
    super (new TTextField());
    tTreeTable = tTREE_TABLE;
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Overridden to determine an offset that tree would place the editor at. The
  *  offset  is  determined  from  the \a getRowBounds() \a JTree  method,  and
  *  additionally  from the icon that \a DefaultTreeCellRenderer  will use. The
  *  offset  is  then  set  on  the  \a TTextField  component  created  in  the
  *  constructor, and returned.
  *  @param tTABLE
  *         The \a JTable that is asking the editor to edit; can be null.
  *  @param tVALUE
  *         The value of the cell to be edited; it is up to the specific editor
  *         to  interpret and  draw  the value.  For example,  if value  is the
  *         string "true",  it  could  be rendered as a  string or it  could be
  *         rendered as a check box that is checked. Can be null.
  *  @param gIS_SELECTED
  *         True if the cell is to be rendered with highlighting.
  *  @param iROW
  *         The row of the cell being edited.
  *  @param iCOLUMN
  *         The column of the cell being edited.
  */
  public Component getTableCellEditorComponent ( JTable  tTABLE       ,
                                                 Object  tVALUE       ,
                                                 boolean gIS_SELECTED ,
                                                 int     iROW         ,
                                                 int     iCOLUMN      )
  {

    int                iOffset;
    Rectangle          tBoundsRectangle;
    Component          tComponent;
    JTree              tTree;
    TreeCellRenderer   tTreeCellRenderer;

    tComponent        = super.getTableCellEditorComponent (tTABLE, tVALUE, gIS_SELECTED, iROW, iCOLUMN);
    tTree             = tTreeTable.getTree();
    tBoundsRectangle  = tTree.getRowBounds (iROW);
    iOffset           = tBoundsRectangle.x;
    tTreeCellRenderer = tTree.getCellRenderer();
    if ( tTreeCellRenderer instanceof DefaultTreeCellRenderer )
    {
      Icon     tIcon;
      Object   tNode = tTree.getPathForRow (iROW).getLastPathComponent();

      if ( tTree.getModel().isLeaf (tNode) )
      {
        tIcon = ((DefaultTreeCellRenderer) tTreeCellRenderer).getLeafIcon();
      }
      else if ( tTree.isExpanded (iROW) )
      {
        tIcon = ((DefaultTreeCellRenderer) tTreeCellRenderer).getOpenIcon();
      }
      else
      {
        tIcon = ((DefaultTreeCellRenderer) tTreeCellRenderer).getClosedIcon();
      }
      if ( tIcon != null )
      {
        iOffset += ((DefaultTreeCellRenderer) tTreeCellRenderer).getIconTextGap();
        iOffset += tIcon.getIconWidth();
      }
    }
    ((TTextField) getComponent()).setOffset (iOffset);
    return tComponent;

  }  // getTableCellEditorComponent()

  /**
  *  This is overridden to forward  the event to  the tree.  This will check if
  *  the path is editable when  the click count greater than or equal to 3,  or
  *  the event is null.
  *  @param tEVENT_OBJECT
  *         The event that the editor should  use to consider  whether to begin
  *         editing or not.
  */
  public boolean isCellEditable (EventObject tEVENT_OBJECT)
  {

    TreePath   tTreePath;
    boolean    gIsEditable = false;
    TTree      tTree       = tTreeTable.getTree();

    //
    //  Checks if the user has hit the F2 key.
    //
    if ( tEVENT_OBJECT == null )
    {
      tTreePath = tTree.getSelectionPath();
      if ( tTreePath != null )
      {
        gIsEditable = tTree.isPathEditable (tTreePath);
      }
    }
    else if (tEVENT_OBJECT instanceof MouseEvent)
    {
      MouseEvent   tNewMouseEvent;
      MouseEvent   tMouseEvent = (MouseEvent) tEVENT_OBJECT;

      //
      //  If the modifiers are not 0  (or the left mouse button),  the tree may
      //  try and toggle the selection, and the table will then try and toggle,
      //  resulting in the  selection  remaining the  same.  To avoid this,  it
      //  only dispatches when the modifiers are 0  (or the left mouse button).
      //
      if ( ( tMouseEvent.getModifiers() == 0 )                       ||
           ( tMouseEvent.getModifiers() == InputEvent.BUTTON1_MASK )  )
      {
        for (int I = tTreeTable.getColumnCount() - 1; ( I >= 0 ) ;--I)
        {
          if ( tTreeTable.getColumnClass (I) == ITreeTableModel.class )
          {
            tNewMouseEvent =
              new MouseEvent ( tTree                                                      ,
                               tMouseEvent.getID()                                        ,
                               tMouseEvent.getWhen()                                      ,
                               tMouseEvent.getModifiers()                                 ,
                               tMouseEvent.getX() - tTreeTable.getCellRect (0, I, true).x ,
                               tMouseEvent.getY()                                         ,
                               tMouseEvent.getClickCount()                                ,
                               tMouseEvent.isPopupTrigger()                               );
            tTree.dispatchEvent (tNewMouseEvent);
            break;
          }
        }
      }
      if ( tMouseEvent.getClickCount() >= 3 )
      {
        gIsEditable = false;
        tTreePath   = tTree.getClosestPathForLocation (tMouseEvent.getX(), tMouseEvent.getY());
        if ( tTreePath != null )
        {
          gIsEditable = tTree.isPathEditable (tTreePath);
        }
      }
    }
    return gIsEditable;

  }  // isCellEditable()

}  // class TTreeTableCellEditor
