/*
*  Name:        TTreeTableModelToTableModelAdapter.java
*  Author:      Philip Milne
*               Scott Violet
*  Maintainer:  Rafael Jesus Alcantara Perez
*  Summary:     Tree-table model to table model adapter.
*  Date:        $Date: 2004-05-10 19:21:28 +0200 (lun, 10 may 2004) $
*  Revision:    $Revision: 9351 $
*
*  Copyright (C) 2003-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*
*
*  Copyright 1997-2000 Sun Microsystems, Inc. All Rights Reserved.
*
*  Redistribution and use in source and binary forms, with or
*  without modification, are permitted provided that the following
*  conditions are met:
*
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*
*  - Redistribution in binary form must reproduce the above
*    copyright notice, this list of conditions and the following
*    disclaimer in the documentation and/or other materials
*    provided with the distribution.
*
*  Neither the name of Sun Microsystems, Inc. or the names of
*  contributors may be used to endorse or promote products derived
*  from this software without specific prior written permission.
*
*  This software is provided "AS IS," without a warranty of any
*  kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
*  WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
*  EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
*  DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT OF OR
*  RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THIS SOFTWARE OR
*  ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE
*  FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT,
*  SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
*  CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF
*  THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF SUN HAS
*  BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
*
*  You acknowledge that this software is not designed, licensed or
*  intended for use in the design, construction, operation or
*  maintenance of any nuclear facility.
*/

///  Package name for TREETABLE (tree-table support classes).
package org.mpcl.nui.treetable;

import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;


/**
*  This is an adapter class that takes an \a ITreeTableModel and implements the
*  \a TableModel  interface.  The implementation  is trivial,  with  all of the
*  event dispatching support provided by the  superclass \a AbstractTableModel. 
*/
public class TTreeTableModelToTableModelAdapter extends AbstractTableModel
{

  ///  The tree.
  private JTree   tTree;

  ///  The tree-table model.
  private ITreeTableModel   tTreeTableModel;


  //
  //  C O N S T R U C T O R S
  //

  /**
  *  Builds a new instance.
  *  @param tTREE_TABLE_MODEL A tree-table model.
  *  @param tTREE             A tree.
  */
  public TTreeTableModelToTableModelAdapter (ITreeTableModel tTREE_TABLE_MODEL, JTree tTREE)
  {

    tTree           = tTREE;
    tTreeTableModel = tTREE_TABLE_MODEL;
    tTree.addTreeExpansionListener
    (
      new TreeExpansionListener()
      {
        //
        //  Do not use fireTableRowsInserted() here;  the selection model would
        //  get updated twice.
        //
        public void treeExpanded (TreeExpansionEvent tTREE_EXPANSION_EVENT)
        {  
          fireTableDataChanged();
        }

        public void treeCollapsed (TreeExpansionEvent tTREE_EXPANSION_EVENT)
        {
          fireTableDataChanged(); 
        }
      }
    );

    //
    //  Installs a  TreeModelListener  that  can update  the  table  when  tree
    //  changes.  We  use  \a delayedFireTableDataChanged()  as we  can  not be
    //  guaranteed the tree will have finished  processing the event before us.
    //
    tTreeTableModel.addTreeModelListener
    (
      new TreeModelListener()
      {
        public void treeNodesChanged (TreeModelEvent tTREE_MODEL_EVENT)
        {
          delayedFireTableDataChanged();
        }

        public void treeNodesInserted (TreeModelEvent tTREE_MODEL_EVENT)
        {
          delayedFireTableDataChanged();
        }

        public void treeNodesRemoved (TreeModelEvent tTREE_MODEL_EVENT)
        {
          delayedFireTableDataChanged();
        }

        public void treeStructureChanged (TreeModelEvent tTREE_MODEL_EVENT)
        {
          delayedFireTableDataChanged();
        }
      }
    );

  }  // TTreeTableModelToTableModelAdapter()

  public void setValueAt (Object tVALUE, int iROW, int iCOLUMN)
  {
    tTreeTableModel.setValueAt (tVALUE, getNodeForRow (iROW), iROW, iCOLUMN);
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Invokes \a fireTableDataChanged  after all  the pending  events have  been
  *  processed. \a SwingUtilities.invokeLater is used to handle this.
  */
  protected void delayedFireTableDataChanged()
  {
// <FIXME comment="can we remove the memory allocation?">
    SwingUtilities.invokeLater
    (
      new Runnable()
      {
        public void run()
        {
          fireTableDataChanged();
        }
      }
    );
// </FIXME>
  }

  /**
  *  Returns the node at row \a iROW.
  *  @return The node.
  */
  protected Object getNodeForRow (int iROW)
  {
    return tTree.getPathForRow (iROW).getLastPathComponent();         
  }


  //
  //  S E L E C T O R S
  //

  public Class getColumnClass (int iCOLUMN)
  {
    return tTreeTableModel.getColumnClass (iCOLUMN);
  }

  public int getColumnCount()
  {
    return tTreeTableModel.getColumnCount();
  }

  public String getColumnName (int iCOLUMN)
  {
    return tTreeTableModel.getColumnName (iCOLUMN);
  }

  public int getRowCount()
  {
    return tTree.getRowCount();
  }

  public Object getValueAt (int iROW, int iCOLUMN)
  {
    return tTreeTableModel.getValueAt (getNodeForRow (iROW), iCOLUMN);
  }

  public boolean isCellEditable (int iROW, int iCOLUMN)
  {
    return tTreeTableModel.isCellEditable (getNodeForRow (iROW), iCOLUMN); 
  }

}  // class TTreeTableModelToTableModelAdapter
