/*
*  Name:      TFixedPointDecimalTranslator.java
*  Author:    Rafael Jesus Alcantara Perez
*  Summary:   Class for translating numbers to its fixed point decimal
*             string representation and vice-versa using locale.
*  Date:      $Date: 2004-09-21 18:49:46 +0200 (mar, 21 sep 2004) $
*  Revision:  $Revision: 10150 $
*
*  Copyright (C) 2001-2004  Rafael Jesus Alcantara Perez <rafa@dedaloingenieros.com>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
*  MA 02111-1307, USA.
*/

///  Package name for TEXT related classes.
package org.mpcl.text;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;


/**
*  Class  to  convert   from   numbers  to  their  string   representation  and
*  vice-versa.
*/
public class TFixedPointDecimalTranslator extends TAbstractNumberTranslator
{
  
  ///  Number format.
  DecimalFormat   tDecimalFormat;


  //
  //  C O N S T R U C T O R S
  //

  private void initialize()
  {
    tDecimalFormat.setDecimalSeparatorAlwaysShown (true);
    setFractionDigits (2);
  }

  private void setFractionDigits (int iFRACTION_DIGITS)
  {
    tDecimalFormat.setMaximumFractionDigits (iFRACTION_DIGITS);
    tDecimalFormat.setMinimumFractionDigits (iFRACTION_DIGITS);
  }


  //
  //  C O N S T R U C T O R S
  //

  ///  Builds a new instance.
  public TFixedPointDecimalTranslator()
  {
    tDecimalFormat = (DecimalFormat) NumberFormat.getInstance();
    initialize();
  }

  /**
  *  Builds a new instance.
  *  @param tLOCALE Locale.
  */
  public TFixedPointDecimalTranslator (Locale tLOCALE)
  {
    tDecimalFormat = (DecimalFormat) NumberFormat.getInstance (tLOCALE);
    initialize();
  }


  //
  //  S E L E C T O R S
  //

  /**
  *  Translates the number in \a yNUMBER to a double value.
  *  @param yNUMBER String with the locale representation of a double value.
  *  @pre yNUMBER represents a double value in the selected locale.
  *  @return A double value corresponding to the number in \a yNUMBER converted
  *          using selected locale.
  *  @see DecimalFormat#parse().
  */
  public double toDouble (String yNUMBER) throws ParseException
  {
    return tDecimalFormat.parse (yNUMBER).doubleValue();
  }

  /**
  *  Translates the number in \a yNUMBER to a float value.
  *  @param yNUMBER String with the locale representation of a float value.
  *  @pre yNUMBER represents a float value in the selected locale.
  *  @return A float value  corresponding to the number in \a yNUMBER converted
  *          using selected locale.
  *  @see DecimalFormat#parse().
  */
  public float toFloat (String yNUMBER) throws ParseException
  {
    return tDecimalFormat.parse (yNUMBER).floatValue();
  }

  /**
  *  Translates the number in \a yNUMBER to a integer value.
  *  @param yNUMBER String with the locale representation of a integer value.
  *  @pre yNUMBER represents a integer value in the selected locale.
  *  @return An  integer  value  corresponding  to  the  number  in  \a yNUMBER
  *          converted using selected locale.
  *  @see DecimalFormat#parse().
  */
  public int toInt (String yNUMBER) throws ParseException
  {
    return tDecimalFormat.parse (yNUMBER).intValue();
  }

  /**
  *  Translates  the  number \a dNUMBER  to its  string  representation  in the
  *  selected locale.
  *  @param dNUMBER Double value.
  *  @return An  string with the  representation  of the double value converted
  *          using selected locale.
  *  @see DecimalFormat#parse().
  */
  public String toString (double dNUMBER)
  {
    return tDecimalFormat.format (dNUMBER);
  }

  /**
  *  Translates  the  number \a tNUMBER  to its  string  representation  in the
  *  selected locale.
  *  @param tNUMBER Number.
  *  @return An  string with the  representation  of the number converted using
  *          selected locale.
  *  @see #TFixedPointDecimalTranslator().
  */
  public String toString (Number tNUMBER)
  {
    return tDecimalFormat.format (tNUMBER);
  }

}  // class TFixedPointDecimalTranslator
